/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  AttachmentContainerE2EComponent,
  AttachmentListE2EComponent,
} from '../../../components/attachment/attachment.e2e.component';
import { KommentarListInVorgangE2EComponent } from '../../../components/kommentar/kommentar-list.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { deleteDownloadFolder, dropCollections, readFileFromDownloads } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { TEST_FILE_WITH_CONTENT, TEST_FILE_WITHOUT_CONTENT } from '../../../support/data.util';
import { uploadFile, uploadFiles } from '../../../support/file-upload';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

describe('Kommentar attachments', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const kommentarContainer: KommentarListInVorgangE2EComponent = vorgangPage.getKommentarContainer();

  const attachmentContainer: AttachmentContainerE2EComponent = kommentarContainer.getAttachmentContainer();
  const attachmentList: AttachmentListE2EComponent = attachmentContainer.getList();

  const kommentarText: string = 'Test text to test the test text test';

  const vorgang: VorgangE2E = createVorgang();

  before(() => {
    initVorgang(vorgang);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(mainPage.getVorgangList().getRoot());
  });

  after(() => {
    dropCollections();
    deleteDownloadFolder();
  });

  describe('Upload attachements', () => {
    it('should navigate to Vorgang', () => {
      vorgangList.getListItem(vorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show button for uploading attachments when creating new Kommentar', () => {
      kommentarContainer.getHinzufuegenButton().click();
      waitForSpinnerToDisappear();

      exist(attachmentContainer.getUploadInput());
    });

    it('should upload attachements', () => {
      uploadFile(attachmentContainer.getUploadInput(), TEST_FILE_WITH_CONTENT);
      waitForSpinnerToDisappear();

      uploadFile(attachmentContainer.getUploadInput(), TEST_FILE_WITHOUT_CONTENT);
      waitForSpinnerToDisappear();

      exist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
      exist(attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
    });

    it('should show attachements after saving Kommentar', () => {
      kommentarContainer.getTextInput().type(kommentarText);
      kommentarContainer.getFormularSpeichernButton().click();
      waitForSpinnerToDisappear();

      exist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
      exist(attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
    });
  });

  describe('Delete attachements', () => {
    it('should delete attachments', () => {
      kommentarContainer.getKommentar(kommentarText).getRoot().click();
      waitForSpinnerToDisappear();

      attachmentList.getItem(TEST_FILE_WITH_CONTENT).getDeleteButton().click();
      waitForSpinnerToDisappear();

      attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getDeleteButton().click();
      waitForSpinnerToDisappear();

      notExist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
      notExist(attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
    });

    it('should not show attachments after saving Kommentar', () => {
      kommentarContainer.getFormularSpeichernButton().click();
      waitForSpinnerToDisappear();

      notExist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
      notExist(attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
    });
  });

  describe('upload multiple attachments', () => {
    it('should show files after upload is done', () => {
      kommentarContainer.getKommentar(kommentarText).getRoot().click();
      waitForSpinnerToDisappear();

      uploadFiles(attachmentContainer.getUploadInput(), [TEST_FILE_WITH_CONTENT, TEST_FILE_WITHOUT_CONTENT]);
      waitForSpinnerToDisappear();
      kommentarContainer.getFormularSpeichernButton().click();
      waitForSpinnerToDisappear();

      exist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
      exist(attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
    });
  });

  describe('Download Kommentar attachments', () => {
    it('should download uploaded attachment', () => {
      cy.wait(1000);

      attachmentList.getItem(TEST_FILE_WITH_CONTENT).getDownloadButton().click();

      exist(readFileFromDownloads(TEST_FILE_WITH_CONTENT));
    });
  });
});
