/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  createEmptyStateResource,
  createStateResource,
  EMPTY_STRING,
  InvalidParam,
  setInvalidParamValidationError,
  StateResource,
} from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import { fakeAsync, TestBed, tick } from '@angular/core/testing';
import { AbstractControl, FormArray, FormBuilder, FormControl, FormGroup, UntypedFormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, UrlSegment } from '@angular/router';
import { faker } from '@faker-js/faker/.';
import { createDummy, Dummy } from 'libs/tech-shared/test/dummy';
import { singleCold, singleColdCompleted, singleHot } from 'libs/tech-shared/test/marbles';
import { createSpy, mock, Mock } from 'libs/test-utils/src/lib/mocking';
import { Observable, of, throwError } from 'rxjs';
import { createUrlSegment } from '../../../../navigation-shared/test/navigation-test-factory';
import { KeycloakFormService, PatchConfig } from './keycloak-formservice';

import { expect } from '@jest/globals';
import { ValidationMessageCode } from '../../../../tech-shared/src/lib/validation/tech.validation.messages';
import { createInvalidParam } from '../../../../tech-shared/test/error';
import { createErrorRepresentation, createKeycloakHttpErrorResponse } from '../test/keycloak';
import * as FormUtil from './form.util';
import { ErrorRepresentation, KeycloakErrorMessage, KeycloakFieldName, KeycloakHttpErrorResponse } from './keycloak-error.model';
import { extractErrorRepresentations, isFieldErrorMessage, isSingleErrorMessage } from './keycloak-error.util';

jest.mock('./keycloak-error.util', () => ({
  ...jest.requireActual('./keycloak-error.util'),
  extractErrorRepresentations: jest.fn(),
  isSingleErrorMessage: jest.fn(),
  isFieldErrorMessage: jest.fn(),
}));

jest.mock('@alfa-client/tech-shared', () => ({
  ...jest.requireActual('@alfa-client/tech-shared'),
  setInvalidParamValidationError: jest.fn(),
}));

const setInvalidParamValidationErrorMock: jest.Mock = setInvalidParamValidationError as jest.Mock;
const extractErrorRepresentationsMock: jest.Mock = extractErrorRepresentations as jest.Mock;
const isSingleErrorMessageMock: jest.Mock = isSingleErrorMessage as jest.Mock;
const isFieldErrorMessageMock: jest.Mock = isFieldErrorMessage as jest.Mock;

describe('KeycloakFormService', () => {
  let service: KeycloakFormService<Dummy>;

  let formBuilder: FormBuilder = new FormBuilder();
  let activatedRoute: Mock<ActivatedRoute>;

  let initFormSpy: jest.SpyInstance<FormGroup>;
  let evaluateRouteSpy: jest.SpyInstance<void>;

  const formGroup: FormGroup = new FormGroup({});
  const activatedRouteSegements: UrlSegment[] = [createUrlSegment()];

  beforeEach(() => {
    activatedRoute = <any>{ ...mock(ActivatedRoute), url: of<UrlSegment[]>(activatedRouteSegements) };

    initFormSpy = createSpy(TestKeycloakFormService, '_initForm').mockReturnValue(formGroup);
    evaluateRouteSpy = createSpy(TestKeycloakFormService, '_evaluateRoute');
    extractErrorRepresentationsMock.mockClear();
    isSingleErrorMessageMock.mockClear();
    isFieldErrorMessageMock.mockClear();
    setInvalidParamValidationErrorMock.mockClear();

    TestBed.configureTestingModule({
      providers: [
        TestKeycloakFormService,
        { provide: FormBuilder, useValue: formBuilder },
        { provide: ActivatedRoute, useValue: activatedRoute },
      ],
    });

    service = TestBed.inject(TestKeycloakFormService);
  });

  it('should create', () => {
    expect(service).toBeTruthy();
  });

  describe('after creation', () => {
    describe('form', () => {
      it('should be initialized', () => {
        expect(initFormSpy).toHaveBeenCalled();
      });

      it('should be set', () => {
        expect(service.form).toBe(formGroup);
      });
    });

    it('should evaluate route', () => {
      expect(evaluateRouteSpy).toHaveBeenCalled();
    });
  });

  describe('evaluate route', () => {
    const patchConfig: PatchConfig = { id: 'dummyId', doPatch: true };

    beforeEach(() => {
      service._patchConfig = undefined;
      service._buildPatchConfig = jest.fn().mockReturnValue(patchConfig);
    });

    it('should build patch config by url segments form activated route', () => {
      service._evaluateRoute();

      expect(service._buildPatchConfig).toHaveBeenCalledWith(activatedRouteSegements);
    });

    it('should set patchConfig', () => {
      service._evaluateRoute();

      expect(service._patchConfig).toBe(patchConfig);
    });
  });

  describe('get', () => {
    describe('if patch is necessarry', () => {
      const loadedStateResource: StateResource<Dummy> = createStateResource(createDummy());

      beforeEach(() => {
        service._initLoading = jest.fn().mockReturnValue(singleCold(loadedStateResource));
        service._patchConfig = { doPatch: true };
      });

      it('should init loading', () => {
        service.get();

        expect(service._initLoading).toHaveBeenCalled();
      });

      it('should return loaded resource', () => {
        const loadedResource: Observable<StateResource<Dummy>> = service.get();

        expect(loadedResource).toBeObservable(singleCold(loadedStateResource));
      });
    });

    it('should return empty state resource if patch is NOT necessarry', (done) => {
      service._patchConfig = { doPatch: false };

      service.get().subscribe((stateResource: StateResource<Dummy>) => {
        expect(stateResource).toEqual(createEmptyStateResource());
        done();
      });
    });
  });

  describe('init loading', () => {
    const patchConfig: PatchConfig = { id: 'dummyId', doPatch: true };
    const dummyStateResource: StateResource<unknown> = createStateResource(createDummy());

    beforeEach(() => {
      service._patchConfig = patchConfig;
      service._load = jest.fn().mockReturnValue(singleHot(dummyStateResource));
      service._patchIfLoaded = jest.fn();
      service._doAfterPatch = jest.fn();
    });

    it('should call load', () => {
      service._initLoading();

      expect(service._load).toHaveBeenCalledWith('dummyId');
    });

    it('should call patch if loaded', () => {
      service._load = jest.fn().mockReturnValue(of(dummyStateResource));

      service._initLoading().subscribe();

      expect(service._patchIfLoaded).toHaveBeenCalledWith(dummyStateResource);
    });

    it('should call do after patch', () => {
      service._load = jest.fn().mockReturnValue(of(dummyStateResource));

      service._initLoading().subscribe();

      expect(service._doAfterPatch).toHaveBeenCalledWith(dummyStateResource);
    });

    it('should return loaded value', () => {
      const loadedDummyStateResource: Observable<StateResource<Dummy>> = service._initLoading();

      expect(loadedDummyStateResource).toBeObservable(singleCold(dummyStateResource));
    });
  });

  describe('patch if loaded', () => {
    const dummy: Dummy = createDummy();

    it('should call patch on loaded state resource', () => {
      service._patch = jest.fn();

      service._patchIfLoaded(createStateResource(dummy));

      expect(service._patch).toHaveBeenCalledWith(dummy);
    });
  });

  describe('submit', () => {
    const dummyStateResource: StateResource<Dummy> = createStateResource(createDummy());

    beforeEach(() => {
      service._doSubmit = jest.fn().mockReturnValue(singleHot(dummyStateResource));
      service._processInvalidForm = jest.fn().mockReturnValue(of(createEmptyStateResource()));
      service._processResponseValidationErrors = jest.fn().mockReturnValue(of(createEmptyStateResource()));
      service.form.setErrors(null);
    });

    describe('on client validation error', () => {
      it('should process invalid form', () => {
        service.form.setErrors({ dummy: 'dummy error' });

        service.submit().subscribe();

        expect(service._processInvalidForm).toHaveBeenCalled();
      });

      it('should return invalid form processing result on invalid form', () => {
        service.form.setErrors({ dummy: 'dummy error' });

        expect(service.submit()).toBeObservable(singleColdCompleted(createEmptyStateResource()));
      });
    });

    it('should call do submit', () => {
      service.submit().subscribe();

      expect(service._doSubmit).toHaveBeenCalled();
    });

    it('should return submit response', () => {
      const submitResponse: Observable<StateResource<Dummy>> = service.submit();

      expect(submitResponse).toBeObservable(singleCold(dummyStateResource));
    });

    describe('on server validation error', () => {
      const keycloakHttpError: KeycloakHttpErrorResponse = createKeycloakHttpErrorResponse();

      beforeEach(() => {
        service._doSubmit = jest.fn().mockReturnValue(throwError(() => keycloakHttpError));
      });

      it('should process response validation errors', () => {
        service.submit().subscribe();

        expect(service._processResponseValidationErrors).toHaveBeenCalledWith(keycloakHttpError);
      });

      it('should return processing response validation errors result', () => {
        expect(service.submit()).toBeObservable(singleColdCompleted(createEmptyStateResource()));
      });
    });
  });

  describe('process invalid form', () => {
    beforeEach(() => {
      service._showValidationErrorForAllInvalidControls = jest.fn();
    });

    it('should show validation errors on all invalid controls', () => {
      service._processInvalidForm();

      expect(service._showValidationErrorForAllInvalidControls).toHaveBeenCalledWith(service.form);
    });

    it('should return delayed empty state resource', fakeAsync(() => {
      const expectedEmits: StateResource<unknown>[] = [];

      service._processInvalidForm().subscribe((value: StateResource<unknown>) => {
        expectedEmits.push(value);
      });
      tick(200);

      expect(expectedEmits).toEqual([createEmptyStateResource(true), createEmptyStateResource()]);
    }));
  });

  describe('process response validation errors', () => {
    const keycloakHttpError: KeycloakHttpErrorResponse = createKeycloakHttpErrorResponse();

    beforeEach(() => {
      service._handleUnknownKeycloakError = jest.fn();
      service._setValidationErrorsOnControls = jest.fn();
      service._mapKeycloakErrorRepresentationsToInvalidParams = jest.fn();
    });

    it('should set validation errors on controls', () => {
      const invalidParams: InvalidParam[] = [createInvalidParam()];
      service._mapKeycloakErrorRepresentationsToInvalidParams = jest.fn().mockReturnValue(invalidParams);

      service._processResponseValidationErrors(keycloakHttpError);

      expect(service._setValidationErrorsOnControls).toHaveBeenCalledWith(invalidParams);
    });

    it('should map keycloak error representations to invalid params', () => {
      const errorRepresentation: ErrorRepresentation = createErrorRepresentation();
      extractErrorRepresentationsMock.mockReturnValue([errorRepresentation]);

      service._processResponseValidationErrors(keycloakHttpError);

      expect(service._mapKeycloakErrorRepresentationsToInvalidParams).toHaveBeenCalledWith([errorRepresentation]);
    });

    it('should extract error representations', () => {
      service._processResponseValidationErrors(keycloakHttpError);

      expect(extractErrorRepresentationsMock).toHaveBeenCalledWith(keycloakHttpError);
    });

    it('should emit empty state resource', () => {
      expect(service._processResponseValidationErrors(keycloakHttpError)).toBeObservable(
        singleColdCompleted(createEmptyStateResource()),
      );
    });

    it('should handle unknown keycloak error', () => {
      const error: Error = new Error('Fehler');
      extractErrorRepresentationsMock.mockImplementation(() => {
        throw error;
      });

      service._processResponseValidationErrors(keycloakHttpError);

      expect(service._handleUnknownKeycloakError).toHaveBeenCalledWith(keycloakHttpError);
    });

    it('should emit empty state resource exception', () => {
      const error: Error = new Error('Fehler');
      extractErrorRepresentationsMock.mockImplementation(() => {
        throw error;
      });

      expect(service._processResponseValidationErrors(keycloakHttpError)).toBeObservable(
        singleColdCompleted(createEmptyStateResource()),
      );
    });
  });

  describe('show validation errors on all invalid controls', () => {
    it('should update value and validity on invalid control', () => {
      const control: AbstractControl = new FormControl();
      control.setErrors({ dummy: 'error' });
      const spy: jest.SpyInstance = jest.spyOn(control, 'updateValueAndValidity');

      service._showValidationErrorForAllInvalidControls(control);

      expect(spy).toHaveBeenCalled();
    });

    it('should update value and validity form invalid control from group', () => {
      const control: AbstractControl = new FormControl();
      control.setErrors({ dummy: 'error' });
      const spy: jest.SpyInstance = jest.spyOn(service, '_showValidationErrorForAllInvalidControls');
      const group: UntypedFormGroup = new FormGroup({
        someControl: control,
      });

      service._showValidationErrorForAllInvalidControls(group);

      expect(spy).toHaveBeenCalledWith(control);
    });

    it('should update value and validity on invalid control from group', () => {
      const control: AbstractControl = new FormControl();
      control.setErrors({ dummy: 'error' });
      const spy: jest.SpyInstance = jest.spyOn(control, 'updateValueAndValidity');
      const group: UntypedFormGroup = new FormGroup({
        someControl: control,
      });

      service._showValidationErrorForAllInvalidControls(group);

      expect(spy).toHaveBeenCalled();
    });

    it('should update value and validity form invalid control from array', () => {
      const control: AbstractControl = new FormControl();
      control.setErrors({ dummy: 'error' });
      const spy: jest.SpyInstance = jest.spyOn(service, '_showValidationErrorForAllInvalidControls');
      const array: FormArray = new FormArray([control]);

      service._showValidationErrorForAllInvalidControls(array);

      expect(spy).toHaveBeenCalledWith(control);
    });

    it('should update value and validity on invalid control from group', () => {
      const control: AbstractControl = new FormControl();
      control.setErrors({ dummy: 'error' });
      const spy: jest.SpyInstance = jest.spyOn(control, 'updateValueAndValidity');
      const array: FormArray = new FormArray([control]);

      service._showValidationErrorForAllInvalidControls(array);

      expect(spy).toHaveBeenCalled();
    });
  });

  describe('set validation errors on controls', () => {
    it('should set invalid param validation error', () => {
      const invalidParam: InvalidParam = createInvalidParam();

      service._setValidationErrorsOnControls([invalidParam]);

      expect(setInvalidParamValidationErrorMock).toHaveBeenCalledWith(service.form, invalidParam);
    });
  });

  describe('map keycloak error representations to invalid params', () => {
    it('should map', () => {
      service._mapKeycloakErrorRepresentationToInvalidParam = jest.fn();
      const errorRepresentation: ErrorRepresentation = createErrorRepresentation();

      service._mapKeycloakErrorRepresentationsToInvalidParams([errorRepresentation]);

      expect(service._mapKeycloakErrorRepresentationToInvalidParam).toHaveBeenCalledWith(errorRepresentation);
    });
  });

  describe('map keycloak error representation to invalid param', () => {
    const controlName: string = faker.word.noun();
    const invalidParam: InvalidParam = createInvalidParam();

    beforeEach(() => {
      service._evaluateControlName = jest.fn().mockReturnValue(controlName);
      service._mapInvalidEmail = jest.fn().mockReturnValue(invalidParam);
      service._mapInvalidLength = jest.fn().mockReturnValue(invalidParam);
      service._mapEmailExists = jest.fn().mockReturnValue(invalidParam);
      service._mapUsernameExists = jest.fn().mockReturnValue(invalidParam);
      service._mapUserNameIsMissing = jest.fn().mockReturnValue(invalidParam);
    });

    it('should map invalid email', () => {
      service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.INVALID_EMAIL,
      });

      expect(service._mapInvalidEmail).toHaveBeenCalledWith(controlName);
    });

    it('should return invalid param for invalid email', () => {
      const param: InvalidParam = service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.INVALID_EMAIL,
      });

      expect(param).toEqual(invalidParam);
    });

    it('should map invalid length', () => {
      const errorRepresentation: ErrorRepresentation = {
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.INVALID_LENGTH,
      };

      service._mapKeycloakErrorRepresentationToInvalidParam(errorRepresentation);

      expect(service._mapInvalidLength).toHaveBeenCalledWith(controlName, errorRepresentation);
    });

    it('should return invalid param for invalid length', () => {
      const param: InvalidParam = service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.INVALID_LENGTH,
      });

      expect(param).toEqual(invalidParam);
    });

    it('should map email exists', () => {
      service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.EMAIL_EXISTS,
      });

      expect(service._mapEmailExists).toHaveBeenCalledWith(controlName);
    });

    it('should return invalid param for email exists', () => {
      const param: InvalidParam = service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.EMAIL_EXISTS,
      });

      expect(param).toEqual(invalidParam);
    });

    it('should map username exists', () => {
      service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.USERNAME_EXISTS,
      });

      expect(service._mapUsernameExists).toHaveBeenCalledWith(controlName);
    });

    it('should return invalid param for username exists', () => {
      const param: InvalidParam = service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.USERNAME_EXISTS,
      });

      expect(param).toEqual(invalidParam);
    });

    it('should map username is missing', () => {
      service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
      });

      expect(service._mapUserNameIsMissing).toHaveBeenCalledWith(controlName);
    });

    it('should return invalid param for username is missing', () => {
      const param: InvalidParam = service._mapKeycloakErrorRepresentationToInvalidParam({
        ...createErrorRepresentation(),
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
      });

      expect(param).toEqual(invalidParam);
    });

    it('should throw error', () => {
      expect(() =>
        service._mapKeycloakErrorRepresentationToInvalidParam({
          ...createErrorRepresentation(),
          errorMessage: 'unknown' as KeycloakErrorMessage,
        }),
      ).toThrowError();
    });
  });

  describe('map invalid email', () => {
    it('should return invalid param', () => {
      expect(service._mapInvalidEmail('control')).toEqual({
        name: 'control',
        reason: ValidationMessageCode.FIELD_INVALID,
        constraintParameters: [],
        value: null,
      } as InvalidParam);
    });
  });

  describe('map invalid length', () => {
    it('should return invalid param', () => {
      expect(service._mapInvalidLength('control', { params: ['email', 1, 2] })).toEqual({
        name: 'control',
        reason: ValidationMessageCode.FIELD_SIZE,
        constraintParameters: [
          { name: 'min', value: '1' },
          { name: 'max', value: '2' },
        ],
        value: null,
      } as InvalidParam);
    });
  });

  describe('map email exists', () => {
    it('should return invalid param', () => {
      expect(service._mapEmailExists('control')).toEqual({
        name: 'control',
        reason: ValidationMessageCode.FIELD_VALUE_ALREADY_EXISTS,
        constraintParameters: [{ name: 'value', value: 'Email-Adresse' }],
        value: null,
      } as InvalidParam);
    });
  });

  describe('map username exists', () => {
    it('should return invalid param', () => {
      expect(service._mapUsernameExists('control')).toEqual({
        name: 'control',
        reason: ValidationMessageCode.FIELD_VALUE_ALREADY_EXISTS,
        constraintParameters: [{ name: 'value', value: 'Benutzername' }],
        value: null,
      } as InvalidParam);
    });
  });

  describe('map username is missing', () => {
    it('should return invalid param', () => {
      expect(service._mapUserNameIsMissing('control')).toEqual({
        name: 'control',
        reason: ValidationMessageCode.FIELD_EMPTY,
        constraintParameters: [],
        value: null,
      } as InvalidParam);
    });
  });

  describe('evaluate control name', () => {
    const errorRepresentation: ErrorRepresentation = createErrorRepresentation();

    beforeEach(() => {
      service._mapKeycloakErrorMessageToControlName = jest.fn();
      service._mapKeycloakFieldNameToControlName = jest.fn();
    });

    describe('on single error message', () => {
      beforeEach(() => {
        isSingleErrorMessageMock.mockReturnValue(true);
        isFieldErrorMessageMock.mockReturnValue(false);
      });

      it('should check if single error message', () => {
        service._evaluateControlName(errorRepresentation);

        expect(isSingleErrorMessageMock).toHaveBeenCalledWith(errorRepresentation);
      });

      it('should map to control name', () => {
        service._evaluateControlName(errorRepresentation);

        expect(service._mapKeycloakErrorMessageToControlName).toHaveBeenCalledWith(errorRepresentation.errorMessage);
      });

      it('should return control name', () => {
        const controlName: string = faker.word.noun();
        service._mapKeycloakErrorMessageToControlName = jest.fn().mockReturnValue(controlName);

        expect(service._evaluateControlName(errorRepresentation)).toEqual(controlName);
      });
    });

    describe('on field error message', () => {
      beforeEach(() => {
        isSingleErrorMessageMock.mockReturnValue(false);
        isFieldErrorMessageMock.mockReturnValue(true);
      });

      it('should check if field error message', () => {
        service._evaluateControlName(errorRepresentation);

        expect(isFieldErrorMessageMock).toHaveBeenCalledWith(errorRepresentation);
      });

      it('should map to control name', () => {
        service._evaluateControlName(errorRepresentation);

        expect(service._mapKeycloakFieldNameToControlName).toHaveBeenCalledWith(errorRepresentation.field);
      });

      it('should return control name', () => {
        const controlName: string = faker.word.noun();
        service._mapKeycloakFieldNameToControlName = jest.fn().mockReturnValue(controlName);

        expect(service._evaluateControlName(errorRepresentation)).toEqual(controlName);
      });
    });

    it('should throw error', () => {
      isSingleErrorMessageMock.mockReturnValue(false);
      isFieldErrorMessageMock.mockReturnValue(false);

      expect(() => service._evaluateControlName(errorRepresentation)).toThrowError();
    });
  });

  describe('patch', () => {
    const valueToPatch: Dummy = { name: 'newName' };
    let patchFormSpy: jest.SpyInstance;

    beforeEach(() => {
      service.form = formBuilder.group({ name: 'dummyName' });
      patchFormSpy = jest.spyOn(FormUtil, 'patchForm').mockImplementation();
    });

    it('should call patch form', () => {
      service._patch(valueToPatch);

      expect(patchFormSpy).toHaveBeenCalledWith(valueToPatch, service.form);
    });

    it('should reset form value', () => {
      service._patch(valueToPatch);

      expect(service.form.value).toEqual({ name: null });
    });

    it('should set patched', () => {
      service._patched = false;

      service._patch(valueToPatch);

      expect(service._patched).toBeTruthy();
    });
  });

  describe('get form value', () => {
    const dummyName: string = 'DummyName';

    it('should return form value', () => {
      service.form = formBuilder.group({ name: dummyName });

      const formValue: unknown = service._getFormValue();

      expect(formValue).toEqual({ name: dummyName });
    });
  });

  describe('is patch', () => {
    it('should return patched value', () => {
      service._patched = true;

      const isPatch: boolean = service.isPatch();

      expect(isPatch).toBeTruthy();
    });
  });

  describe('is invalid', () => {
    it('should return true on invalid form', () => {
      service.form = formBuilder.group({ name: [EMPTY_STRING, Validators.required] });

      const isInvalid: boolean = service.isInvalid();

      expect(isInvalid).toBeTruthy();
    });
  });

  describe('get id', () => {
    const patchConfigId: string = faker.string.alphanumeric();
    const patchConfig: PatchConfig = { id: patchConfigId, doPatch: false };

    it('should return id from patch config', () => {
      service._patchConfig = patchConfig;

      const id: string = service.getId();

      expect(id).toBe(patchConfigId);
    });
  });
});

@Injectable()
export class TestKeycloakFormService extends KeycloakFormService<Dummy> {
  public static readonly FIELD: string = 'attribute';

  public static SUBMIT_OBSERVABLE = () => of(createEmptyStateResource());
  public static DELETE_OBSERVABLE = () => of(createEmptyStateResource());
  public static LOAD_OBSERVABLE = () => of(createEmptyStateResource());

  _initForm(): FormGroup {
    return this.formBuilder.group({
      [TestKeycloakFormService.FIELD]: new FormControl(null),
    });
  }

  _buildPatchConfig(urlSegments: UrlSegment[]): PatchConfig {
    return <PatchConfig>{};
  }

  _load(id: string): Observable<StateResource<Dummy>> {
    return TestKeycloakFormService.LOAD_OBSERVABLE();
  }

  _doAfterPatch(stateResource: StateResource<Dummy>) {}

  _doSubmit(): Observable<StateResource<Dummy>> {
    return TestKeycloakFormService.SUBMIT_OBSERVABLE();
  }

  _doDelete(): Observable<StateResource<Dummy>> {
    return TestKeycloakFormService.DELETE_OBSERVABLE();
  }

  _mapKeycloakFieldNameToControlName(keycloakFieldName: KeycloakFieldName): string {
    throw new Error('Method not implemented.');
  }

  _mapKeycloakErrorMessageToControlName(keycloakErrorMessage: KeycloakErrorMessage): string {
    throw new Error('Method not implemented.');
  }

  _handleUnknownKeycloakError(error: KeycloakHttpErrorResponse): void {
    throw new Error('Method not implemented.');
  }
}
