import { AuswertungResource } from '@admin-client/reporting-shared';
import { ROUTES } from '@admin-client/shared';
import { NavigationService, RouteData } from '@alfa-client/navigation-shared';
import {
  createEmptyStateResource,
  createStateResource,
  decodeUrlFromEmbedding,
  ResourceRepository,
  ResourceServiceConfig,
  StateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { UrlSegment } from '@angular/router';
import { faker } from '@faker-js/faker';
import { afterAll, expect } from '@jest/globals';
import { ResourceUri } from '@ngxp/rest';
import { Observable, of } from 'rxjs';
import { createRouteData, createUrlSegment } from '../../../../../navigation-shared/test/navigation-test-factory';
import { singleColdCompleted } from '../../../../../tech-shared/test/marbles';
import { createAuswertungResource } from '../../../test/auswertung';

import * as self from './auswertung-resource.service';

jest.mock('@alfa-client/tech-shared', () => ({
  ...jest.requireActual('@alfa-client/tech-shared'),
  decodeUrlFromEmbedding: jest.fn(),
}));

const decodeUrlFromEmbeddingMock: jest.Mock = decodeUrlFromEmbedding as jest.Mock;

describe('AuswertungResourceService', () => {
  let repository: Mock<ResourceRepository>;
  let navigationService: Mock<NavigationService>;

  beforeEach(() => {
    repository = mock(ResourceRepository);
    navigationService = mock(NavigationService);
  });

  describe('build config', () => {
    const getResourceByNavigationRouteSpy: jest.SpyInstance = jest
      .spyOn(self, '_getResourceByNavigationRoute')
      .mockImplementation();

    afterAll(() => {
      getResourceByNavigationRouteSpy.mockRestore();
    });

    it('should get resource by navigation route', () => {
      self._buildResourceServiceConfig(useFromMock(repository), useFromMock(navigationService));

      expect(getResourceByNavigationRouteSpy).toHaveBeenCalled();
    });

    it('should have auswertung static resource', () => {
      const staticResource: StateResource<AuswertungResource> = createStateResource(createAuswertungResource());
      getResourceByNavigationRouteSpy.mockReturnValue(of(staticResource));

      const config: ResourceServiceConfig<AuswertungResource> = self._buildResourceServiceConfig(
        useFromMock(repository),
        useFromMock(navigationService),
      );

      expect(config.baseResource).toBeObservable(singleColdCompleted(staticResource));
    });
  });

  describe('get resource by navigation route', () => {
    const routeData: RouteData = createRouteData();
    const _isAuswertungEditUrl: jest.SpyInstance = jest.spyOn(self, '_isAuswertungEditUrl').mockImplementation();
    const _getAuswertungResourceByRoute: jest.SpyInstance = jest
      .spyOn(self, '_getAuswertungResourceByRoute')
      .mockImplementation();

    beforeEach(() => {
      navigationService.getCurrentRouteData.mockReturnValue(of(routeData));
    });

    afterAll(() => {
      _isAuswertungEditUrl.mockRestore();
      _getAuswertungResourceByRoute.mockRestore();
    });

    it('should get current route data', () => {
      self._getResourceByNavigationRoute(useFromMock(repository), useFromMock(navigationService)).subscribe();

      expect(navigationService.getCurrentRouteData).toHaveBeenCalled();
    });

    it('should check if url contains resource uri', () => {
      self._getResourceByNavigationRoute(useFromMock(repository), useFromMock(navigationService)).subscribe();

      expect(_isAuswertungEditUrl).toHaveBeenCalled();
    });

    it('should get auswertung resource by route', () => {
      self._getResourceByNavigationRoute(useFromMock(repository), useFromMock(navigationService)).subscribe();

      expect(_getAuswertungResourceByRoute).toHaveBeenCalled();
    });

    it('should return auswertung by route', () => {
      const stateResource: StateResource<AuswertungResource> = createStateResource(createAuswertungResource());
      _getAuswertungResourceByRoute.mockReturnValue(of(stateResource));
      _isAuswertungEditUrl.mockReturnValue(true);

      const resourceByRoute$: Observable<StateResource<AuswertungResource>> = self._getResourceByNavigationRoute(
        useFromMock(repository),
        useFromMock(navigationService),
      );

      expect(resourceByRoute$).toBeObservable(singleColdCompleted(stateResource));
    });

    it('should return empty state resource', () => {
      const stateResource: StateResource<AuswertungResource> = createEmptyStateResource();
      _isAuswertungEditUrl.mockReturnValue(false);

      const resourceByRoute$: Observable<StateResource<AuswertungResource>> = self._getResourceByNavigationRoute(
        useFromMock(repository),
        useFromMock(navigationService),
      );

      expect(resourceByRoute$).toBeObservable(singleColdCompleted(stateResource));
    });
  });

  describe('is auswertung edit url', () => {
    it.each([0, 1, 3])('should return false of wrong number of segments = %d', (numberOfSegments: number) => {
      const routeData: RouteData = {
        ...createRouteData(),
        urlSegments: Array(numberOfSegments).fill(createUrlSegment()),
      };

      const isEditUrl: boolean = self._isAuswertungEditUrl(routeData);

      expect(isEditUrl).toBe(false);
    });

    it('should return false if first path is wrong', () => {
      const routeData: RouteData = {
        ...createRouteData(),
        urlSegments: Array(2).fill(createUrlSegment()),
      };

      const isEditUrl: boolean = self._isAuswertungEditUrl(routeData);

      expect(isEditUrl).toBe(false);
    });

    it('should return false if second path is wrong', () => {
      const firstUrlSegment: UrlSegment = createUrlSegment();
      firstUrlSegment.path = ROUTES.AUSWERTUNGEN;
      const secondUrlSegment: UrlSegment = createUrlSegment();
      secondUrlSegment.path = 'neu';
      const routeData: RouteData = {
        ...createRouteData(),
        urlSegments: [firstUrlSegment, secondUrlSegment],
      };

      const isEditUrl: boolean = self._isAuswertungEditUrl(routeData);

      expect(isEditUrl).toBe(false);
    });

    it('should return true', () => {
      const routeData: RouteData = createAuswertungEditRouteData();

      const isEditUrl: boolean = self._isAuswertungEditUrl(routeData);

      expect(isEditUrl).toBe(true);
    });
  });

  describe('get auswertung resource by route', () => {
    const routeData: RouteData = createAuswertungEditRouteData();

    const uri: ResourceUri = faker.internet.url();
    const resource: AuswertungResource = createAuswertungResource();

    beforeEach(() => {
      decodeUrlFromEmbeddingMock.mockReturnValue(uri);
      repository.getResource.mockReturnValue(of(resource));
    });

    it('should decode url', () => {
      self._getAuswertungResourceByRoute(useFromMock(repository), routeData);

      expect(decodeUrlFromEmbeddingMock).toHaveBeenCalledWith(routeData.urlSegments[2].path);
    });

    it('should get resource', () => {
      self._getAuswertungResourceByRoute(useFromMock(repository), routeData);

      expect(repository.getResource).toHaveBeenCalledWith(uri);
    });

    it('should return state resource', () => {
      const stateResource$: Observable<StateResource<AuswertungResource>> = self._getAuswertungResourceByRoute(
        useFromMock(repository),
        routeData,
      );

      expect(stateResource$).toBeObservable(singleColdCompleted(createStateResource(resource)));
    });
  });

  function createAuswertungEditRouteData() {
    const firstUrlSegment: UrlSegment = createUrlSegment();
    firstUrlSegment.path = 'statistik';
    const secondUrlSegment: UrlSegment = createUrlSegment();
    secondUrlSegment.path = 'auswertungen';
    const thirdUrlSegment: UrlSegment = createUrlSegment();
    thirdUrlSegment.path = faker.internet.url();

    return {
      ...createRouteData(),
      urlSegments: [firstUrlSegment, secondUrlSegment, thirdUrlSegment],
    };
  }
});
