/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AuswertungResource, AuswertungService } from '@admin-client/reporting-shared';
import { createStateResource, EMPTY_STRING, StateResource } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { TestBed } from '@angular/core/testing';
import { FormArray, FormControl, FormGroup } from '@angular/forms';
import { expect } from '@jest/globals';
import { omit } from 'lodash-es';
import { of } from 'rxjs';
import { singleColdCompleted } from '../../../../../../../tech-shared/test/marbles';
import { createAuswertung, createAuswertungResource } from '../../../../../../reporting-shared/test/auswertung';
import { AuswertungFormservice } from './auswertung.formservice';

describe('AuswertungFormService', () => {
  let formService: AuswertungFormservice;

  let service: Mock<AuswertungService>;

  beforeEach(() => {
    service = mock(AuswertungService);

    TestBed.configureTestingModule({
      providers: [AuswertungFormservice, { provide: AuswertungService, useValue: service }],
    });

    formService = TestBed.inject(AuswertungFormservice);
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  describe('on do submit', () => {
    const stateResource: StateResource<AuswertungResource> = createStateResource(createAuswertungResource());

    beforeEach(() => {
      service.create.mockReturnValue(of(stateResource));
      service.save.mockReturnValue(of(stateResource));
    });

    it('should create', () => {
      formService.isPatch = jest.fn().mockReturnValue(false);
      formService.form = <any>createAuswertung();

      formService.submit();

      expect(service.create).toHaveBeenCalledWith(formService.form.value);
    });

    it('should save', () => {
      formService.isPatch = jest.fn().mockReturnValue(true);
      formService.form = <any>createAuswertung();

      formService.submit();

      expect(service.save).toHaveBeenCalledWith(formService.form.value);
    });
  });

  describe('add mapping', () => {
    it('should add mapping control', () => {
      formService.addMapping();

      const mappingFormArray: FormArray = <FormArray>formService.form.controls[AuswertungFormservice.FIELD_MAPPINGS];
      expect(mappingFormArray).toHaveLength(2);
      expect(mappingFormArray.controls[0].value).toEqual({
        [AuswertungFormservice.FIELD_MAPPING_SOURCE_PATH]: EMPTY_STRING,
        [AuswertungFormservice.FIELD_MAPPING_TARGET_PATH]: EMPTY_STRING,
      });
    });
  });

  describe('remove mapping', () => {
    it('should remove mapping control', () => {
      (<FormArray>formService.form.controls[AuswertungFormservice.FIELD_MAPPINGS]).push(
        new FormGroup({
          [AuswertungFormservice.FIELD_MAPPING_SOURCE_PATH]: new FormControl('controlToRemove'),
          [AuswertungFormservice.FIELD_MAPPING_TARGET_PATH]: new FormControl('controlToRemove'),
        }),
      );

      formService.removeMapping(1);

      const mappingFormArray: FormArray = <FormArray>formService.form.controls[AuswertungFormservice.FIELD_MAPPINGS];
      expect(mappingFormArray).toHaveLength(1);
      expect(mappingFormArray.controls[0].value).toEqual({
        [AuswertungFormservice.FIELD_MAPPING_SOURCE_PATH]: EMPTY_STRING,
        [AuswertungFormservice.FIELD_MAPPING_TARGET_PATH]: EMPTY_STRING,
      });
    });
  });

  describe('get mappings', () => {
    it('should return mappings as array', () => {
      const mappings: FormArray = formService.mappings;

      expect(mappings).toHaveLength(1);
      expect(mappings.controls[0].value).toEqual({
        [AuswertungFormservice.FIELD_MAPPING_SOURCE_PATH]: EMPTY_STRING,
        [AuswertungFormservice.FIELD_MAPPING_TARGET_PATH]: EMPTY_STRING,
      });
    });
  });

  describe('get', () => {
    const stateResource: StateResource<AuswertungResource> = createStateResource(createAuswertungResource());

    beforeEach(() => {
      service.get.mockReturnValue(of(stateResource));
      formService._patchForm = jest.fn();
    });

    it('should get by current url', () => {
      formService.get();

      expect(service.get).toHaveBeenCalled();
    });

    it('should patch form', () => {
      formService.get().subscribe();

      expect(formService._patchForm).toHaveBeenCalledWith(stateResource.resource);
    });

    it('should emit state resource', () => {
      expect(formService.get()).toBeObservable(singleColdCompleted(stateResource));
    });
  });

  describe('patch form', () => {
    const auswertungResource: AuswertungResource = createAuswertungResource();

    it('should patch', () => {
      formService._patchForm(auswertungResource);

      expect(formService.form.value).toEqual(omit(auswertungResource, '_links'));
    });
  });
});
