/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AdminRoles, AlfaRoles, RoleMappings, User, UserRepository } from '@admin-client/user-shared';
import { KeycloakDefaults } from '@admin/keycloak-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { fakeAsync, TestBed, tick } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { expect } from '@jest/globals';
import KcAdminClient from '@keycloak/keycloak-admin-client';
import GroupRepresentation from '@keycloak/keycloak-admin-client/lib/defs/groupRepresentation';
import MappingsRepresentation from '@keycloak/keycloak-admin-client/lib/defs/mappingsRepresentation';
import { RequiredActionAlias } from '@keycloak/keycloak-admin-client/lib/defs/requiredActionProviderRepresentation';
import { RoleMappingPayload } from '@keycloak/keycloak-admin-client/lib/defs/roleRepresentation';
import { Users } from '@keycloak/keycloak-admin-client/lib/resources/users';
import { cold } from 'jest-marbles';
import { omit, times } from 'lodash-es';
import { createUser } from '../../test/user';

describe('UserRepository', () => {
  let repository: UserRepository;

  let kcAdminClient: Mock<KcAdminClient>;

  const user: User = createUser();
  let kcAdminClientUsers: Mock<Users>;

  beforeEach(() => {
    kcAdminClientUsers = {
      ...mock(Users),
      executeActionsEmail: jest.fn().mockImplementation(),
      create: jest.fn().mockReturnValue(Promise.resolve({ id: user.id })),
    };
    kcAdminClient = { ...mock(KcAdminClient), users: <any>kcAdminClientUsers };

    TestBed.configureTestingModule({
      providers: [{ provide: KcAdminClient, useValue: kcAdminClient }],
    });
    repository = TestBed.inject(UserRepository);
  });

  it('should be created', () => {
    expect(repository).toBeTruthy();
  });

  describe('createInKeycloak', () => {
    it('should call kcAdminClient users create', () => {
      repository.createInKeycloak(user);

      expect(kcAdminClient.users['create']).toHaveBeenCalledWith(omit(user, 'groupIds'));
    });

    it('should call updateUserRoles', fakeAsync(() => {
      repository._updateUserRoles = jest.fn();

      repository.createInKeycloak(user).subscribe();
      tick();

      expect(repository._updateUserRoles).toHaveBeenCalledWith(user.id, user.clientRoles);
    }));

    it('should call sendActivationMail', (done) => {
      repository._sendActivationMail = jest.fn();
      repository._updateUserRoles = jest.fn().mockReturnValue(Promise.resolve());

      repository.createInKeycloak(user).subscribe(() => {
        expect(repository._sendActivationMail).toHaveBeenCalledWith(user.id);
        done();
      });
    });
  });

  describe('saveInKeycloak', () => {
    beforeEach(() => {
      kcAdminClient.users = <any>{
        update: jest.fn().mockReturnValue(Promise.resolve()),
      };

      repository._updateUserRoles = jest.fn().mockReturnValue(Promise.resolve());
      repository._updateUserGroups = jest.fn().mockReturnValue(Promise.resolve());
    });

    it('should call kcAdminClient users update', () => {
      repository.saveInKeycloak(user);

      expect(kcAdminClient.users['update']).toHaveBeenCalledWith({ id: user.id }, omit(user, 'groupIds'));
    });

    it('should call updateUserRoles', fakeAsync(() => {
      repository.saveInKeycloak(user).subscribe();
      tick();

      expect(repository._updateUserRoles).toHaveBeenCalledWith(user.id, user.clientRoles);
    }));

    it('should call updateUserGroups', (done) => {
      repository.saveInKeycloak(user).subscribe(() => {
        expect(repository._updateUserGroups).toHaveBeenCalledWith(user.id, user.groupIds);
        done();
      });
    });
  });

  describe('updateUserGroups', () => {
    const oldGroups: string[] = Array.from({ length: 3 }, faker.string.uuid);

    beforeEach(() => {
      repository._getOldUserGroupIds = jest.fn().mockReturnValue(Promise.resolve(oldGroups));
      repository._deleteUserGroups = jest.fn();
      repository._addUserGroups = jest.fn();
    });

    it('should call getOldUserGroups', async () => {
      await repository._updateUserGroups(user.id, user.groups);

      expect(repository._getOldUserGroupIds).toHaveBeenCalled();
    });

    it('should call deleteUserGroups', async () => {
      await repository._updateUserGroups(user.id, user.groups);

      expect(repository._deleteUserGroups).toHaveBeenCalledWith(user.id, user.groups, oldGroups);
    });

    it('should call addUserGroups', async () => {
      await repository._updateUserGroups(user.id, user.groups);

      expect(repository._addUserGroups).toHaveBeenCalledWith(user.id, user.groups, oldGroups);
    });
  });

  describe('getOldUserGroupIds', () => {
    const groupReps: GroupRepresentation[] = Array.from({ length: 3 }, () => ({ id: faker.string.uuid() }));
    const groupIds: string[] = groupReps.map((group) => group.id);

    beforeEach(() => {
      kcAdminClient.users = <any>{
        listGroups: jest.fn().mockReturnValue(Promise.resolve(groupReps)),
      };
    });

    it('should call kcAdminClient users listGroups', async () => {
      await repository._getOldUserGroupIds(user.id);

      expect(kcAdminClient.users['listGroups']).toHaveBeenCalledWith({ id: user.id });
    });

    it('should return group ids', async () => {
      const result: string[] = await repository._getOldUserGroupIds(user.id);

      expect(result).toEqual(groupIds);
    });
  });

  describe('deleteUserGroups', () => {
    const oldUserGroupIds: string[] = Array.from({ length: 2 }, faker.string.uuid);
    const newUserGroupIds: string[] = [oldUserGroupIds[0]];

    beforeEach(() => {
      kcAdminClient.users = <any>{
        delFromGroup: jest.fn().mockReturnValue(Promise.resolve()),
      };
    });

    it('should call kcAdminClient users listGroups to be called', async () => {
      await repository._deleteUserGroups(user.id, newUserGroupIds, oldUserGroupIds);

      expect(kcAdminClient.users['delFromGroup']).toHaveBeenCalledWith({ id: user.id, groupId: oldUserGroupIds[1] });
    });

    it('should call kcAdminClient users listGroups for every group to delete', async () => {
      await repository._deleteUserGroups(user.id, [], oldUserGroupIds);

      expect(kcAdminClient.users['delFromGroup']).toHaveBeenCalledTimes(oldUserGroupIds.length);
    });
  });

  describe('addUserGroups', () => {
    const newUserGroupIds: string[] = Array.from({ length: 2 }, faker.string.uuid);
    const oldUserGroupIds: string[] = [newUserGroupIds[0]];

    beforeEach(() => {
      kcAdminClient.users = <any>{
        addToGroup: jest.fn().mockReturnValue(Promise.resolve()),
      };
    });

    it('should call kcAdminClient users addToGroup', () => {
      repository._addUserGroups(user.id, newUserGroupIds, oldUserGroupIds);

      expect(kcAdminClient.users['addToGroup']).toHaveBeenCalledWith({ id: user.id, groupId: newUserGroupIds[1] });
    });

    it('should call kcAdminClient users addToGroup for every group', () => {
      const groups = Array.from({ length: 3 }, () => faker.string.uuid());
      user.groups = groups;
      repository._addUserGroups(user.id, newUserGroupIds, []);

      expect(kcAdminClient.users['addToGroup']).toHaveBeenCalledTimes(newUserGroupIds.length);
    });
  });

  describe('UpdateUserRoles', () => {
    const clientId: string = faker.string.uuid();

    beforeEach(() => {
      repository._updateUserRolesForClient = jest.fn();
      repository._getClientId = jest.fn().mockReturnValue(clientId);
    });

    it('should call getClientId for admin', async () => {
      await repository._updateUserRoles(user.id, { admin: [AdminRoles.ADMIN], alfa: [] });

      expect(repository._getClientId).toHaveBeenCalledWith(UserRepository.ADMIN_CLIENT_NAME);
    });

    it('should call UpdateUserRolesForClient for admin', async () => {
      await repository._updateUserRoles(user.id, { admin: [AdminRoles.ADMIN], alfa: [] });

      expect(repository._updateUserRolesForClient).toHaveBeenCalledWith(user.id, [AdminRoles.ADMIN], clientId);
    });

    it('should call getClientId for admin', async () => {
      await repository._updateUserRoles(user.id, { alfa: [AlfaRoles.POSTSTELLE], admin: [] });

      expect(repository._getClientId).toHaveBeenCalledWith(UserRepository.ALFA_CLIENT_NAME);
    });

    it('should call UpdateUserRolesForClient for admin', async () => {
      await repository._updateUserRoles(user.id, { alfa: [AlfaRoles.POSTSTELLE], admin: [] });

      expect(repository._updateUserRolesForClient).toHaveBeenCalledWith(user.id, [AlfaRoles.POSTSTELLE], clientId);
    });
  });

  describe('updateUserRolesForClient', () => {
    const clientId: string = faker.string.uuid();
    const clientRoles: string[] = times(3, () => faker.word.sample());
    const newClientRoleMappings: RoleMappingPayload[] = clientRoles.map((role: string) => ({
      id: faker.string.uuid(),
      name: role,
    }));
    const oldClientRoleMappings: RoleMappingPayload[] = newClientRoleMappings.slice(1);
    const clientRoleMappings: RoleMappings = { newClientRoleMappings, oldClientRoleMappings };

    beforeEach(() => {
      repository._getClientRoleMappings = jest.fn().mockReturnValue(Promise.resolve(clientRoleMappings));
      repository._deleteUserRoles = jest.fn();
      repository._addUserRoles = jest.fn();
    });

    it('should call getClientRoleMappings', async () => {
      await repository._updateUserRolesForClient(user.id, clientRoles, clientId);

      expect(repository._getClientRoleMappings).toHaveBeenCalledWith(user.id, clientId, clientRoles);
    });

    it('should call deleteUserRoles', async () => {
      await repository._updateUserRolesForClient(user.id, clientRoles, clientId);

      expect(repository._deleteUserRoles).toHaveBeenCalledWith(user.id, clientRoleMappings, clientId);
    });

    it('should call addUserRoles', async () => {
      await repository._updateUserRolesForClient(user.id, clientRoles, clientId);

      expect(repository._addUserRoles).toHaveBeenCalledWith(user.id, clientRoleMappings, clientId);
    });
  });

  describe('getClientRoleMappings', () => {
    const clientId: string = faker.string.uuid();
    const newClientRoles: string[] = times(3, () => faker.word.sample());
    const newClientRoleMappings: RoleMappingPayload[] = newClientRoles.map((role: string) => ({
      id: faker.string.uuid(),
      name: role,
    }));
    const oldClientRoleMappings: RoleMappingPayload[] = newClientRoleMappings.slice(1);
    const clientRoleMappings: RoleMappings = { newClientRoleMappings, oldClientRoleMappings };

    beforeEach(() => {
      repository._getOldUserRoleMappings = jest.fn().mockReturnValue(Promise.resolve(oldClientRoleMappings));
      repository._getClientId = jest.fn().mockReturnValue(Promise.resolve(clientId));
      repository._mapUserRoles = jest.fn().mockReturnValue(Promise.resolve(newClientRoleMappings));
      repository._deleteUserRoles = jest.fn();
      repository._addUserRoles = jest.fn();
    });

    it('should call mapUserRoles', () => {
      repository._getClientRoleMappings(user.id, clientId, newClientRoles);

      expect(repository._mapUserRoles).toHaveBeenCalledWith(clientId, newClientRoles);
    });

    it('should call get old user role mappings', async () => {
      await repository._getClientRoleMappings(user.id, clientId, newClientRoles);

      expect(repository._getOldUserRoleMappings).toHaveBeenCalledWith(user.id, clientId);
    });

    it('should return clientRoleMappings', async () => {
      const result: RoleMappings = await repository._getClientRoleMappings(user.id, clientId, newClientRoles);

      expect(result).toEqual(clientRoleMappings);
    });
  });

  describe('getOldUserRoleMappings', () => {
    const clientId: string = faker.string.uuid();
    const roleMapping: RoleMappingPayload[] = [{ id: faker.string.uuid(), name: faker.word.sample() }];

    beforeEach(() => {
      kcAdminClient.users = <any>{
        listClientRoleMappings: jest.fn().mockReturnValue(Promise.resolve(roleMapping)),
      };
    });

    it('should call users listClientRoleMappings', () => {
      repository._getOldUserRoleMappings(user.id, clientId);

      expect(kcAdminClient.users['listClientRoleMappings']).toHaveBeenCalledWith({ id: user.id, clientUniqueId: clientId });
    });

    it('should return roleMapping', async () => {
      const result: RoleMappingPayload[] = await repository._getOldUserRoleMappings(user.id, clientId);

      expect(result).toEqual(roleMapping);
    });
  });

  describe('deleteUserRoles', () => {
    const clientId: string = faker.string.uuid();
    const clientRoles: string[] = times(3, () => faker.word.sample());
    const oldClientRoleMappings: RoleMappingPayload[] = clientRoles.map((role: string) => ({
      id: faker.string.uuid(),
      name: role,
    }));
    const newClientRoleMappings: RoleMappingPayload[] = oldClientRoleMappings.slice(1);
    const clientRoleMappings: RoleMappings = { newClientRoleMappings, oldClientRoleMappings };

    beforeEach(() => {
      repository._deleteUserRolesInKeycloak = jest.fn();

      kcAdminClient.users = <any>{
        addClientRoleMappings: jest.fn().mockReturnValue(Promise.resolve()),
      };
    });

    it('should call deleteUserRolesInKeycloak', async () => {
      await repository._deleteUserRoles(user.id, clientRoleMappings, clientId);

      expect(repository._deleteUserRolesInKeycloak).toHaveBeenCalledWith(user.id, clientId, [oldClientRoleMappings[0]]);
    });

    it('should call not call deleteUserRolesInKeycloak if no roles to delete', async () => {
      await repository._deleteUserRoles(
        user.id,
        { oldClientRoleMappings, newClientRoleMappings: oldClientRoleMappings },
        clientId,
      );

      expect(repository._deleteUserRolesInKeycloak).not.toHaveBeenCalled();
    });
  });

  describe('addUserRoles', () => {
    const clientId: string = faker.string.uuid();
    const clientRoles: string[] = times(3, () => faker.word.sample());
    const newClientRoleMappings: RoleMappingPayload[] = clientRoles.map((role: string) => ({
      id: faker.string.uuid(),
      name: role,
    }));
    const oldClientRoleMappings: RoleMappingPayload[] = newClientRoleMappings.slice(1);
    const clientRoleMappings: RoleMappings = { newClientRoleMappings, oldClientRoleMappings };

    beforeEach(() => {
      repository._addUserRolesInKeycloak = jest.fn();

      kcAdminClient.users = <any>{
        addClientRoleMappings: jest.fn().mockReturnValue(Promise.resolve()),
      };
    });

    it('should call addUserRolesInKeycloak', async () => {
      await repository._addUserRoles(user.id, clientRoleMappings, clientId);

      expect(repository._addUserRolesInKeycloak).toHaveBeenCalledWith(user.id, clientId, [newClientRoleMappings[0]]);
    });

    it('should not call addUserRolesInKeycloak if no roles to add', async () => {
      await repository._addUserRoles(user.id, { newClientRoleMappings, oldClientRoleMappings: newClientRoleMappings }, clientId);

      expect(repository._addUserRolesInKeycloak).not.toHaveBeenCalled();
    });
  });

  describe('getclientId', () => {
    const clientId = faker.string.uuid();

    beforeEach(() => {
      kcAdminClient.clients = <any>{
        find: jest.fn().mockReturnValue(Promise.resolve([{ id: clientId }])),
      };
    });

    it('should call kcAdminClient clients find', async () => {
      await repository._getClientId(UserRepository.ADMIN_CLIENT_NAME);

      expect(kcAdminClient.clients['find']).toHaveBeenCalledWith({ clientId: UserRepository.ADMIN_CLIENT_NAME });
    });

    it('should return clientId', async () => {
      const result: string = await repository._getClientId(UserRepository.ADMIN_CLIENT_NAME);

      expect(result).toEqual(clientId);
    });
  });

  describe('mapUserRoles', () => {
    const clientId: string = faker.string.uuid();
    const clientRoles: RoleMappingPayload[] = Array.from({ length: 3 }, () => ({
      id: faker.string.uuid(),
      name: faker.word.sample(),
    }));
    const userRoles: string[] = clientRoles.map((role: RoleMappingPayload): string => role.name).slice(1);

    beforeEach(() => {
      kcAdminClient.clients = <any>{
        listRoles: jest.fn().mockReturnValue(Promise.resolve(clientRoles)),
      };
    });

    it('should call kcAdminClient clients listRoles', async () => {
      await repository._mapUserRoles(clientId, userRoles);

      expect(kcAdminClient.clients['listRoles']).toHaveBeenCalledWith({ id: clientId });
    });

    it('should return roleMapping', async () => {
      const result: RoleMappingPayload[] = await repository._mapUserRoles(clientId, userRoles);

      expect(result).toEqual(clientRoles.slice(1));
    });

    it('should filter roles if they are not in clientRoles', async () => {
      const result: RoleMappingPayload[] = await repository._mapUserRoles(clientId, [faker.word.sample()]);

      expect(result).toEqual([]);
    });
  });

  describe('addUserRolesInKeycloak', () => {
    beforeEach(() => {
      kcAdminClient.users = <any>{
        addClientRoleMappings: jest.fn().mockReturnValue(Promise.resolve()),
      };
    });

    it('should call kcAdminClient users addClientRoleMappings', async () => {
      const clientId: string = faker.string.uuid();
      const roles: RoleMappingPayload[] = Array.from({ length: 3 }, () => ({
        id: faker.string.uuid(),
        name: faker.word.sample(),
      }));

      await repository._addUserRolesInKeycloak(user.id, clientId, roles);

      expect(kcAdminClient.users['addClientRoleMappings']).toHaveBeenCalledWith({ id: user.id, clientUniqueId: clientId, roles });
    });
  });

  describe('deleteUserRolesInKeycloak', () => {
    beforeEach(() => {
      kcAdminClient.users = <any>{
        delClientRoleMappings: jest.fn().mockReturnValue(Promise.resolve()),
      };
    });

    it('should call kcAdminClient users delClientRoleMappings', async () => {
      const clientId: string = faker.string.uuid();
      const roles: RoleMappingPayload[] = Array.from({ length: 3 }, () => ({
        id: faker.string.uuid(),
        name: faker.word.sample(),
      }));

      await repository._deleteUserRolesInKeycloak(user.id, clientId, roles);

      expect(kcAdminClient.users['delClientRoleMappings']).toHaveBeenCalledWith({ id: user.id, clientUniqueId: clientId, roles });
    });
  });

  describe('update user roles for admin', () => {
    const clientId: string = faker.string.uuid();

    beforeEach(() => {
      repository._updateUserRolesForClient = jest.fn();
      repository._getClientId = jest.fn().mockReturnValue(new Promise((resolve) => resolve(clientId)));
    });

    it('should get client id', async () => {
      await repository._updateUserRolesForAdmin(user.id, {
        admin: [AdminRoles.ADMIN],
        alfa: [],
      });

      expect(repository._getClientId).toHaveBeenCalledWith(KeycloakDefaults.clients.realmManagement.clientName);
    });

    it('should update user roles', async () => {
      await repository._updateUserRolesForAdmin(user.id, {
        admin: [AdminRoles.ADMIN],
        alfa: [],
      });

      expect(repository._updateUserRolesForClient).toHaveBeenCalledWith(
        user.id,
        [
          KeycloakDefaults.clients.realmManagement.roles.managerUsers.roleName,
          KeycloakDefaults.clients.realmManagement.roles.queryGroups.roleName,
          KeycloakDefaults.clients.realmManagement.roles.viewClients.roleName,
        ],
        clientId,
      );
    });

    it('should NOT update user roles for no ADMIN_ADMIN role', async () => {
      await repository._updateUserRolesForAdmin(user.id, {
        admin: [faker.word.noun()],
        alfa: [],
      });

      expect(repository._updateUserRolesForClient).not.toHaveBeenCalled();
    });
  });

  describe('sendActivationMail', () => {
    it('should call kcAdminClient users executeActionsEmail', () => {
      const userId: string = faker.string.uuid();

      repository._sendActivationMail(userId);

      expect(kcAdminClientUsers.executeActionsEmail).toHaveBeenCalledWith({
        id: userId,
        actions: [RequiredActionAlias.VERIFY_EMAIL, RequiredActionAlias.UPDATE_PASSWORD],
        lifespan: 3600 * 24 * 7,
      });
    });
  });

  describe('handleCreateError', () => {
    it('should throw error', () => {
      const error: Error = new Error('error');
      const result = repository._handleCreateError(error);

      expect(result).toBeObservable(cold('#', null, new Error('An error occurred while creating the user.')));
    });
  });

  describe('handleSaveError', () => {
    it('should throw error', () => {
      const error: Error = new Error('error');
      const result = repository._handleSaveError(error);

      expect(result).toBeObservable(cold('#', null, new Error('An error occurred while saving the user.')));
    });
  });

  describe('deleteUser', () => {
    const userId: string = faker.string.uuid();

    beforeEach(() => {
      kcAdminClient.users = <any>{
        del: jest.fn().mockReturnValue(Promise.resolve(undefined)),
      };
    });

    it('should call kcAdminClient users del', () => {
      repository.deleteUser(userId);

      expect(kcAdminClient.users['del']).toHaveBeenCalledWith({ id: userId });
    });

    it('should return void', (done) => {
      repository.deleteUser(userId).subscribe((result: void) => {
        expect(result).toBeUndefined();
        done();
      });
    });
  });

  describe('getUsers', () => {
    const user: User = createUser();
    const userArray: User[] = [user, user, user];
    const group: string = faker.word.sample();
    const alfaRole: string = faker.word.sample();
    const adminRole: string = faker.word.sample();
    const groupRep: GroupRepresentation[] = [{ name: group }];
    const roleRep: MappingsRepresentation = {
      clientMappings: {
        alfa: { mappings: [{ name: alfaRole }] },
        admin: { mappings: [{ name: adminRole }] },
      },
    };

    beforeEach(() => {
      jest.clearAllMocks();
      kcAdminClient.users = <any>{
        find: jest.fn().mockReturnValue(Promise.resolve(userArray)),
        listGroups: jest.fn().mockReturnValue(Promise.resolve(groupRep)),
        listRoleMappings: jest.fn().mockReturnValue(Promise.resolve(roleRep)),
      };
    });

    it('should call kcAdminClient users find', () => {
      repository.getUsers();

      expect(kcAdminClient.users['find']).toHaveBeenCalled();
    });

    it('should call mapToUser', fakeAsync(() => {
      repository._mapToUser = jest.fn().mockReturnValue(user);

      repository.getUsers().subscribe();
      tick();

      expect(repository._mapToUser).toHaveBeenCalledTimes(userArray.length);
    }));

    it('should call kcadminClient listGroups for every user', fakeAsync(() => {
      repository.getUsers().subscribe();
      tick();

      expect(kcAdminClient.users['listGroups']).toHaveBeenCalledTimes(userArray.length);
    }));

    it('should call kcadminClient listRoleMappings for every user', fakeAsync(() => {
      repository.getUsers().subscribe();
      tick();

      expect(kcAdminClient.users['listRoleMappings']).toHaveBeenCalledTimes(userArray.length);
    }));

    it('should return users with groups and roles', (done) => {
      repository.getUsers().subscribe((users: User[]) => {
        users.forEach((user) =>
          expect(user).toEqual({ ...user, groups: [group], clientRoles: { alfa: [alfaRole], admin: [adminRole] } }),
        );
        done();
      });
    });

    it('should return users with empty groups and roles if they have none', (done) => {
      kcAdminClient.users['listGroups'] = jest.fn().mockReturnValue(Promise.resolve([]));
      kcAdminClient.users['listRoleMappings'] = jest.fn().mockReturnValue(Promise.resolve({}));

      repository.getUsers().subscribe((users: User[]) => {
        users.forEach((user) => expect(user).toEqual({ ...user, groups: [], clientRoles: { alfa: [], admin: [] } }));
        done();
      });
    });
  });

  describe('getUserById', () => {
    const user: User = createUser();
    const group: string = faker.word.sample();
    const alfaRole: string = faker.word.sample();
    const adminRole: string = faker.word.sample();
    const groupRep: GroupRepresentation[] = [{ name: group }];
    const roleRep: MappingsRepresentation = {
      clientMappings: {
        alfa: { mappings: [{ name: alfaRole }] },
        admin: { mappings: [{ name: adminRole }] },
      },
    };

    beforeEach(() => {
      jest.clearAllMocks();
      kcAdminClient.users = <any>{
        findOne: jest.fn().mockReturnValue(Promise.resolve(user)),
        listGroups: jest.fn().mockReturnValue(Promise.resolve(groupRep)),
        listRoleMappings: jest.fn().mockReturnValue(Promise.resolve(roleRep)),
      };
    });

    it('should call kcAdminClient users findOne', () => {
      repository.getUserById(user.id);

      expect(kcAdminClient.users['findOne']).toHaveBeenCalledWith({ id: user.id });
    });

    it('should call kcAdminClient users listGroups', fakeAsync(() => {
      repository.getUserById(user.id).subscribe();
      tick();

      expect(kcAdminClient.users['listGroups']).toHaveBeenCalledWith({ id: user.id });
    }));

    it('should call kcAdminClient users listRoleMappings', fakeAsync(() => {
      repository.getUserById(user.id).subscribe();
      tick();

      expect(kcAdminClient.users['listRoleMappings']).toHaveBeenCalledWith({ id: user.id });
    }));

    it('should return user with groups and roles', (done) => {
      repository.getUserById(user.id).subscribe((userState: StateResource<User>) => {
        expect(userState.resource).toEqual({ ...user, groups: [group], clientRoles: { alfa: [alfaRole], admin: [adminRole] } });
        done();
      });
    });
  });
});
