/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidWizardContainerComponent } from '@alfa-client/bescheid';
import { BescheidResource, BescheidService, BescheidWizardDialogResult, BescheidWizardStep } from '@alfa-client/bescheid-shared';
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { createEmptyStateResource, createStateResource, ESCAPE_KEY, StateResource } from '@alfa-client/tech-shared';
import {
  createDialogRefMock,
  DialogRefMock,
  existsAsHtmlElement,
  getElementFromFixtureByType,
  Mock,
  mock,
  triggerEvent,
  useFromMock,
} from '@alfa-client/test-utils';
import { OzgcloudDialogService } from '@alfa-client/ui-shared';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonComponent, IconComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createBescheidResource, createBescheidStateResource } from '../../../../bescheid-shared/src/test/bescheid';
import { createCommandErrorResource, createSuccessfullyDoneCommandResource } from '../../../../command-shared/test/command';
import { getDataTestIdAttributeOf, getDataTestIdOf } from '../../../../tech-shared/test/data-test';
import { singleColdCompleted } from '../../../../tech-shared/test/marbles';
import { createKeydownKeyboardEvent } from '../../../../test-utils/src/lib/keyboard';
import { createVorgangWithEingangResource } from '../../../../vorgang-shared/test/vorgang';
import { BescheidWizardComponent } from './bescheid-wizard/bescheid-wizard.component';
import {
  BescheidWizardCancelDialogContainerComponent,
  CancelWizardDialogData,
  CancelWizardDialogResult,
} from './bescheid-wizard/cancel-dialog-container/bescheid-wizard-cancel-dialog-container.component';
import { BescheidFormService } from './bescheid.formservice';

describe('BescheidWizardContainerComponent', () => {
  let component: BescheidWizardContainerComponent;
  let fixture: ComponentFixture<BescheidWizardContainerComponent>;

  const bescheidWizard: string = getDataTestIdOf('bescheid-wizard');
  const closeButton: string = getDataTestIdAttributeOf('close-bescheid');

  let bescheidService: Mock<BescheidService>;
  let ozgcloudDialogService: Mock<OzgcloudDialogService>;
  let formService: Mock<BescheidFormService>;
  let wizardDialogRef: DialogRefMock<BescheidWizardDialogResult>;

  const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();

  beforeEach(() => {
    bescheidService = mock(BescheidService);
    bescheidService.getBescheidCreated.mockReturnValue(EMPTY);
    ozgcloudDialogService = mock(OzgcloudDialogService);
    formService = mock(BescheidFormService);
    wizardDialogRef = createDialogRefMock();
  });

  beforeEach(async () => {
    TestBed.overrideComponent(BescheidWizardContainerComponent, {
      set: {
        providers: [
          {
            provide: BescheidFormService,
            useValue: formService,
          },
        ],
      },
    });

    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardContainerComponent,
        MockComponent(BescheidWizardComponent),
        MockComponent(BescheidWizardCancelDialogContainerComponent),
        MockComponent(ButtonComponent),
        MockComponent(IconComponent),
      ],
      providers: [
        {
          provide: DIALOG_DATA,
          useValue: { vorgangWithEingangResource },
        },
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: OzgcloudDialogService,
          useValue: ozgcloudDialogService,
        },
        {
          provide: DialogRef,
          useValue: wizardDialogRef,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('constructor', () => {
      it('should set vorgang with eingang resource from dialog data', () => {
        expect(component.vorgangWithEingangResource).toBe(vorgangWithEingangResource);
      });
    });

    describe('ngOnInit', () => {
      it('should init service', () => {
        component.ngOnInit();

        expect(bescheidService.init).toHaveBeenCalled();
      });

      it('should set vorgang with eingang on form service', () => {
        formService.setVorgangWithEingangResource = jest.fn();

        component.ngOnInit();

        expect(formService.setVorgangWithEingangResource).toHaveBeenCalledWith(vorgangWithEingangResource);
      });

      it('should set active step', () => {
        bescheidService.getActiveStep.mockReturnValue(of(BescheidWizardStep.BescheidVersenden));

        component.ngOnInit();

        expect(component.activeStep$).toBeObservable(singleColdCompleted(BescheidWizardStep.BescheidVersenden));
      });

      it('should subscribe to bescheid resource', () => {
        component.subscribeToBescheidResourceIfExists = jest.fn();

        component.ngOnInit();

        expect(component.subscribeToBescheidResourceIfExists).toHaveBeenCalled();
      });

      it('should subscribe escape key handler', () => {
        component.handleEscapeKey = jest.fn();

        component.ngOnInit();

        expect(component.handleEscapeKey).toHaveBeenCalled();
      });
    });

    describe('subscribeToBescheidResourceIfExists', () => {
      const bescheidDraftStateResource: StateResource<BescheidResource> = createBescheidStateResource();

      beforeEach(() => {
        bescheidService.getBescheidDraft.mockReturnValue(of(bescheidDraftStateResource));
      });

      describe('when bescheid draft link exist', () => {
        beforeEach(() => {
          component.vorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEID_DRAFT]);
        });

        it('should get bescheid draft', () => {
          component.subscribeToBescheidResourceIfExists();

          expect(bescheidService.getBescheidDraft).toHaveBeenCalled();
        });

        it('should emit bescheid draft state resource', () => {
          component.subscribeToBescheidResourceIfExists();

          expect(component.bescheidDraftStateResource$).toBeObservable(singleColdCompleted(bescheidDraftStateResource));
        });
      });

      describe('when bescheid draft link NOT exists', () => {
        beforeEach(() => {
          component.vorgangWithEingangResource = createVorgangWithEingangResource();
        });

        it('should not get bescheid draft', () => {
          component.subscribeToBescheidResourceIfExists();

          expect(bescheidService.getBescheidDraft).not.toHaveBeenCalled();
        });

        it('should emit empty bescheid draft state resource', () => {
          component.subscribeToBescheidResourceIfExists();

          expect(component.bescheidDraftStateResource$).toBeObservable(
            singleColdCompleted(createEmptyStateResource<BescheidResource>()),
          );
        });

        it('should not patch form values', () => {
          formService.patchValues = jest.fn();

          component.subscribeToBescheidResourceIfExists();

          expect(formService.patchValues).not.toHaveBeenCalled();
        });
      });
    });

    describe('isBescheidSuccessfullyCreated', () => {
      it('should return true', () => {
        const createBescheidCommandResource: CommandResource = {
          ...createSuccessfullyDoneCommandResource(),
          order: CommandOrder.CREATE_BESCHEID,
        };
        const isBescheidSuccessfullyCreated: boolean = component.isBescheidSuccessfullyCreated(createBescheidCommandResource);

        expect(isBescheidSuccessfullyCreated).toBeTruthy();
      });

      it('should return false on different order', () => {
        const createBescheidCommandResource: CommandResource = {
          ...createSuccessfullyDoneCommandResource(),
          order: CommandOrder.UPDATE_BESCHEID,
        };
        const isBescheidSuccessfullyCreated: boolean = component.isBescheidSuccessfullyCreated(createBescheidCommandResource);

        expect(isBescheidSuccessfullyCreated).toBeFalsy();
      });

      it('should return false on error command', () => {
        const createBescheidCommandResource: CommandResource = createCommandErrorResource();
        const isBescheidSuccessfullyCreated: boolean = component.isBescheidSuccessfullyCreated(createBescheidCommandResource);

        expect(isBescheidSuccessfullyCreated).toBeFalsy();
      });
    });

    describe('cancelWizard', () => {
      beforeEach(() => {
        component.openCancelDialog = jest.fn();
        component.subscribeToCancelDialogClosed = jest.fn();
      });

      it('should open cancel dialog', () => {
        const bescheidResource: BescheidResource = createBescheidResource();

        component.cancelWizard(bescheidResource);

        expect(component.openCancelDialog).toHaveBeenCalledWith(bescheidResource);
      });

      it('should NOT open cancel dialog', () => {
        component.cancelDialogRef = createDialogRefMock() as any;
        const bescheidResource: BescheidResource = createBescheidResource();

        component.cancelWizard(bescheidResource);

        expect(component.openCancelDialog).not.toHaveBeenCalled();
      });

      it('should handle dialog close', () => {
        const bescheidResource: BescheidResource = createBescheidResource();

        component.cancelWizard(bescheidResource);

        expect(component.subscribeToCancelDialogClosed).toHaveBeenCalled();
      });

      it('should NOT handle dialog close', () => {
        component.cancelDialogRef = createDialogRefMock() as any;
        const bescheidResource: BescheidResource = createBescheidResource();

        component.cancelWizard(bescheidResource);

        expect(component.subscribeToCancelDialogClosed).not.toHaveBeenCalled();
      });
    });

    describe('openCancelDialog', () => {
      it('should show close dialog', () => {
        const bescheidResource: BescheidResource = createBescheidResource();
        ozgcloudDialogService.openInCallingComponentContext.mockReturnValue({
          closed: EMPTY,
        });

        component.openCancelDialog(bescheidResource);

        expect(ozgcloudDialogService.openInCallingComponentContext).toHaveBeenCalledWith(
          BescheidWizardCancelDialogContainerComponent,
          component.viewContainerRef,
          {
            bescheidResource,
          } as CancelWizardDialogData,
        );
      });

      it('should set cancelDialogRef', () => {
        const bescheidResource: BescheidResource = createBescheidResource();
        ozgcloudDialogService.openInCallingComponentContext.mockReturnValue({
          closed: EMPTY,
        });

        component.openCancelDialog(bescheidResource);

        expect(component.cancelDialogRef).toBeDefined();
      });
    });

    describe('subscribeToCancelDialogClosed', () => {
      let cancelDialogResult: CancelWizardDialogResult;

      beforeEach(() => {
        cancelDialogResult = { closeWizard: true };
        component.cancelDialogRef = { ...createDialogRefMock(), closed: of(cancelDialogResult) } as any;
        component.handleCancelDialogClosed = jest.fn();
      });

      it('should handle cancel dialog closed', () => {
        component.subscribeToCancelDialogClosed();

        expect(component.handleCancelDialogClosed).toHaveBeenCalledWith(cancelDialogResult);
      });
    });

    describe('handleCancelDialogClosed', () => {
      it('should unset cancelDialogRef', () => {
        component.cancelDialogRef = useFromMock(mock(DialogRef<CancelWizardDialogResult>));

        component.handleCancelDialogClosed({ closeWizard: true });

        expect(component.cancelDialogRef).toBeNull();
      });

      it('should close wizard', () => {
        component.handleCancelDialogClosed({ closeWizard: true });

        expect(wizardDialogRef.close).toHaveBeenCalledWith({ reloadVorgang: true });
      });

      it('should NOT close wizard', () => {
        component.handleCancelDialogClosed({ closeWizard: false });

        expect(wizardDialogRef.close).not.toHaveBeenCalled();
      });

      it('should NOT close wizard on empty result', () => {
        component.handleCancelDialogClosed(null);

        expect(wizardDialogRef.close).not.toHaveBeenCalled();
      });
    });

    describe('handleEscapeKey', () => {
      beforeEach(() => {
        component.cancelWizard = jest.fn();
      });

      it('should close wizard if bescheid resource loaded', () => {
        const bescheidResource: BescheidResource = createBescheidResource();
        component.bescheidDraftStateResource$ = of(createStateResource(bescheidResource));
        wizardDialogRef.keydownEvents = of(createKeydownKeyboardEvent(ESCAPE_KEY));

        component.handleEscapeKey();

        expect(component.cancelWizard).toHaveBeenCalledWith(bescheidResource);
      });

      it('should close wizard if bescheid resource not exists', () => {
        component.bescheidDraftStateResource$ = of(createEmptyStateResource<BescheidResource>());
        wizardDialogRef.keydownEvents = of(createKeydownKeyboardEvent(ESCAPE_KEY));

        component.handleEscapeKey();

        expect(component.cancelWizard).toHaveBeenCalledWith(null);
      });

      it('should not close if no escape key pressed', () => {
        const bescheidResource: BescheidResource = createBescheidResource();
        component.bescheidDraftStateResource$ = of(createStateResource(bescheidResource));
        wizardDialogRef.keydownEvents = of(createKeydownKeyboardEvent('a'));

        component.handleEscapeKey();

        expect(component.cancelWizard).not.toHaveBeenCalled();
      });

      it('should not close if bescheid resource loading', () => {
        const bescheidResource: BescheidResource = createBescheidResource();
        component.bescheidDraftStateResource$ = of(createStateResource(bescheidResource, true));
        wizardDialogRef.keydownEvents = of(createKeydownKeyboardEvent(ESCAPE_KEY));

        component.handleEscapeKey();

        expect(component.cancelWizard).not.toHaveBeenCalled();
      });
    });

    describe('onVorgangAbgeschlossen', () => {
      it('should close wizard', () => {
        component.onVorgangAbgeschlossen();

        expect(wizardDialogRef.close).toHaveBeenCalled();
      });
    });
  });

  describe('template', () => {
    describe('wizard', () => {
      function getElementComponent(): BescheidWizardComponent {
        return getElementFromFixtureByType(fixture, BescheidWizardComponent);
      }

      it('should have been called with inputs', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource();
        bescheidService.getActiveStep.mockReturnValue(of(BescheidWizardStep.DokumenteHochladen));
        const bescheidStateResource: StateResource<BescheidResource> = createBescheidStateResource();
        component.bescheidDraftStateResource$ = of(bescheidStateResource);
        component.ngOnInit();

        fixture.detectChanges();

        expect(getElementComponent().vorgangWithEingangResource).toEqual(component.vorgangWithEingangResource);
        expect(getElementComponent().activeStep).toEqual(2);
        expect(getElementComponent().bescheidStateResource).toEqual(bescheidStateResource);
      });

      describe('output', () => {
        describe('vorgangAbgeschlossen', () => {
          it('should call onVorgangAbgeschlossen', () => {
            component.onVorgangAbgeschlossen = jest.fn();

            triggerEvent({
              fixture,
              name: 'vorgangAbgeschlossen',
              elementSelector: bescheidWizard,
            });

            expect(component.onVorgangAbgeschlossen).toHaveBeenCalled();
          });
        });
      });
    });

    describe('close button', () => {
      it('should exists', () => {
        existsAsHtmlElement(fixture, closeButton);
      });

      describe('output', () => {
        describe('clickEmitter', () => {
          it('should call cancelWizard', () => {
            component.cancelWizard = jest.fn();

            triggerEvent({
              fixture,
              name: 'clickEmitter',
              elementSelector: closeButton,
            });

            expect(component.cancelWizard).toHaveBeenCalled();
          });
        });
      });
    });
  });
});
