/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidLinkRel, BescheidService } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { createEmptyStateResource, HasLinkPipe, StateResource } from '@alfa-client/tech-shared';
import {
  existsAsHtmlElement,
  getElementFromFixtureByType,
  mock,
  notExistsAsHtmlElement,
  triggerEvent,
  useFromMock,
} from '@alfa-client/test-utils';
import { DateEditorComponent } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule, UntypedFormBuilder } from '@angular/forms';
import { IconComponent, RadioButtonCardComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createBescheidResource } from '../../../../../../../bescheid-shared/src/test/bescheid';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { singleColdCompleted } from '../../../../../../../tech-shared/test/marbles';
import { createVorgangWithEingangResource } from '../../../../../../../vorgang-shared/test/vorgang';
import { BescheidFormService } from '../../../bescheid.formservice';
import { BescheidWizardWeiterButtonComponent } from '../../weiter-button/bescheid-wizard-weiter-button.component';
import { BescheidWizardAntragBescheidenFormComponent } from './bescheid-wizard-antrag-bescheiden-form.component';

describe('BescheidWizardAntragBescheidenFormComponent', () => {
  let component: BescheidWizardAntragBescheidenFormComponent;
  let fixture: ComponentFixture<BescheidWizardAntragBescheidenFormComponent>;

  const weiterButton: string = getDataTestIdOf('weiter-button');

  let formService: BescheidFormService;

  beforeEach(async () => {
    formService = new BescheidFormService(new UntypedFormBuilder(), useFromMock(mock(BescheidService)));
    formService.submit = jest.fn().mockReturnValue(EMPTY);

    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardAntragBescheidenFormComponent,
        HasLinkPipe,
        MockComponent(BescheidWizardWeiterButtonComponent),
        MockComponent(RadioButtonCardComponent),
        MockComponent(IconComponent),
        MockComponent(DateEditorComponent),
      ],
      providers: [
        {
          provide: BescheidFormService,
          useValue: formService,
        },
      ],
      imports: [ReactiveFormsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardAntragBescheidenFormComponent);
    component = fixture.componentInstance;
    component.weiterClickEmitter = useFromMock(mock(EventEmitter<void>));
    fixture.detectChanges();
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it('should set initial values', () => {
      expect(component.submitStateResource$).toBeObservable(singleColdCompleted(createEmptyStateResource()));
    });

    describe('submit', () => {
      it('should call form service', () => {
        component.submit();

        expect(formService.submit).toHaveBeenCalled();
      });

      it('should emit weiter click', () => {
        formService.submit = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        component.submit();
        component.submitStateResource$.subscribe();

        expect(component.weiterClickEmitter.emit).toHaveBeenCalled();
      });
    });
  });

  describe('template', () => {
    describe('weiter button', () => {
      it('should exists if bescheid update link exists', () => {
        component.bescheidResource = createBescheidResource([BescheidLinkRel.UPDATE]);

        fixture.detectChanges();

        existsAsHtmlElement(fixture, weiterButton);
      });

      it('should exists if create link exists', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
        ]);

        fixture.detectChanges();

        existsAsHtmlElement(fixture, weiterButton);
      });

      it('should NOT exists if update and create links missing', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource();
        component.bescheidResource = createBescheidResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, weiterButton);
      });

      it('should NOT exists if update link missing', () => {
        component.bescheidResource = createBescheidResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, weiterButton);
      });

      it('should have been called with inputs', (done) => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
        ]);

        fixture.detectChanges();

        const weiterButtonComponent: BescheidWizardWeiterButtonComponent = getElementFromFixtureByType(
          fixture,
          BescheidWizardWeiterButtonComponent,
        );

        component.submitStateResource$.subscribe((submitStateResource: StateResource<CommandResource>) => {
          expect(weiterButtonComponent.submitStateResource).toEqual(submitStateResource);
          done();
        });
      });

      describe('output', () => {
        describe('clickEmitter', () => {
          it('should submit', () => {
            component.submit = jest.fn();
            component.bescheidResource = createBescheidResource([BescheidLinkRel.UPDATE]);

            fixture.detectChanges();

            triggerEvent({ fixture, name: 'clickEmitter', elementSelector: weiterButton });

            expect(component.submit).toHaveBeenCalled();
          });
        });
      });
    });
  });
});
