/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidService } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { createErrorStateResource, StateResource } from '@alfa-client/tech-shared';
import {
  createDialogRefMock,
  DialogRefMock,
  existsAsHtmlElement,
  getElementComponentFromFixtureByCss,
  Mock,
  mock,
  triggerEvent,
} from '@alfa-client/test-utils';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { expect } from '@jest/globals';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { IconComponent } from '@ods/system';
import { createBescheidResource } from 'libs/bescheid-shared/src/test/bescheid';
import { createCommandStateResource, createSuccessfullyDoneCommandStateResource } from 'libs/command-shared/test/command';
import { getDataTestIdAttributeOf } from 'libs/tech-shared/test/data-test';
import { createApiError } from 'libs/tech-shared/test/error';
import { singleColdCompleted } from 'libs/tech-shared/test/marbles';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { BescheidFormService } from '../../bescheid.formservice';
import {
  BescheidWizardCancelDialogContainerComponent,
  CancelWizardDialogData,
} from './bescheid-wizard-cancel-dialog-container.component';

describe('BescheidWizardCancelDialogContainerComponent', () => {
  let component: BescheidWizardCancelDialogContainerComponent;
  let fixture: ComponentFixture<BescheidWizardCancelDialogContainerComponent>;

  const speichernButton: string = getDataTestIdAttributeOf('bescheiderstellung-abbrechen-entwurf-speichern');
  const verwerfenButton: string = getDataTestIdAttributeOf('bescheiderstellung-abbrechen-entwurf-verwerfen');

  const dialogData: CancelWizardDialogData = {
    bescheidResource: createBescheidResource(),
  };

  let formService: Mock<BescheidFormService>;
  let bescheidService: Mock<BescheidService>;
  let dialogRef: DialogRefMock;

  beforeEach(() => {
    formService = mock(BescheidFormService);
    formService.submit = jest.fn().mockReturnValue(EMPTY);
    bescheidService = mock(BescheidService);
    bescheidService.deleteBescheid.mockReturnValue(EMPTY);
    dialogRef = createDialogRefMock();
  });

  async function configureTestingModule(dialogData: CancelWizardDialogData) {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardCancelDialogContainerComponent,
        MockComponent(ButtonWithSpinnerComponent),
        MockComponent(IconComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: BescheidFormService,
          useValue: formService,
        },
        {
          provide: DIALOG_DATA,
          useValue: dialogData,
        },
        {
          provide: DialogRef,
          useValue: dialogRef,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardCancelDialogContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  }

  beforeEach(async () => {
    await configureTestingModule(dialogData);
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    describe('ngOnInit', () => {
      it('should lock bescheid sending', () => {
        component.ngOnInit();

        expect(bescheidService.lockBescheidSending).toHaveBeenCalled();
      });
    });

    describe('ngOnDestroy', () => {
      it('should unlock bescheid sending', () => {
        component.ngOnDestroy();

        expect(bescheidService.unlockBescheidSending).toHaveBeenCalled();
      });
    });

    describe('save', () => {
      beforeEach(() => {
        component.closeDialogWithWizard = jest.fn();
      });

      it('should submit form', () => {
        component.save();

        expect(formService.submit).toHaveBeenCalled();
      });

      it('should set save state resource', () => {
        const commandStateResource: StateResource<CommandResource> = createCommandStateResource();
        formService.submit = jest.fn().mockReturnValue(of(commandStateResource));

        component.save();

        expect(component.saveStateResource$).toBeObservable(singleColdCompleted(commandStateResource));
      });

      it('should close dialog with wizard', () => {
        const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        formService.submit = jest.fn().mockReturnValue(of(commandStateResource));

        component.save();
        component.saveStateResource$.subscribe();

        expect(component.closeDialogWithWizard).toHaveBeenCalled();
      });

      it('should NOT close dialog with wizard', () => {
        const commandStateResource: StateResource<CommandResource> = createErrorStateResource(createApiError());
        formService.submit = jest.fn().mockReturnValue(of(commandStateResource));

        component.save();
        component.saveStateResource$.subscribe();

        expect(component.closeDialogWithWizard).not.toHaveBeenCalled();
      });
    });

    describe('cancel', () => {
      beforeEach(() => {
        component.closeDialogWithWizard = jest.fn();
      });

      it('should delete bescheid', () => {
        component.cancel();

        expect(bescheidService.deleteBescheid).toHaveBeenCalled();
      });

      it('should NOT delete bescheid', async () => {
        await configureTestingModule({} as CancelWizardDialogData);

        component.cancel();

        expect(bescheidService.deleteBescheid).not.toHaveBeenCalled();
      });

      it('should set delete command state resource', () => {
        const commandStateResource: StateResource<CommandResource> = createCommandStateResource();
        bescheidService.deleteBescheid = jest.fn().mockReturnValue(of(commandStateResource));

        component.cancel();

        expect(component.deleteStateResource$).toBeObservable(singleColdCompleted(commandStateResource));
      });

      it('should close dialog on success', () => {
        const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        bescheidService.deleteBescheid = jest.fn().mockReturnValue(of(commandStateResource));

        component.cancel();
        component.deleteStateResource$.subscribe();

        expect(component.closeDialogWithWizard).toHaveBeenCalled();
      });

      it('should NOT close dialog on error', () => {
        const commandStateResource: StateResource<CommandResource> = createErrorStateResource(createApiError());
        bescheidService.deleteBescheid = jest.fn().mockReturnValue(of(commandStateResource));

        component.cancel();
        component.deleteStateResource$.subscribe();

        expect(component.closeDialogWithWizard).not.toHaveBeenCalled();
      });

      it('should close dialog with cancel result on non existing bescheid', async () => {
        await configureTestingModule({} as CancelWizardDialogData);
        component.closeDialogWithWizard = jest.fn();

        component.cancel();

        expect(component.closeDialogWithWizard).toHaveBeenCalled();
      });
    });

    describe('closeDialogWithWizard', () => {
      it('should close', () => {
        component.closeDialogWithWizard();

        expect(dialogRef.close).toHaveBeenCalledWith({ closeWizard: true });
      });
    });
  });

  describe('template', () => {
    describe('save button', () => {
      it('should exists', () => {
        existsAsHtmlElement(fixture, speichernButton);
      });

      it('should have been called with inputs', () => {
        const commandStateResource: StateResource<CommandResource> = createCommandStateResource();
        component.saveStateResource$ = of(commandStateResource);

        const elementComponent: ButtonWithSpinnerComponent = getElementComponentFromFixtureByCss(fixture, speichernButton);

        fixture.detectChanges();

        expect(elementComponent.stateResource).toBe(commandStateResource);
      });

      describe('output', () => {
        it('should call save', () => {
          component.save = jest.fn();

          triggerEvent({ fixture, name: 'clickEmitter', elementSelector: speichernButton });

          expect(component.save).toHaveBeenCalled();
        });
      });
    });

    describe('delete button', () => {
      it('should exists', () => {
        existsAsHtmlElement(fixture, verwerfenButton);
      });

      it('should have been called with inputs', () => {
        const commandStateResource: StateResource<CommandResource> = createCommandStateResource();
        component.deleteStateResource$ = of(commandStateResource);

        const elementComponent: ButtonWithSpinnerComponent = getElementComponentFromFixtureByCss(fixture, verwerfenButton);

        fixture.detectChanges();

        expect(elementComponent.stateResource).toBe(commandStateResource);
      });

      describe('output', () => {
        it('should call delete', () => {
          component.cancel = jest.fn();

          triggerEvent({ fixture, name: 'clickEmitter', elementSelector: verwerfenButton });

          expect(component.cancel).toHaveBeenCalled();
        });
      });
    });
  });
});
