/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Bescheid, BescheidLinkRel, BescheidResource, BescheidSendBy, BescheidService, BescheidWizardStep, Wizard, } from '@alfa-client/bescheid-shared';
import { CommandResource, switchMapCommandSuccessfullyDone } from '@alfa-client/command-shared';
import { AbstractFormService, convertToBoolean, formatForDatabase, isNotNil, StateResource } from '@alfa-client/tech-shared';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Injectable } from '@angular/core';
import { FormControl, UntypedFormArray, UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { getUrl, hasLink, ResourceUri } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { first, Observable, startWith, switchMap } from 'rxjs';

@Injectable()
export class BescheidFormService extends AbstractFormService<CommandResource> {
  static readonly FIELD_BESCHIEDEN_AM = 'beschiedenAm';
  static readonly FIELD_BEWILLIGT = 'bewilligt';
  static readonly FIELD_BESCHEID_DOCUMENT = 'bescheidDocument';
  static readonly FIELD_ATTACHMENTS = 'attachments';
  public static readonly FIELD_SEND_BY = 'sendBy';
  static readonly FIELD_NACHRICHT_SUBJECT = 'nachrichtSubject';
  static readonly FIELD_NACHRICHT_TEXT = 'nachrichtText';

  static readonly FIELD_PATH_PREFIX = 'command.body';

  private vorgangWithEingangResource: VorgangWithEingangResource;
  private bescheidResource: BescheidResource;

  constructor(
    formBuilder: UntypedFormBuilder,
    private readonly bescheidService: BescheidService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [BescheidFormService.FIELD_BESCHIEDEN_AM]: new UntypedFormControl(new Date(Date.now())),
      [BescheidFormService.FIELD_BEWILLIGT]: new UntypedFormControl('true'),
      [BescheidFormService.FIELD_SEND_BY]: new UntypedFormControl(BescheidSendBy.NACHRICHT),
      [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: new UntypedFormControl(null),
      [BescheidFormService.FIELD_ATTACHMENTS]: new UntypedFormArray([]),
      [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: new UntypedFormControl(''),
      [BescheidFormService.FIELD_NACHRICHT_TEXT]: new UntypedFormControl(''),
    });
  }

  protected doSubmit(): Observable<StateResource<CommandResource>> {
    return this.bescheidService.getWizard().pipe(
      first(),
      switchMap((wizard: Wizard) => this._submitBescheid(wizard)),
    );
  }

  _submitBescheid(wizard: Wizard): Observable<StateResource<CommandResource>> {
    if (this.isPatch() || isNotNil(this.bescheidResource)) {
      if (this._canSendBescheid(wizard)) {
        return this._updateAndSendBescheid(wizard.sendBy);
      }
      return this._updateBescheid();
    }
    return this._createBescheid();
  }

  _canSendBescheid(wizard: Wizard): boolean {
    return wizard.activeStep === BescheidWizardStep.BescheidVersenden && wizard.canBeSend;
  }

  _createBescheid(): Observable<StateResource<CommandResource>> {
    return this.bescheidService.createBescheid(this.vorgangWithEingangResource, this.getBescheidFormValue());
  }

  _updateBescheid(): Observable<StateResource<CommandResource>> {
    return this.bescheidService.updateBescheid(this.bescheidResource, this.getBescheidFormValue());
  }

  _updateAndSendBescheid(sendBy: BescheidSendBy): Observable<StateResource<CommandResource>> {
    if (sendBy === BescheidSendBy.MANUAL) {
      return this._updateBescheid().pipe(switchMapCommandSuccessfullyDone(() => this.bescheidService.sendBescheidManually()));
    }
    if (sendBy === BescheidSendBy.NACHRICHT) {
      return this._updateBescheid().pipe(switchMapCommandSuccessfullyDone(() => this.bescheidService.sendBescheidMessage()));
    }
  }

  protected getPathPrefix(): string {
    return BescheidFormService.FIELD_PATH_PREFIX;
  }

  public patchValues(bescheidResource: BescheidResource): void {
    this.bescheidResource = bescheidResource;
    const bescheidDocumentUri: ResourceUri = this.getBescheidDocumentUri(bescheidResource);
    const sendBy: BescheidSendBy = this._evaluateSendBy(bescheidResource);
    this.bescheidService.setSendBy(sendBy);
    this.patch({
      [BescheidFormService.FIELD_BESCHIEDEN_AM]: bescheidResource.beschiedenAm,
      [BescheidFormService.FIELD_BEWILLIGT]: String(bescheidResource.bewilligt),
      [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: bescheidDocumentUri,
      [BescheidFormService.FIELD_SEND_BY]: sendBy,
      [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: bescheidResource.nachrichtSubject,
      [BescheidFormService.FIELD_NACHRICHT_TEXT]: bescheidResource.nachrichtText,
    });
    (this.form.controls[BescheidFormService.FIELD_ATTACHMENTS] as UntypedFormArray).clear();
    bescheidResource.attachments.forEach((attachmentLink) =>
      (this.form.controls[BescheidFormService.FIELD_ATTACHMENTS] as UntypedFormArray).push(
        new UntypedFormControl(attachmentLink),
      ),
    );
  }

  private getBescheidDocumentUri(bescheid: BescheidResource): ResourceUri | null {
    if (hasLink(bescheid, BescheidLinkRel.BESCHEID_DOCUMENT)) {
      return getUrl(bescheid, BescheidLinkRel.BESCHEID_DOCUMENT);
    }
    return null;
  }

  _evaluateSendBy(bescheidResource: BescheidResource): BescheidSendBy {
    if (
      bescheidResource.sendBy === BescheidSendBy.NACHRICHT &&
      hasLink(bescheidResource, BescheidLinkRel.BESCHEIDEN_UND_SENDEN)
    ) {
      return BescheidSendBy.NACHRICHT;
    }
    return BescheidSendBy.MANUAL;
  }

  public getBescheidFormValueChanges(): Observable<Bescheid> {
    return this.form.valueChanges.pipe(startWith(this.getBescheidFormValue()));
  }

  public getBescheidFormValue(): Bescheid {
    const value: any = this.getFormValue();
    return {
      ...value,
      beschiedenAm: formatForDatabase(value.beschiedenAm),
      bewilligt: convertToBoolean(value.bewilligt),
      attachments: value.attachments,
    };
  }

  public setVorgangWithEingangResource(vorgangWithEingangResource: VorgangWithEingangResource): void {
    this.vorgangWithEingangResource = vorgangWithEingangResource;
  }

  public clearBescheidDocumentFile(): void {
    this.updateBescheidDocumentFile(null);
  }

  public updateBescheidDocumentFile(uri?: ResourceUri): void {
    if (isNil(uri)) {
      this.getBescheidDocumentControl().patchValue(null);
    } else {
      this.getBescheidDocumentControl().patchValue(uri);
    }
  }

  private getBescheidDocumentControl(): FormControl {
    return <FormControl>this.form.controls[BescheidFormService.FIELD_BESCHEID_DOCUMENT];
  }

  public isBetreffInvalid(): boolean {
    return this.form.controls[BescheidFormService.FIELD_NACHRICHT_SUBJECT].invalid;
  }

  public isNachrichtInvalid(): boolean {
    return this.form.controls[BescheidFormService.FIELD_NACHRICHT_TEXT].invalid;
  }
}
