/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  ApiError,
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
  LinkRelationName,
} from '@alfa-client/tech-shared';
import { HttpErrorResponse } from '@angular/common/http';
import { faker } from '@faker-js/faker';
import { Action } from '@ngrx/store';
import { getUrl, Resource, ResourceUri } from '@ngxp/rest';
import { createApiError, createHttpErrorResponse } from '../../../../tech-shared/test/error';
import { createDummyResource } from '../../../../tech-shared/test/resource';
import { createCommandResource, createCreateCommand, createCreateCommandProps } from '../../../test/command';
import { CommandResource, CreateCommand, CreateCommandProps } from '../command.model';
import { buildCommandStateKey, CommandState, initialState, reducer } from './command.reducer';

import * as CommandActions from '../+state/command.actions';

describe('Command Reducer', () => {
  const linkRel: LinkRelationName = faker.string.alpha(6);
  const resource: Resource = createDummyResource([linkRel]);
  const uri: ResourceUri = getUrl(resource, linkRel);
  const commandResource: CommandResource = createCommandResource();
  const createCommandProps: CreateCommandProps = { ...createCreateCommandProps(), resource, linkRel };

  describe('unknown action', () => {
    it('should return current state', () => {
      const action: Action = {} as Action;

      const result: any = reducer(initialState, action);

      expect(result).toBe(initialState);
    });
  });

  describe('createCommand', () => {
    const command: CreateCommand = createCreateCommand();

    it('should create empty loading map entry', () => {
      const action: Action<string> = CommandActions.createCommand({ resource, linkRel, command });

      const state: CommandState = reducer(initialState, action);

      expect(state.commandMap[buildCommandStateKey(command.order, uri)]).toEqual(createEmptyStateResource(true));
    });
  });

  describe('createCommandSuccess', () => {
    it('should create stateResource entry by loaded resource', () => {
      const action: Action<string> = CommandActions.createCommandSuccess({
        createCommandProps,
        command: commandResource,
      });

      const state: CommandState = reducer(initialState, action);

      expect(state.commandMap[buildCommandStateKey(createCommandProps.command.order, uri)]).toEqual(
        createStateResource(commandResource),
      );
    });
  });

  describe('createCommandFailure', () => {
    const error: ApiError = createApiError();
    const httpErrorResponse: HttpErrorResponse = { ...createHttpErrorResponse(), error };

    it('should create errorStateResource entry by occured error', () => {
      const action: Action<string> = CommandActions.createCommandFailure({
        createCommandProps,
        command: commandResource,
        error: httpErrorResponse,
      });

      const state: CommandState = reducer(initialState, action);

      expect(state.commandMap[buildCommandStateKey(createCommandProps.command.order, uri)]).toEqual(
        createErrorStateResource(error),
      );
    });
  });
});
