/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  EMPTY_STRING,
  LinkRelationName,
  ResourceServiceConfig,
  SingleResourceStateService,
  StateResource,
  StateService,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { Resource } from '@ngxp/rest';
import { Observable, of, take } from 'rxjs';
import { createCommandResource, createCreateCommandProps, createDoneCommandResource } from '../../../command-shared/test/command';
import { multipleCold, singleCold } from '../../../tech-shared/test/marbles';
import { createDummyResource } from '../../../tech-shared/test/resource';
import { CommandResourceService } from './command-resource.service';
import { CommandLinkRel } from './command.linkrel';
import { CommandResource, CreateCommandProps, DeleteCommandProps, SaveCommandProps } from './command.model';
import { CommandService } from './command.service';
import { getEffectedResourceUrl } from './command.util';

describe('CommandResourceService', () => {
  let service: CommandResourceService<Resource, Resource>;
  let config: ResourceServiceConfig<Resource>;
  let stateService: Mock<StateService>;
  let commandService: Mock<CommandService>;

  const getLinkRel: LinkRelationName = 'dummyGetLinkRel';

  const editLinkRel: string = 'dummyEditLinkRel';
  const editOrder: string = 'dummyEditOrder';

  const deleteOrder: string = 'dummyDeleteOrder';
  const deleteLinkRel: string = 'dummyDeleteLinkRel';

  const dummyResource: Resource = createDummyResource();
  const dummyStateResource: StateResource<Resource> = createStateResource(dummyResource);

  const configResource: Resource = createDummyResource([getLinkRel, editLinkRel]);
  const configStateResource: StateResource<Resource> = createStateResource(configResource);
  const configStateResource$: Observable<StateResource<Resource>> = of(configStateResource);

  beforeEach(() => {
    config = {
      stateInfo: { name: 'dummyStateName', path: 'dummyStatePath' },
      baseResource: configStateResource$,
      getLinkRel,
      edit: { order: editOrder, linkRel: editLinkRel },
      delete: { order: deleteOrder, linkRel: deleteLinkRel },
    };
    stateService = mock(StateService);
    stateService.createSingleResourceService = jest.fn().mockReturnValue(mock(SingleResourceStateService));
    commandService = mock(CommandService);

    service = new CommandResourceService(config, useFromMock(stateService), useFromMock(commandService));
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('save', () => {
    const FIRST_EMIT_AFTER_INITIAL_START_EMIT: number = 2;

    const dummyToSave: unknown = {};

    const commandResource: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);
    const commandStateResource: StateResource<CommandResource> = createStateResource(commandResource);

    beforeEach(() => {
      service.doSave = jest.fn().mockReturnValue(of(commandStateResource));
      service.resourceStateService.selectResource = jest.fn().mockReturnValue(of(dummyStateResource));
      service.loadByResourceUri = jest.fn();
    });

    it('should do save', () => {
      service.save(dummyToSave).pipe(take(FIRST_EMIT_AFTER_INITIAL_START_EMIT)).subscribe();

      expect(service.doSave).toHaveBeenCalledWith(dummyResource, dummyToSave, {
        snackBarMessage: EMPTY_STRING,
      });
    });

    it('should loadByResourceUri on sucessfully done command', () => {
      service.save(dummyToSave).pipe(take(FIRST_EMIT_AFTER_INITIAL_START_EMIT)).subscribe();

      expect(service.loadByResourceUri).toHaveBeenCalledWith(getEffectedResourceUrl(commandResource));
    });

    it('should return initial value', () => {
      service.doSave = jest.fn().mockReturnValue(singleCold(commandStateResource, '-a'));

      const initialResponse$: Observable<StateResource<CommandResource>> = service.save(dummyToSave);

      expect(initialResponse$).toBeObservable(multipleCold(createEmptyStateResource(true), commandStateResource));
    });
  });

  describe('do save', () => {
    const dummyToSave: unknown = {};
    const dummyResource: Resource = createDummyResource();
    const saveCommandProps: SaveCommandProps = { snackBarMessage: EMPTY_STRING };
    const createCommandProps: CreateCommandProps = createCreateCommandProps();

    let buildSaveCommandPropsSpy: jest.SpyInstance;

    beforeEach(() => {
      buildSaveCommandPropsSpy = service.buildSaveCommandProps = jest.fn().mockReturnValue(createCommandProps);
    });

    it('should call command service', () => {
      service.doSave(dummyResource, dummyToSave, saveCommandProps);

      expect(commandService.createCommandByProps).toHaveBeenCalledWith(createCommandProps);
    });

    it('should build command', () => {
      service.doSave(dummyResource, dummyToSave, saveCommandProps);

      expect(buildSaveCommandPropsSpy).toHaveBeenCalledWith(dummyResource, dummyToSave, saveCommandProps);
    });
  });

  describe('build save command props', () => {
    const dummyResource: Resource = createDummyResource();
    const dummyToSave: unknown = {};
    const saveCommandProps: SaveCommandProps = { snackBarMessage: EMPTY_STRING };

    it('should have resource', () => {
      const props: CreateCommandProps = buildSaveCommandProps();

      expect(props.resource).toBe(dummyResource);
    });

    it('should have linkRel', () => {
      const props: CreateCommandProps = buildSaveCommandProps();

      expect(props.linkRel).toBe(editLinkRel);
    });

    describe('command', () => {
      it('should have order', () => {
        const props: CreateCommandProps = buildSaveCommandProps();

        expect(props.command.order).toBe(editOrder);
      });

      it('should have body', () => {
        const props: CreateCommandProps = buildSaveCommandProps();

        expect(props.command.body).toBe(dummyToSave);
      });
    });

    it('should have snackBarMessage', () => {
      const props: CreateCommandProps = buildSaveCommandProps();

      expect(props.snackBarMessage).toBe(saveCommandProps.snackBarMessage);
    });

    function buildSaveCommandProps(): CreateCommandProps {
      return service.buildSaveCommandProps(dummyResource, dummyToSave, saveCommandProps);
    }
  });

  describe('doPatch', () => {
    it('should throw error', () => {
      expect(() => service.doPatch(configResource, {})).toThrow('Method not implemented.');
    });
  });

  describe('delete', () => {
    const FIRST_EMIT_AFTER_INITIAL_START_EMIT: number = 2;

    const commandResource: CommandResource = createDoneCommandResource();
    const commandStateResource: StateResource<CommandResource> = createStateResource(commandResource);

    beforeEach(() => {
      service.doDelete = jest.fn().mockReturnValue(of(commandStateResource));
      service.resourceStateService.selectResource = jest.fn().mockReturnValue(of(dummyStateResource));
      service.clearResource = jest.fn();
    });

    it('should do delete', () => {
      service.delete().pipe(take(FIRST_EMIT_AFTER_INITIAL_START_EMIT)).subscribe();

      expect(service.doDelete).toHaveBeenCalledWith(dummyResource, {
        snackBarMessage: EMPTY_STRING,
      });
    });

    it('should clear resource on sucessfully done command', () => {
      service.delete().pipe(take(FIRST_EMIT_AFTER_INITIAL_START_EMIT)).subscribe();

      expect(service.clearResource).toHaveBeenCalled();
    });

    it('should return initial value', () => {
      service.doDelete = jest.fn().mockReturnValue(singleCold(commandStateResource, '-a'));

      const initialResponse$: Observable<StateResource<CommandResource>> = service.delete();

      expect(initialResponse$).toBeObservable(multipleCold(createEmptyStateResource(true), commandStateResource));
    });
  });

  describe('do delete', () => {
    const dummyResource: Resource = createDummyResource();
    const deleteCommandProps: DeleteCommandProps = { snackBarMessage: EMPTY_STRING };
    const createCommandProps: CreateCommandProps = createCreateCommandProps();

    let buildDeleteCommandPropsSpy: jest.SpyInstance;

    beforeEach(() => {
      buildDeleteCommandPropsSpy = service.buildDeleteCommandProps = jest.fn().mockReturnValue(createCommandProps);
    });

    it('should call command service', () => {
      service.doDelete(dummyResource, deleteCommandProps);

      expect(commandService.createCommandByProps).toHaveBeenCalledWith(createCommandProps);
    });

    it('should build command', () => {
      service.doDelete(dummyResource, deleteCommandProps);

      expect(buildDeleteCommandPropsSpy).toHaveBeenCalledWith(dummyResource, deleteCommandProps);
    });
  });

  describe('build delete command props', () => {
    const resource: Resource = createDummyResource();
    const deleteCommandProps: SaveCommandProps = { snackBarMessage: EMPTY_STRING };

    it('should have resource', () => {
      const props: CreateCommandProps = service.buildDeleteCommandProps(resource, deleteCommandProps);

      expect(props.resource).toBe(resource);
    });

    it('should have linkRel', () => {
      const props: CreateCommandProps = service.buildDeleteCommandProps(resource, deleteCommandProps);

      expect(props.linkRel).toBe(deleteLinkRel);
    });

    describe('command', () => {
      it('should have order', () => {
        const props: CreateCommandProps = service.buildDeleteCommandProps(resource, deleteCommandProps);

        expect(props.command.order).toBe(deleteOrder);
      });

      it('should have body', () => {
        const props: CreateCommandProps = service.buildDeleteCommandProps(resource, deleteCommandProps);

        expect(props.command.body).toBeNull();
      });
    });

    it('should have snackBarMessage', () => {
      const props: CreateCommandProps = service.buildDeleteCommandProps(resource, deleteCommandProps);

      expect(props.snackBarMessage).toBe(deleteCommandProps.snackBarMessage);
    });
  });
});
