/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  createEmptyStateResource,
  EMPTY_STRING,
  isStateResoureStable,
  ResourceService,
  ResourceServiceConfig,
  StateResource,
  StateService,
} from '@alfa-client/tech-shared';
import { Resource } from '@ngxp/rest';
import { filter, first, Observable, startWith, switchMap } from 'rxjs';
import { CommandResource, CreateCommandProps, DeleteCommandProps, SaveCommandProps } from './command.model';
import { tapOnCommandSuccessfullyDone } from './command.rxjs.operator';
import { CommandService } from './command.service';
import { getEffectedResourceUrl } from './command.util';

export class CommandResourceService<B extends Resource, T extends Resource> extends ResourceService<B, T> {
  constructor(
    protected config: ResourceServiceConfig<B>,
    protected stateService: StateService,
    private commandService: CommandService,
  ) {
    super(config, stateService);
  }

  public save(
    toSave: unknown,
    saveCommandProps: SaveCommandProps = { snackBarMessage: EMPTY_STRING },
  ): Observable<StateResource<CommandResource>> {
    return this.selectResource().pipe(
      first(),
      switchMap((stateResource: StateResource<T>) => this.doSave(stateResource.resource, toSave, saveCommandProps)),
      tapOnCommandSuccessfullyDone((commandStateResource: StateResource<CommandResource>) =>
        this.loadByResourceUri(getEffectedResourceUrl(commandStateResource.resource)),
      ),
      filter(isStateResoureStable),
      startWith(createEmptyStateResource<CommandResource>(true)),
    );
  }

  doSave(resource: T, toSave: unknown, saveCommandProps: SaveCommandProps): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommandByProps(this.buildSaveCommandProps<T>(resource, toSave, saveCommandProps));
  }

  buildSaveCommandProps<T extends Resource>(
    resource: T,
    toSave: unknown,
    saveCommandProps: SaveCommandProps,
  ): CreateCommandProps {
    return {
      resource,
      linkRel: this.config.edit.linkRel,
      command: {
        order: this.config.edit.order,
        body: toSave,
      },
      snackBarMessage: saveCommandProps.snackBarMessage,
    };
  }

  doPatch(resource: T, toPatch: unknown): Observable<T> {
    throw new Error('Method not implemented.');
  }

  public delete(
    deleteCommandProps: DeleteCommandProps = { snackBarMessage: EMPTY_STRING },
  ): Observable<StateResource<CommandResource>> {
    return this.selectResource().pipe(
      switchMap((stateResource: StateResource<T>) => this.doDelete(stateResource.resource, deleteCommandProps)),
      tapOnCommandSuccessfullyDone(() => this.clearResource()),
      filter(isStateResoureStable),
      startWith(createEmptyStateResource<CommandResource>(true)),
    );
  }

  doDelete(resource: T, deleteCommandProps: DeleteCommandProps): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommandByProps(this.buildDeleteCommandProps(resource, deleteCommandProps));
  }

  buildDeleteCommandProps<T extends Resource>(resource: T, deleteCommandProps: DeleteCommandProps): CreateCommandProps {
    return {
      resource,
      linkRel: this.config.delete.linkRel,
      command: {
        order: this.config.delete.order,
        body: null,
      },
      snackBarMessage: deleteCommandProps.snackBarMessage,
    };
  }
}
