/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createStateResource } from '@alfa-client/tech-shared';
import { getUrl } from '@ngxp/rest';
import { createCommandErrorResource, createCommandListResource, createCommandResource } from 'libs/command-shared/test/command';
import { CommandLinkRel } from './command.linkrel';
import { CommandErrorMessage } from './command.message';
import { CommandListResource, CommandOrder, CommandResource, CommandStatus, PendingCommandMap } from './command.model';
import {
  buildPendingCommandMap,
  getPendingCommandByOrder,
  hasCommandError,
  isConcurrentModification,
  isDone,
  isPending,
  isRevokeable,
  isSuccessfulDone,
  notHasCommandError,
} from './command.util';

describe('CommandUtil', () => {
  describe('isRevokeable', () => {
    it('should return true if Link exists', () => {
      const result: boolean = isRevokeable(createCommandResource([CommandLinkRel.REVOKE]));

      expect(result).toBe(true);
    });
    it('should return false if Link does NOT exists', () => {
      const result: boolean = isRevokeable(createCommandResource());

      expect(result).toBe(false);
    });
  });

  describe('isPending', () => {
    it('should return true if Link exists', () => {
      const result: boolean = isPending(createCommandResource([CommandLinkRel.UPDATE]));

      expect(result).toBe(true);
    });
    it('should return false if Link does NOT exists', () => {
      const result: boolean = isPending(createCommandResource());

      expect(result).toBe(false);
    });
  });

  describe('isDone', () => {
    it('should return true if Link exists', () => {
      const result: boolean = isDone(createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]));

      expect(result).toBe(true);
    });
    it('should return false if Link does NOT exists', () => {
      const result: boolean = isDone(createCommandResource());

      expect(result).toBe(false);
    });
  });

  describe('hasError', () => {
    it('should be true if no update link is present and command has error message', () => {
      const result: boolean = hasCommandError(createCommandErrorResource());

      expect(result).toBeTruthy();
    });

    it('should be false if update link is present', () => {
      const result: boolean = hasCommandError(createCommandResource([CommandLinkRel.UPDATE]));

      expect(result).toBeFalsy();
    });

    it('should be false if error message is not present', () => {
      const result: boolean = hasCommandError(createCommandResource());

      expect(result).toBeFalsy();
    });
  });

  describe('getPendingCommandByOrder', () => {
    const order: CommandOrder = <CommandOrder>'dummyOrder';
    const anotherOrder: CommandOrder = <CommandOrder>'anotherOrder';

    it('should return null on non existing pending command', () => {
      const listResource: CommandListResource = createCommandListResource();

      const pendingCommand: CommandResource = getPendingCommandByOrder(listResource, [order]);

      expect(pendingCommand).toBeNull();
    });

    it('should return existing pending command on single order', () => {
      const command: CommandResource = { ...createCommandResource(), order };
      const listResource: CommandListResource = createCommandListResource([command]);

      const pendingCommand: CommandResource = getPendingCommandByOrder(listResource, [order]);

      expect(pendingCommand).toBe(command);
    });

    it('should return existing pending command on multiple orders', () => {
      const command: CommandResource = { ...createCommandResource(), order: anotherOrder };
      const listResource: CommandListResource = createCommandListResource([command]);

      const pendingCommand: CommandResource = getPendingCommandByOrder(listResource, [order, anotherOrder]);

      expect(pendingCommand).toBe(command);
    });
  });

  describe('build pending command map', () => {
    it('should return map', () => {
      const pendingCommand1: CommandResource = {
        ...createCommandResource([CommandLinkRel.RELATED_RESOURCE]),
        status: CommandStatus.PENDING,
      };
      const pendingCommand2: CommandResource = {
        ...createCommandResource([CommandLinkRel.RELATED_RESOURCE]),
        status: CommandStatus.PENDING,
      };
      const commandList: CommandListResource = createCommandListResource([pendingCommand1, pendingCommand2]);

      const map: PendingCommandMap = buildPendingCommandMap(commandList);

      expect(map[getUrl(pendingCommand1, CommandLinkRel.RELATED_RESOURCE)]).toEqual(createStateResource(pendingCommand1));
      expect(map[getUrl(pendingCommand2, CommandLinkRel.RELATED_RESOURCE)]).toEqual(createStateResource(pendingCommand2));
    });
  });

  describe('isConcurrentModification', () => {
    it('should return true on matching error message', () => {
      const doesMatch: boolean = isConcurrentModification(CommandErrorMessage.CONCURRENT_MODIFICATION);

      expect(doesMatch).toBeTruthy();
    });
  });

  describe('isSuccessfulDone', () => {
    it('should return true if command is done and has no error', () => {
      const isDoneSuccessfuly: boolean = isSuccessfulDone({
        ...createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
      });

      expect(isDoneSuccessfuly).toBeTruthy();
    });

    it('should return false if command is done with error', () => {
      const isDoneSuccessfuly: boolean = isSuccessfulDone({
        ...createCommandErrorResource([CommandLinkRel.EFFECTED_RESOURCE]),
      });

      expect(isDoneSuccessfuly).toBeFalsy();
    });

    it('should return false if command is not done without error', () => {
      const isDoneSuccessfuly: boolean = isSuccessfulDone(createCommandResource());

      expect(isDoneSuccessfuly).toBeFalsy();
    });

    it('should return false if command is not done with error', () => {
      const isDoneSuccessfuly: boolean = isSuccessfulDone(createCommandErrorResource());

      expect(isDoneSuccessfuly).toBeFalsy();
    });
  });

  describe('notHasCommandError', () => {
    it('should return true', () => {
      const hasError: boolean = notHasCommandError(createCommandResource());

      expect(hasError).toBeTruthy();
    });

    it('should return false', () => {
      const hasError: boolean = notHasCommandError(createCommandErrorResource());

      expect(hasError).toBeFalsy();
    });
  });
});
