/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ENTER_KEY, ESCAPE_KEY } from '@alfa-client/tech-shared';
import { getElementFromFixture, mock, useFromMock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed, fakeAsync, tick } from '@angular/core/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createKeydownKeyboardEvent } from '../../../../../test-utils/src/lib/keyboard';
import { DropdownMenuComponent } from './dropdown-menu.component';
import { EventEmitter } from '@angular/core';

describe('DropdownMenuComponent', () => {
  let component: DropdownMenuComponent;
  let fixture: ComponentFixture<DropdownMenuComponent>;

  const dropdownButton: string = getDataTestIdOf('dropdown-button');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [DropdownMenuComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(DropdownMenuComponent);
    component = fixture.componentInstance;
    component.opened = useFromMock(mock(EventEmitter<void>));
    component.closed = useFromMock(mock(EventEmitter<void>));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('togglePopup', () => {

    it('should mark as open', () => {
      component.isPopupOpen = false;

      component.togglePopup();

      expect(component.isPopupOpen).toBe(true);
    });

    it('should mark as NOT open', () => {
      component.isPopupOpen = true;

      component.togglePopup();

      expect(component.isPopupOpen).toBe(false);
    });

    it('should emit event on open', () => {
      component.isPopupOpen = false;

      component.togglePopup();

      expect(component.opened.emit).toHaveBeenCalled();
    });

    it('should emit event on close', () => {
      component.isPopupOpen = true;

      component.togglePopup();

      expect(component.closed.emit).toHaveBeenCalled();
    });
  });

  describe('toggleFocusedButton', () => {
    it('should change false to true', () => {
      component.isButtonFocused = false;

      component.toggleFocusedButton();

      expect(component.isButtonFocused).toBe(true);
    });

    it('should change true to false', () => {
      component.isButtonFocused = true;

      component.toggleFocusedButton();

      expect(component.isButtonFocused).toBe(false);
    });
  });

  describe('handleButtonClick', () => {
    beforeEach(() => {
      component.togglePopup = jest.fn();
      component.focusList = jest.fn();
    });

    it('should toggle popup', () => {
      component.handleButtonClick();

      expect(component.togglePopup).toHaveBeenCalled();
    });

    it('should focus list if popup is visible', () => {
      component.isPopupOpen = true;

      component.handleButtonClick();

      expect(component.focusList).toHaveBeenCalled();
    });

    it('should not focus list if popup is hidden', () => {
      component.handleButtonClick();

      expect(component.focusList).not.toHaveBeenCalled();
    });
  });

  describe('focusList', () => {
    it('should focus popup list item', fakeAsync(() => {
      component.isPopupOpen = true;
      fixture.detectChanges();
      component.popupListRef.nativeElement.focus = jest.fn();

      component.focusList();
      tick();

      expect(component.popupListRef.nativeElement.focus).toHaveBeenCalled();
    }));
  });

  describe('on focus button', () => {
    it('should toggle focused button', () => {
      component.toggleFocusedButton = jest.fn();
      const buttonElement: HTMLElement = getElementFromFixture(fixture, dropdownButton);

      buttonElement.focus();

      expect(component.toggleFocusedButton).toHaveBeenCalled();
    });
  });

  describe('on blur button', () => {
    it('should toggle focused button', () => {
      component.toggleFocusedButton = jest.fn();
      const buttonElement: HTMLElement = getElementFromFixture(fixture, dropdownButton);

      buttonElement.dispatchEvent(new Event('blur'));

      expect(component.toggleFocusedButton).toHaveBeenCalled();
    });
  });

  describe('aria-expanded', () => {
    it('should be true if popup is open', () => {
      component.isPopupOpen = true;
      fixture.detectChanges();

      const buttonElement: HTMLElement = getElementFromFixture(fixture, dropdownButton);

      expect(buttonElement.getAttribute('aria-expanded')).toBe('true');
    });

    it('should be false if popup is closed', () => {
      component.isPopupOpen = false;
      fixture.detectChanges();

      const buttonElement: HTMLElement = getElementFromFixture(fixture, dropdownButton);

      expect(buttonElement.getAttribute('aria-expanded')).toBe('false');
    });
  });

  describe('closePopupAndFocusButton', () => {
    beforeEach(() => {
      component.isPopupOpen = true;
      jest.spyOn(component.buttonRef.nativeElement, 'focus');
    });
    it('should close popup', () => {
      component.closePopupAndFocusButton();

      expect(component.isPopupOpen).toBe(false);
    });

    it('should focus button', () => {
      component.closePopupAndFocusButton();

      expect(component.buttonRef.nativeElement.focus).toHaveBeenCalled();
    });
  });

  describe('isPopupClosed', () => {
    it('should return true', () => {
      component.isPopupOpen = false;

      const result: boolean = component.isPopupClosed();

      expect(result).toBe(true);
    });

    it('should return false', () => {
      component.isPopupOpen = true;

      const result: boolean = component.isPopupClosed();

      expect(result).toBe(false);
    });
  });

  describe('onKeydownHandler', () => {
    beforeEach(() => {
      component.closePopupAndFocusButton = jest.fn();
    });

    describe('popup is closed', () => {
      beforeEach(() => {
        component.isPopupClosed = jest.fn().mockReturnValue(true);
      });

      it('should not close popup', () => {
        component.closePopupAndFocusButton = jest.fn();

        component.onKeydownHandler(createKeydownKeyboardEvent(ESCAPE_KEY));

        expect(component.closePopupAndFocusButton).not.toHaveBeenCalled();
      });
    });

    describe('popup is open', () => {
      beforeEach(() => {
        component.isPopupClosed = jest.fn().mockReturnValue(false);
      });

      it('should close popup', () => {
        component.onKeydownHandler(createKeydownKeyboardEvent(ESCAPE_KEY));

        expect(component.closePopupAndFocusButton).toHaveBeenCalled();
      });

      it('should not close popup', () => {
        component.onKeydownHandler(createKeydownKeyboardEvent(ENTER_KEY));

        expect(component.closePopupAndFocusButton).not.toHaveBeenCalled();
      });
    });
  });
  describe('onClickHandler', () => {
    const e: MouseEvent = new MouseEvent('test');

    beforeEach(() => {
      component.closePopupAndFocusButton = jest.fn();
      component.buttonRef.nativeElement.contains = jest.fn();
    });

    describe('popup is closed', () => {
      beforeEach(() => {
        component.isPopupClosed = jest.fn().mockReturnValue(true);
      });

      it('should not check for button containing event target', () => {
        component.onClickHandler(e);

        expect(component.buttonRef.nativeElement.contains).not.toHaveBeenCalled();
      });
    });

    describe('popup is open', () => {
      beforeEach(() => {
        component.isPopupClosed = jest.fn().mockReturnValue(false);
      });

      it('should check for button containing event target', () => {
        component.onClickHandler(e);

        expect(component.buttonRef.nativeElement.contains).toHaveBeenCalled();
      });

      it('should handle click', () => {
        component.onClickHandler(e);

        expect(component.closePopupAndFocusButton).toHaveBeenCalled();
      });

      it('should not handle click', () => {
        component.buttonRef.nativeElement.contains = jest.fn().mockReturnValue(true);

        component.onClickHandler(e);

        expect(component.closePopupAndFocusButton).not.toHaveBeenCalled();
      });
    });
  });

  describe('closeMenu', () => {
    it('should set isPopupOpen to false', () => {
      component.isPopupOpen = true;

      component.closeMenu();

      expect(component.isPopupOpen).toBeFalsy();
    });

    it('should emit close event', () => {
      component.closeMenu();

      expect(component.closed.emit).toHaveBeenCalled();
    });
  });
});
