/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileService } from '@alfa-client/binary-file-shared';
import { createStateResource, EMPTY_STRING, StateResource } from '@alfa-client/tech-shared';
import { mock, Mock, useFromMock } from '@alfa-client/test-utils';
import { createKommentar, createKommentarResource } from 'libs/kommentar-shared/test/kommentar';
import { Observable, of } from 'rxjs';
import { KommentarListResourceService } from './kommentar-list-resource.service';
import { KommentarResourceService } from './kommentar-recource.service';
import { Kommentar, KOMMENTAR_UPLOADED_ATTACHMENTS, KommentarResource } from './kommentar.model';
import { KommentarService } from './kommentar.service';

import { CommandLinkRel, CommandResource } from '@alfa-client/command-shared';
import { faker } from '@faker-js/faker/.';
import { ResourceUri } from '@ngxp/rest';
import { createCommandResource } from '../../../command-shared/test/command';
import { singleCold } from '../../../tech-shared/test/marbles';
import { KommentarLinkRel } from './kommentar.linkrel';

jest.mock('./kommentar-list-resource.service');
jest.mock('./kommentar-recource.service');

describe('KommentarService', () => {
  let service: KommentarService;
  let binaryFileService: Mock<BinaryFileService>;
  let kommentarResourceService: Mock<KommentarResourceService>;
  let kommentarListResourceService: Mock<KommentarListResourceService>;

  const kommentar: Kommentar = createKommentar();
  const kommentarResource: KommentarResource = createKommentarResource();
  const commandStateRessource: StateResource<CommandResource> = createStateResource(
    createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
  );

  beforeEach(() => {
    binaryFileService = mock(BinaryFileService);
    kommentarResourceService = mock(KommentarResourceService);
    kommentarListResourceService = mock(KommentarListResourceService);
    service = new KommentarService(
      useFromMock(binaryFileService),
      useFromMock(kommentarResourceService),
      useFromMock(kommentarListResourceService),
    );
  });

  describe('getKommentar', () => {
    it('should call getList from listResourceService', () => {
      service.getKommentare();

      expect(kommentarListResourceService.getList).toHaveBeenCalled();
    });
  });

  describe('formularVisibility', () => {
    it('should be true after showFormular', () => {
      const isFormularVisible: Observable<boolean> = service.isNewKommentarFormularVisible();

      service.showNewKommentarFormular();

      expect(isFormularVisible).toBeObservable(singleCold(true));
    });

    it('should be false after hideFormular', () => {
      const isFormularVisible: Observable<boolean> = service.isNewKommentarFormularVisible();

      service.hideNewKommentarFormular();

      expect(isFormularVisible).toBeObservable(singleCold(false));
    });
  });

  describe('create Kommentar', () => {
    beforeEach(() => {
      kommentarListResourceService.createListItem.mockReturnValue(of(commandStateRessource));
    });

    it('should call kommentarListResource service createListItem', () => {
      service.createKommentar(kommentar);

      expect(kommentarListResourceService.createListItem).toHaveBeenCalled();
    });

    it('should hide formular after create Kommentar', () => {
      const isFormularVisible: Observable<boolean> = service.isNewKommentarFormularVisible();

      service.createKommentar(kommentar).subscribe();

      expect(isFormularVisible).toBeObservable(singleCold(false));
    });

    it('should refresh kommentarListResource', () => {
      service.createKommentar(kommentar).subscribe();

      expect(kommentarListResourceService.refresh).toHaveBeenCalled();
    });
  });

  describe('edit Kommentar', () => {
    beforeEach(() => {
      kommentarResourceService.save.mockReturnValue(of(commandStateRessource));
    });

    it('should call kommentarResource service save', () => {
      service.editKommentar(kommentar);

      expect(kommentarResourceService.save).toHaveBeenCalled();
    });

    it('should hide formular', () => {
      const isFormularVisible: Observable<boolean> = service.isNewKommentarFormularVisible();

      service.editKommentar(kommentar).subscribe();

      expect(isFormularVisible).toBeObservable(singleCold(false));
    });

    it('should refresh kommentarListResource', () => {
      service.editKommentar(kommentar).subscribe();

      expect(kommentarListResourceService.refresh).toHaveBeenCalled();
    });
  });

  describe('attachments', () => {
    it('should be loaded if link avaible', () => {
      const kommentarResource: KommentarResource = createKommentarResource([KommentarLinkRel.ATTACHMENTS]);

      service._getAttachments(kommentarResource);

      expect(binaryFileService.getFiles).toHaveBeenCalledWith(kommentarResource, KommentarLinkRel.ATTACHMENTS);
    });

    it('should not be loaded if no link available', () => {
      service._getAttachments(kommentarResource);

      expect(binaryFileService.getFiles).not.toHaveBeenCalled();
    });
  });

  describe('selectKommentar', () => {
    it('should select Kommentar in kommentarList Service', () => {
      service.selectKommentar(kommentarResource);

      expect(kommentarListResourceService.select).toHaveBeenCalled();
    });
  });

  describe('getKommentarStateResource', () => {
    it('should call kommentarResource service get ', () => {
      service.getKommentarStateResource();

      expect(kommentarResourceService.get).toHaveBeenCalled();
    });
  });

  describe('clearUploadedFiles', () => {
    it('should call binary file service', () => {
      service.clearUploadedFiles();

      expect(binaryFileService.clearUploadedFiles).toHaveBeenCalledWith(KOMMENTAR_UPLOADED_ATTACHMENTS);
    });
  });

  describe('is new kommentar formular visible', () => {
    it('should emit true', () => {
      service._currentlyEdited$.next(EMPTY_STRING);
      service.formularVisibility$.next(true);

      expect(service.isNewKommentarFormularVisible()).toBeObservable(singleCold(true));
    });

    it('should emit false if any kommentar is being edited', () => {
      service._currentlyEdited$.next(faker.internet.url());
      service.formularVisibility$.next(true);

      expect(service.isNewKommentarFormularVisible()).toBeObservable(singleCold(false));
    });

    it('should emit false if not visible', () => {
      service._currentlyEdited$.next(EMPTY_STRING);
      service.formularVisibility$.next(false);

      expect(service.isNewKommentarFormularVisible()).toBeObservable(singleCold(false));
    });
  });

  describe('show new kommentar formular', () => {
    beforeEach(() => {
      service.setCurrentlyEdited = jest.fn();
    });

    it('should set currently edited to empty string', () => {
      service.showNewKommentarFormular();

      expect(service.setCurrentlyEdited).toHaveBeenCalledWith(EMPTY_STRING);
    });

    it('should set formular visibility', () => {
      service.showNewKommentarFormular();

      expect(service.formularVisibility$).toBeObservable(singleCold(true));
    });
  });

  describe('set currently edited kommentar uri', () => {
    beforeEach(() => {
      service.clearUploadedFiles = jest.fn();
      service.hideNewKommentarFormular = jest.fn();
    });

    it('should clear uploaded files', () => {
      service.setCurrentlyEdited(faker.internet.url());

      expect(service.clearUploadedFiles).toHaveBeenCalled();
    });

    it('should hide kommentar creation formular', () => {
      service.setCurrentlyEdited(faker.internet.url());

      expect(service.hideNewKommentarFormular).toHaveBeenCalled();
    });

    it('should NOT hide kommentar creation formular', () => {
      service.setCurrentlyEdited(EMPTY_STRING);

      expect(service.hideNewKommentarFormular).not.toHaveBeenCalled();
    });

    it('should emit currently edited resource uri', () => {
      const resourceUri: ResourceUri = faker.internet.url();

      service.setCurrentlyEdited(resourceUri);

      expect(service._currentlyEdited$).toBeObservable(singleCold(resourceUri));
    });
  });
});
