/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandOrder, CommandResource, CreateCommandProps } from '@alfa-client/command-shared';
import { ApiError, createEmptyStateResource, createErrorStateResource, createStateResource } from '@alfa-client/tech-shared';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Action } from '@ngrx/store';
import { createCommand, createCommandResource, createCreateCommandProps } from 'libs/command-shared/test/command';
import { createLoeschAnforderungResource } from 'libs/loesch-anforderung-shared/test/loesch-anforderung';
import { createApiError } from 'libs/tech-shared/test/error';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { LoeschAnforderungLinkRel } from '../loesch-anforderung.linkrel';
import { LoeschAnforderungResource } from '../loesch-anforderung.model';
import { createRouteData } from './../../../../navigation-shared/test/navigation-test-factory';
import { LoeschAnforderungState, initialState, reducer } from './loesch-anforderung.reducer';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';
import * as NavigationActions from '../../../../navigation-shared/src/lib/+state/navigation.actions';
import * as VorgangActions from '../../../../vorgang-shared/src/lib/+state/vorgang.actions';
import * as LoeschAnforderungActions from './loesch-anforderung.actions';

describe('LoeschAnforderung Reducer', () => {
  describe('unknown action', () => {
    it('should return current state', () => {
      const action = {} as Action;

      const result = reducer(initialState, action);

      expect(result).toBe(initialState);
    });
  });

  describe('LoeschAnforderung', () => {
    describe('on "loadLoeschAnforderungSuccess" action', () => {
      it('should set loeschenAnfordernResource', () => {
        const loeschAnforderung: LoeschAnforderungResource = createLoeschAnforderungResource();
        const action = LoeschAnforderungActions.loadLoeschAnforderungSuccess({ loeschAnforderung });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.loeschAnforderung).toEqual(createStateResource(loeschAnforderung));
      });
    });

    describe('on "loadLoeschAnforderungFailure" action', () => {
      it('should set loeschenAnfordernResource as errorStateResource', () => {
        const apiError: ApiError = createApiError();
        const action = LoeschAnforderungActions.loadLoeschAnforderungFailure({ apiError });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.loeschAnforderung).toEqual(createErrorStateResource(apiError));
      });
    });
  });

  describe('VorgangZumLoeschenMarkieren', () => {
    describe('on "createCommand" action', () => {
      it('should set vorgangLoeschenCommand loading', () => {
        const action = CommandActions.createCommand({
          command: { ...createCommand(), order: CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN },
          resource: createVorgangWithEingangResource(),
          linkRel: LoeschAnforderungLinkRel.EXECUTE_LOESCHEN,
        });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.loeschenAnfordernCommand.loading).toBeTruthy();
      });
    });

    describe('on "createCommandSuccess" action', () => {
      it('should set vorgangLoeschenCommand as StateResource', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN,
        };
        const createCommandProps: CreateCommandProps = createCreateCommandProps();
        const action = CommandActions.createCommandSuccess({ command, createCommandProps });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.loeschenAnfordernCommand).toEqual(createStateResource(command));
      });
    });

    describe('on "revokeCommandSuccess" action', () => {
      it('should clear loeschAnforderung', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN,
        };
        const action = CommandActions.revokeCommandSuccess({ command });

        const state: LoeschAnforderungState = reducer(
          {
            ...initialState,
            loeschAnforderung: createStateResource(createLoeschAnforderungResource()),
          },
          action,
        );

        expect(state.loeschAnforderung).toEqual(createEmptyStateResource());
      });
    });
  });

  describe('VorgangLoeschen', () => {
    describe('on "createCommand" action', () => {
      it('should set vorgangLoeschenCommand loading', () => {
        const action = CommandActions.createCommand({
          command: { ...createCommand(), order: CommandOrder.VORGANG_LOESCHEN },
          resource: createVorgangWithEingangResource(),
          linkRel: LoeschAnforderungLinkRel.EXECUTE_LOESCHEN,
        });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.vorgangLoeschenCommand.loading).toBeTruthy();
      });
    });

    describe('on "createCommandSuccess" action', () => {
      it('should set vorgangLoeschenCommand as StateResource', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.VORGANG_LOESCHEN,
        };
        const createCommandProps: CreateCommandProps = createCreateCommandProps();
        const action = CommandActions.createCommandSuccess({ command, createCommandProps });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.vorgangLoeschenCommand).toEqual(createStateResource(command));
      });
    });
  });

  describe('LoeschAnforderungZuruecknehmen', () => {
    describe('on "createCommand" action', () => {
      it('should set loeschAnforderungZuruecknehmenCommand loading', () => {
        const action = CommandActions.createCommand({
          command: { ...createCommand(), order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN },
          resource: createVorgangWithEingangResource(),
          linkRel: LoeschAnforderungLinkRel.ZURUECKNEHMEN,
        });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.loeschAnforderungZuruecknehmenCommand.loading).toBeTruthy();
      });
    });

    describe('on "createCommandSuccess" action', () => {
      it('should set loeschAnforderungZuruecknehmenCommand as StateResource', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN,
        };
        const createCommandProps: CreateCommandProps = createCreateCommandProps();
        const action = CommandActions.createCommandSuccess({ command, createCommandProps });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.loeschAnforderungZuruecknehmenCommand).toEqual(createStateResource(command));
      });

      it('should clear LoeschAnforderung state', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN,
        };
        const createCommandProps: CreateCommandProps = createCreateCommandProps();
        const action = CommandActions.createCommandSuccess({ command, createCommandProps });

        const state: LoeschAnforderungState = reducer(
          {
            ...initialState,
            loeschAnforderung: createStateResource(createLoeschAnforderungResource()),
          },
          action,
        );

        expect(state.loeschAnforderung).toEqual(createEmptyStateResource());
      });
    });

    describe('on "createCommandFailure" action', () => {
      it('should set loeschAnforderungZuruecknehmenCommand as StateResource', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN,
        };
        const error: ApiError = createApiError();
        const createCommandProps: CreateCommandProps = createCreateCommandProps();
        const action = CommandActions.createCommandFailure({ command, createCommandProps, error });

        const state: LoeschAnforderungState = reducer(initialState, action);

        expect(state.loeschAnforderungZuruecknehmenCommand).toEqual(createErrorStateResource(error));
      });
    });

    describe('on "revokeCommandSuccess" action', () => {
      it('should reload loeschAnforderung', () => {
        const command: CommandResource = {
          ...createCommandResource(),
          order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN,
        };
        const action = CommandActions.revokeCommandSuccess({ command });

        const state: LoeschAnforderungState = reducer(
          {
            ...initialState,
            loeschAnforderung: createStateResource(createLoeschAnforderungResource()),
          },
          action,
        );

        expect(state.loeschAnforderung.reload).toBeTruthy();
      });
    });
  });

  describe('Vorgang', () => {
    const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
    const action: Action<string> = VorgangActions.loadVorgangWithEingangSuccess({
      vorgangWithEingang,
    });

    describe('loadVorgangWithEingangSuccess', () => {
      it('should clear loeschenAnfordernCommand', () => {
        const filledState: LoeschAnforderungState = {
          ...initialState,
          loeschenAnfordernCommand: createStateResource(createCommandResource()),
        };
        const state: LoeschAnforderungState = reducer(filledState, action);

        expect(state.loeschenAnfordernCommand).toEqual(createEmptyStateResource());
      });

      it('should clear loeschAnforderung', () => {
        const filledState: LoeschAnforderungState = {
          ...initialState,
          loeschAnforderung: createStateResource(createLoeschAnforderungResource()),
        };
        const state: LoeschAnforderungState = reducer(filledState, action);

        expect(state.loeschAnforderung).toEqual(createEmptyStateResource());
      });

      it('should clear vorgangLoeschenCommand', () => {
        const filledState: LoeschAnforderungState = {
          ...initialState,
          vorgangLoeschenCommand: createStateResource(createCommandResource()),
        };
        const state: LoeschAnforderungState = reducer(filledState, action);

        expect(state.vorgangLoeschenCommand).toEqual(createEmptyStateResource());
      });

      it('should clear loeschAnforderungZuruecknehmenCommand', () => {
        const filledState: LoeschAnforderungState = {
          ...initialState,
          loeschAnforderungZuruecknehmenCommand: createStateResource(createCommandResource()),
        };
        const state: LoeschAnforderungState = reducer(filledState, action);

        expect(state.loeschAnforderungZuruecknehmenCommand).toEqual(createEmptyStateResource());
      });
    });
  });

  describe('Navigation', () => {
    it('should clear loeschenAnfordernCommand', () => {
      const action: Action<string> = NavigationActions.updateCurrentRouteData({
        routeData: createRouteData(),
      });

      const filledState: LoeschAnforderungState = {
        ...initialState,
        loeschenAnfordernCommand: createStateResource(createCommandResource()),
      };
      const state: LoeschAnforderungState = reducer(filledState, action);

      expect(state.loeschenAnfordernCommand).toEqual(createEmptyStateResource());
    });

    it('should clear loeschAnforderung', () => {
      const action: Action<string> = NavigationActions.updateCurrentRouteData({
        routeData: createRouteData(),
      });

      const filledState: LoeschAnforderungState = {
        ...initialState,
        loeschAnforderung: createStateResource(createLoeschAnforderungResource()),
      };
      const state: LoeschAnforderungState = reducer(filledState, action);

      expect(state.loeschAnforderung).toEqual(createEmptyStateResource());
    });

    it('should clear vorgangLoeschenCommand', () => {
      const action: Action<string> = NavigationActions.updateCurrentRouteData({
        routeData: createRouteData(),
      });

      const filledState: LoeschAnforderungState = {
        ...initialState,
        vorgangLoeschenCommand: createStateResource(createCommandResource()),
      };
      const state: LoeschAnforderungState = reducer(filledState, action);

      expect(state.vorgangLoeschenCommand).toEqual(createEmptyStateResource());
    });

    it('should clear loeschAnforderungZuruecknehmenCommand', () => {
      const action: Action<string> = NavigationActions.updateCurrentRouteData({
        routeData: createRouteData(),
      });

      const filledState: LoeschAnforderungState = {
        ...initialState,
        loeschAnforderungZuruecknehmenCommand: createStateResource(createCommandResource()),
      };
      const state: LoeschAnforderungState = reducer(filledState, action);

      expect(state.loeschAnforderungZuruecknehmenCommand).toEqual(createEmptyStateResource());
    });
  });
});
