/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileService } from '@alfa-client/binary-file-shared';
import { CommandResource, CommandService } from '@alfa-client/command-shared';
import { NavigationFacade, NavigationService, RouteData } from '@alfa-client/navigation-shared';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createLoadingStateResource,
  createStateResource,
  StateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { VORGANG_WITH_EINGANG_ROUTE_PARAM, VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { TestBed } from '@angular/core/testing';
import { MatDialog } from '@angular/material/dialog';
import { faker } from '@faker-js/faker';
import { beforeEach, expect } from '@jest/globals';
import { getUrl } from '@ngxp/rest';
import { hot } from 'jest-marbles';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandErrorResource, createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { BehaviorSubject, Observable, of } from 'rxjs';
import { createProblemDetail } from '../../../tech-shared/test/error';
import { multipleCold, singleColdCompleted, singleHot } from '../../../tech-shared/test/marbles';
import {
  createPostfachFeatures,
  createPostfachMail,
  createPostfachMailListResource,
  createPostfachMailResource,
  createPostfachSettings,
} from '../../test/postfach';
import { PostfachFacade } from './+state/postfach.facade';
import { PostfachMailLinkRel, PostfachMailListLinkRel } from './postfach.linkrel';
import { PostfachMessages } from './postfach.message';
import { PostfachMail, PostfachMailListResource, PostfachMailResource } from './postfach.model';
import { PostfachRepository } from './postfach.repository';
import { PostfachService } from './postfach.service';
import { createSendPostfachMailCommand } from './postfach.util';

describe('PostfachService', () => {
  let service: PostfachService;

  let repository: Mock<PostfachRepository>;
  const commandService: Mock<CommandService> = mock(CommandService);
  const navigationService: Mock<NavigationService> = mock(NavigationService);
  let vorgangService: Mock<VorgangService>;
  const snackbarService: Mock<SnackBarService> = mock(SnackBarService);
  const dialog: Mock<MatDialog> = <Mock<MatDialog>>{ closeAll: jest.fn() };
  const postfachFacade: Mock<PostfachFacade> = mock(PostfachFacade);
  const binaryFileService: Mock<BinaryFileService> = mock(BinaryFileService);
  const navigationFacade = mock(NavigationFacade);

  const urlChangedParams = {};

  const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
  const vorgangWithEingangStateResource: StateResource<VorgangWithEingangResource> = createStateResource(vorgang);

  beforeEach(() => {
    vorgangService = {
      ...mock(VorgangService),
      getVorgangWithEingang: jest.fn().mockReturnValue(of(vorgangWithEingangStateResource)),
      selectVorgangWithEingang: jest.fn().mockReturnValue(of(vorgangWithEingangStateResource)),
    };
    navigationService.urlChanged = jest.fn();
    navigationService.urlChanged.mockReturnValue(of(urlChangedParams));

    repository = mock(PostfachRepository);

    TestBed.configureTestingModule({
      providers: [
        { provide: CommandService, useValue: commandService },
        { provide: NavigationService, useValue: navigationService },
        { provide: VorgangService, useValue: vorgangService },
        { provide: SnackBarService, useValue: snackbarService },
        { provide: MatDialog, useValue: dialog },
        { provide: PostfachFacade, useValue: postfachFacade },
        { provide: BinaryFileService, useValue: binaryFileService },
        { provide: PostfachRepository, useValue: repository },
        { provide: NavigationFacade, useValue: navigationFacade },
        PostfachService,
      ],
    });

    service = TestBed.inject(PostfachService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('send message', () => {
    const postfachMail: PostfachMail = createPostfachMail();
    const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

    beforeEach(() => {
      commandService.createCommand.mockReturnValue(of(commandStateResource));
      vorgangService.getPendingSendPostfachMailCommand.mockReturnValue(of(commandStateResource));
    });

    it('should call vorgang service and set command on loading', () => {
      service.sendMail(postfachMail);

      expect(vorgangService.setPendingSendPostfachMailSingleCommandLoading).toHaveBeenCalled();
    });

    it('should call command service', () => {
      service.sendMail(postfachMail);

      expect(commandService.createCommand).toHaveBeenCalled();
    });

    it('should call vorgang service and get command', () => {
      service.sendMail(postfachMail);

      expect(vorgangService.getPendingSendPostfachMailCommand).toHaveBeenCalled();
    });

    describe('reload postfach list', () => {
      beforeEach(() => {
        service._refreshPostfachMailList = jest.fn();
      });

      it('should execute on command is done', () => {
        const commandStateResource: StateResource<CommandResource> = createStateResource(
          createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
        );
        commandService.createCommand.mockReturnValue(of(commandStateResource));

        service.sendMail(postfachMail);

        expect(service._refreshPostfachMailList).toHaveBeenCalledWith(commandStateResource);
      });

      it('should not execute on pending command', () => {
        commandService.createCommand.mockReturnValue(of(createStateResource(createCommandResource([CommandLinkRel.UPDATE]))));

        service.sendMail(postfachMail);

        expect(service._refreshPostfachMailList).not.toHaveBeenCalled();
      });
    });
  });

  describe('sendExistingNachricht', () => {
    const postfachNachricht: PostfachMailResource = createPostfachMailResource();
    const nachricht: PostfachMail = createPostfachMail();

    const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

    it('should call doSendNachricht', () => {
      service._doSendNachricht = jest.fn();
      commandService.createCommand.mockReturnValue(of(commandStateResource));
      vorgangService.getPendingSendPostfachMailCommand.mockReturnValue(of(commandStateResource));

      service.sendExistingMail(postfachNachricht, nachricht);

      expect(service._doSendNachricht).toHaveBeenCalledWith(
        postfachNachricht,
        PostfachMailLinkRel.SEND,
        createSendPostfachMailCommand(nachricht),
      );
    });
  });

  describe('resend message', () => {
    const postfachMailResource: PostfachMailResource = createPostfachMailResource();

    beforeEach(() => {
      service._handleSendPostfachMailIsDone = jest.fn();

      commandService.createCommand.mockReturnValue(of(createStateResource(createCommandResource())));
      vorgangService.getPendingSendPostfachMailCommand.mockReturnValue(of(createStateResource(createCommandResource())));
    });

    it('should call vorgang service and set command on loading', () => {
      service.resendMail(postfachMailResource);

      expect(vorgangService.setPendingSendPostfachMailSingleCommandLoading).toHaveBeenCalled();
    });

    it('should call command service', () => {
      service.resendMail(postfachMailResource);

      expect(commandService.createCommand).toHaveBeenCalled();
    });

    it('should call vorgang service and get command', () => {
      service.resendMail(postfachMailResource);

      expect(vorgangService.getPendingSendPostfachMailCommand).toHaveBeenCalled();
    });

    it('should NOT call handleSendPostfachMailIsDone if command is pending', () => {
      commandService.createCommand.mockReturnValue(of(createStateResource(createCommandResource([CommandLinkRel.UPDATE]))));
      service.resendMail(postfachMailResource);

      expect(service._handleSendPostfachMailIsDone).not.toHaveBeenCalled();
    });

    it('should call handleSendPostfachMailIsDone if command is done', () => {
      const commandResponse: StateResource<CommandResource> = createStateResource(
        createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
      );
      commandService.createCommand.mockReturnValue(of(commandResponse));

      service.resendMail(postfachMailResource);

      expect(service._handleSendPostfachMailIsDone).toHaveBeenCalledWith(commandResponse);
    });
  });

  describe('handleSendNachrichtCommand', () => {
    const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

    beforeEach(() => {
      service._commandIsDone = jest.fn();
    });

    it('should call vorgang service', () => {
      service._handleSendNachrichtCommand(commandStateResource);

      expect(vorgangService.setPendingSendPostfachMailCommand).toHaveBeenCalledWith(commandStateResource);
    });

    it('should call commandIsDone', () => {
      service._handleSendNachrichtCommand(commandStateResource);

      expect(service._commandIsDone).toHaveBeenCalledWith(commandStateResource);
    });
  });

  describe('commandIsDone', () => {
    beforeEach(() => {
      service._handleSendPostfachMailIsDone = jest.fn();
    });

    it('should call handleSendPostfachMailIsDone', () => {
      const commandStateResource: StateResource<CommandResource> = createStateResource(
        createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
      );

      service._commandIsDone(commandStateResource);

      expect(service._handleSendPostfachMailIsDone).toHaveBeenCalledWith(commandStateResource);
    });

    describe('on command is done', () => {
      it('should call snackbar service', () => {
        const commandStateResource: StateResource<CommandResource> = createStateResource({
          ...createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
          errorMessage: null,
        });

        service._commandIsDone(commandStateResource);

        expect(snackbarService.show).toHaveBeenCalledWith(commandStateResource.resource, PostfachMessages.SEND_SUCCESSFUL);
      });
    });

    describe('on command is done with error', () => {
      it('should call snackbar service', () => {
        const commandStateResource: StateResource<CommandResource> = createStateResource(
          createCommandErrorResource([CommandLinkRel.EFFECTED_RESOURCE]),
        );

        service._commandIsDone(commandStateResource);

        expect(snackbarService.showError).toHaveBeenCalledWith(PostfachMessages.SEND_FAILED);
      });
    });
  });

  describe('listenToNavigation', () => {
    it('should call usnsubcribeToNavigation', () => {
      service._listenToNavigation();

      expect(navigationService.urlChanged).toHaveBeenCalled();
    });

    it('should call navigation service', () => {
      service._unsubscribeToNavigation = jest.fn();

      service._listenToNavigation();

      expect(service._unsubscribeToNavigation).toHaveBeenCalled();
    });

    it('should call onNavigation', () => {
      service._onNavigation = jest.fn();

      service._listenToNavigation();

      expect(service._onNavigation).toHaveBeenCalledWith(urlChangedParams);
    });
  });

  describe('onNavigation', () => {
    describe('to vorgang list', () => {
      beforeEach(() => {
        service._setPollingFalse = jest.fn();
        service._clearPostfachMailList = jest.fn();
        service._unsubscribe = jest.fn();
      });

      it('should set polling false', () => {
        service._onNavigation({});

        expect(service._setPollingFalse).toHaveBeenCalled();
      });

      it('should call clearPostfachMailList', () => {
        service._onNavigation({});

        expect(service._clearPostfachMailList).toHaveBeenCalled();
      });

      it('should call closeOpenDialogs', () => {
        service._onNavigation({});

        expect(dialog.closeAll).toHaveBeenCalled();
      });

      it('should call unsubscribe', () => {
        service._onNavigation({});

        expect(service._unsubscribe).toHaveBeenCalled();
      });
    });

    describe('to vorgang detail page', () => {
      it('should set postfach nachrichten list to reload', () => {
        service.postfachMailList$.next(createEmptyStateResource());

        service._onNavigation({ vorgangWithEingangUrl: 'X' });

        expect(service.postfachMailList$.value.reload).toEqual(true);
      });
    });
  });

  describe('getPendingSendPostfachMailCommand', () => {
    beforeEach(() => {
      service._unsubscribe = jest.fn();
      service._pollSendPostfachMailCommand = jest.fn();

      commandService.pollCommand.mockReturnValue(of(createStateResource(createCommandResource())));
      vorgangService.getPendingSendPostfachMailCommand.mockReturnValue(of(createStateResource(createCommandResource())));
    });

    it('should call navigation service', () => {
      service.getPendingSendPostfachMailCommand();

      expect(navigationService.urlChanged).toHaveBeenCalled();
    });
  });

  describe('getPostfachNachrichtListAsRead', () => {
    const state: StateResource<PostfachMailListResource> = createStateResource(createPostfachMailListResource());
    const stateAfterReset: StateResource<PostfachMailListResource> = createStateResource(createPostfachMailListResource());

    beforeEach(() => {
      service.getPostfachMailListByVorgang = jest.fn().mockReturnValue(of(state));
      service._resetHasNewPostfachNachrichten = jest.fn().mockReturnValue(of(stateAfterReset));
    });

    it('should get postfach nachrichten list', () => {
      service.getPostfachNachrichtListAsRead().subscribe();

      expect(service.getPostfachMailListByVorgang).toHaveBeenCalled();
    });

    it('should call service resetHasNewPostfachNachrichten', () => {
      service.getPostfachNachrichtListAsRead().subscribe();

      expect(service._resetHasNewPostfachNachrichten).toHaveBeenCalled();
    });

    it('should have return with loading', () => {
      service.getPostfachMailListByVorgang = jest.fn().mockReturnValue(singleHot(state));
      service._resetHasNewPostfachNachrichten = jest.fn().mockReturnValue(hot('-a', { a: stateAfterReset }));

      const result = service.getPostfachNachrichtListAsRead();

      expect(result).toBeObservable(multipleCold(createEmptyStateResource(true), stateAfterReset));
    });
  });

  describe('resetHasNewPostfachNachrichten', () => {
    const stateWithoutLink: StateResource<PostfachMailListResource> = createStateResource(createPostfachMailListResource());
    const stateWithLink: StateResource<PostfachMailListResource> = createStateResource(
      createPostfachMailListResource([PostfachMailListLinkRel.RESET_HAS_NEW_POSTFACH_NACHRICHT]),
    );

    beforeEach(() => {
      service._loadAfterResetHasNewPostfachNachrichten = jest.fn().mockReturnValue(of(createEmptyStateResource(true)));
    });

    it('should call loadAfterResetHasNewPostfachNachrichten if link', () => {
      service._resetHasNewPostfachNachrichten(stateWithLink).subscribe();

      expect(service._loadAfterResetHasNewPostfachNachrichten).toHaveBeenCalledWith(stateWithLink.resource);
    });

    it('should return loading state if link', () => {
      const result: Observable<StateResource<PostfachMailListResource>> = service._resetHasNewPostfachNachrichten(stateWithLink);

      expect(result).toBeObservable(singleColdCompleted(createLoadingStateResource<PostfachMailListResource>()));
    });

    it('should return state if no link', () => {
      const result: Observable<StateResource<PostfachMailListResource>> =
        service._resetHasNewPostfachNachrichten(stateWithoutLink);

      expect(result).toBeObservable(singleColdCompleted(stateWithoutLink));
    });
  });

  describe('loadAfterResetHasNewPostfachNachrichten', () => {
    const postfachMailListResource: PostfachMailListResource = createPostfachMailListResource();

    beforeEach(() => {
      service._loadPostfachMailList = jest.fn();
      repository.resetHasNewPostfachNachrichten = jest.fn().mockReturnValue(of({}));
    });

    it('should call repository resetHasNewPostfachNachrichten', () => {
      service._loadAfterResetHasNewPostfachNachrichten(postfachMailListResource);

      expect(repository.resetHasNewPostfachNachrichten).toHaveBeenCalledWith(postfachMailListResource);
    });

    it('should load postfach mail list', () => {
      service._loadAfterResetHasNewPostfachNachrichten(postfachMailListResource);

      expect(service._loadPostfachMailList).toHaveBeenCalled();
    });
  });

  describe('setPostfachNachrichtListAsUnread', () => {
    beforeEach(() => {
      service._navigateToCurrentVorgang = jest.fn();
      repository.markPostfachAsUnread = jest.fn().mockReturnValue(of(null));
    });

    it('should mark postfach as unread', () => {
      service.setPostfachNachrichtListAsUnread();

      expect(repository.markPostfachAsUnread).toHaveBeenCalled();
    });

    it('should navigate to vorgang', () => {
      service.setPostfachNachrichtListAsUnread();

      expect(service._navigateToCurrentVorgang).toHaveBeenCalled();
    });
  });

  describe('navigateToCurrentVorgang', () => {
    it('should navigate to current vorgang', () => {
      const vorgangId: string = faker.string.uuid();
      const routeData: RouteData = {
        urlSegments: [],
        queryParameter: { [VORGANG_WITH_EINGANG_ROUTE_PARAM]: vorgangId },
      };
      navigationFacade.getCurrentRouteData.mockReturnValue(of(routeData));

      service._navigateToCurrentVorgang();

      expect(navigationService.navigateToVorgang).toHaveBeenCalledWith(vorgangId);
    });
  });

  describe('getPostfachMailListByVorgang', () => {
    beforeEach(() => {
      service._loadPostfachMailList = jest.fn();
      service.postfachMailList$.next(createEmptyStateResource());
      service._refreshVorgangSubscription = jest.fn();
    });

    it('should call refresh vorgang subscription', () => {
      service.getPostfachMailListByVorgang();

      expect(service._refreshVorgangSubscription).toHaveBeenCalled();
    });

    it('should call service loadPostfachMailList', () => {
      service.getPostfachMailListByVorgang();

      expect(service._loadPostfachMailList).toHaveBeenCalled();
    });
  });

  describe('loadPostfachMailList', () => {
    beforeEach(() => {
      service._setPostfachMailListLoading = jest.fn();
      service._handleVorgangError = jest.fn();
      service.loadPostfachMailsByVorgang = jest.fn();
    });

    it('should set loading to true', () => {
      service._loadPostfachMailList();

      expect(service._setPostfachMailListLoading).toHaveBeenCalled();
    });

    it('should call vorgang service', () => {
      service._loadPostfachMailList();

      expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
    });

    it('should handle vorgang error', () => {
      service._loadPostfachMailList();

      expect(service._handleVorgangError).toHaveBeenCalledWith(vorgangWithEingangStateResource);
    });

    it('should call loadPostfachMailsByVorgang', () => {
      service._loadPostfachMailList();

      expect(service.loadPostfachMailsByVorgang).toHaveBeenCalledWith(vorgang);
    });
  });

  describe('handleVorgangError', () => {
    beforeEach(() => {
      service.postfachMailList$.next = jest.fn();
    });

    it('should set error', () => {
      const errorState: StateResource<VorgangWithEingangResource> = createErrorStateResource(createProblemDetail());

      service._handleVorgangError(errorState);

      expect(service.postfachMailList$.next).toHaveBeenCalledWith(errorState);
    });

    it('should not set if there is no error', () => {
      const errorState: StateResource<VorgangWithEingangResource> = createStateResource(vorgang);

      service._handleVorgangError(errorState);

      expect(service.postfachMailList$.next).not.toHaveBeenCalled();
    });
  });

  describe('handleSendPostfachMailIsDone', () => {
    const stateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

    beforeEach(() => {
      service._setPollingFalse = jest.fn();
      service._setPostfachMailList = jest.fn();

      commandService.getEffectedResource.mockReturnValue(of(createCommandResource()));
    });

    it('should call command service to get effected resource', () => {
      service._handleSendPostfachMailIsDone(stateResource);

      expect(commandService.getEffectedResource).toHaveBeenCalledWith(stateResource.resource);
    });

    it('should set postfachMailList', () => {
      service._handleSendPostfachMailIsDone(stateResource);

      expect(service._setPostfachMailList).toHaveBeenCalled();
    });

    it('should set polling to false', () => {
      service._handleSendPostfachMailIsDone(stateResource);

      expect(service._setPollingFalse).toHaveBeenCalled();
    });
  });

  describe('isDownloadPdfInProgress', () => {
    it('should call facade to download pdf', (done) => {
      vorgangService.getVorgangWithEingang.mockReturnValue(of(createStateResource(vorgang)));
      postfachFacade.isDownloadPdfInProgress.mockReturnValue(of(true));

      service.isDownloadPdfInProgress().subscribe(() => {
        expect(postfachFacade.downloadPdf).toHaveBeenCalledWith(vorgang);
        done();
      });
    });

    describe('combine latest', () => {
      beforeEach(() => {
        vorgangService.getVorgangWithEingang.mockReturnValue(of(vorgang));
        postfachFacade.isDownloadPdfInProgress = jest.fn().mockReturnValue(true);
      });

      it('should call facade to get isDownloadPdfInProgress', () => {
        service.isDownloadPdfInProgress();

        expect(postfachFacade.isDownloadPdfInProgress).toHaveBeenCalled();
      });

      it('should call vorgang service to get vorgang', () => {
        service.isDownloadPdfInProgress();

        expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
      });
    });
  });

  describe('downloadPdf', () => {
    it('should call facade', () => {
      service.downloadPdf();

      expect(postfachFacade.startDownloadPdf).toHaveBeenCalled();
    });
  });

  describe('getFeatures', () => {
    it('should return features by list stateResource', (done) => {
      service.postfachMailList$ = new BehaviorSubject(createStateResource(createPostfachMailListResource()));

      service.getFeatures().subscribe((features) => {
        expect(features).toEqual(createPostfachFeatures());
        done();
      });
    });
  });

  describe('getSettings', () => {
    it('should return settings by list stateResource', (done) => {
      service.postfachMailList$ = new BehaviorSubject(createStateResource(createPostfachMailListResource()));

      service.getSettings().subscribe((settings) => {
        expect(settings).toEqual(createPostfachSettings());
        done();
      });
    });
  });

  describe('close open dialogs', () => {
    beforeEach(() => {
      service.clearUploadedFiles = jest.fn();
    });

    it('should clear uploaded files', () => {
      service._closeOpenDialogs();

      expect(service.clearUploadedFiles).toHaveBeenCalled();
    });
  });

  describe('listen to vorgang change', () => {
    beforeEach(() => {
      service._handleVorgangChange = jest.fn();
    });

    it('should call vorgang service to get vorgang with eingang', () => {
      service._listenToVorgangChange();

      expect(vorgangService.selectVorgangWithEingang).toHaveBeenCalled();
    });

    it('should call handle vorgang change', () => {
      service._listenToVorgangChange();

      expect(service._handleVorgangChange).toHaveBeenCalledWith(vorgangWithEingangStateResource);
    });
  });

  describe('handle vorgang change', () => {
    it('should set reload flag if list is loaded and state resource is loading', () => {
      service.shouldReload = false;
      service.postfachMailList$.next(createStateResource(createPostfachMailListResource()));

      service._handleVorgangChange({ ...vorgangWithEingangStateResource, loading: true });

      expect(service.shouldReload).toBeTruthy();
    });

    it('should set reload flag if list is loaded and state resource is reloading', () => {
      service.postfachMailList$.next(createStateResource(createPostfachMailListResource()));
      service.shouldReload = false;

      service._handleVorgangChange({ ...vorgangWithEingangStateResource, reload: true });

      expect(service.shouldReload).toBeTruthy();
    });

    describe('on setted reload flag', () => {
      beforeEach(() => {
        service.setPostfachMailOnReload = jest.fn();
        service.shouldReload = true;
      });

      describe('and loaded vorgang resource', () => {
        it('should call set kommentar list reload', () => {
          service._handleVorgangChange(vorgangWithEingangStateResource);

          expect(service.setPostfachMailOnReload).toHaveBeenCalled();
        });

        it('and loaded vorgang resource should call set kommentar list reload', () => {
          service._handleVorgangChange(vorgangWithEingangStateResource);

          expect(service.shouldReload).toBeFalsy();
        });
      });
    });
  });

  describe('get pending resend command', () => {
    const postfachNachricht: PostfachMailResource = createPostfachMailResource();

    const command: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);
    const commandStateResource: StateResource<CommandResource> = createStateResource(command);

    beforeEach(() => {
      vorgangService.getAndPollPendingCommand.mockReturnValue(of(commandStateResource));
      service._refreshPostfachMailList = jest.fn();
    });

    it('should call vorgang service', () => {
      service.getPendingResendCommand(postfachNachricht);

      expect(vorgangService.getAndPollPendingCommand).toHaveBeenCalledWith(getUrl(postfachNachricht));
    });

    it('should call refresh list on command successfully done', () => {
      service.getPendingResendCommand(postfachNachricht).subscribe();

      expect(service._refreshPostfachMailList).toHaveBeenCalled();
    });

    it('should return value', () => {
      const pendingResendCommand$: Observable<StateResource<CommandResource>> =
        service.getPendingResendCommand(postfachNachricht);

      expect(pendingResendCommand$).toBeObservable(singleColdCompleted(commandStateResource));
    });
  });
});
