import { Mock, mock } from '@alfa-client/test-utils';
import { TestBed } from '@angular/core/testing';
import { Actions } from '@ngrx/effects';
import { provideMockActions } from '@ngrx/effects/testing';
import { Action, createAction, props } from '@ngrx/store';
import { provideMockStore } from '@ngrx/store/testing';
import { Resource, ResourceUri } from '@ngxp/rest';
import { ObservableWithSubscriptions, cold, hot } from 'jest-marbles';
import { createApiError } from 'libs/tech-shared/test/error';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { Observable, of } from 'rxjs';
import { ResourceRepository } from '../resource/resource.repository';
import { ApiError } from '../tech.model';
import {
  LoadResourceFailureProps,
  LoadResourceSuccessProps,
  ResourceUriProps,
  SingleResourceLoadActions,
  TypedActionCreator,
  TypedActionCreatorWithProps,
} from './actions';
import { ResourceEffects } from './resource.effects';

describe('ResourceEffects', () => {
  let effects: ResourceEffects<Resource>;

  let actions: Observable<Action>;
  let repository: Mock<ResourceRepository> = mock(ResourceRepository);

  const loadAction: TypedActionCreatorWithProps<ResourceUriProps> = createAction(
    'DummyLoadAction',
    props<ResourceUriProps>(),
  );

  const loadSuccessAction: TypedActionCreatorWithProps<LoadResourceSuccessProps> = createAction(
    'DummyLoadSuccessAction',
    props<LoadResourceSuccessProps>(),
  );

  const loadFailureAction: TypedActionCreatorWithProps<LoadResourceFailureProps> = createAction(
    'DummyLoadFailureAction',
    props<LoadResourceFailureProps>(),
  );

  const clearAction: TypedActionCreator = createAction('DummyClearAction');

  const reloadAction: TypedActionCreator = createAction('DummyReloadAction');

  const resourceActions: SingleResourceLoadActions = {
    loadAction,
    loadSuccessAction,
    loadFailureAction,
    clearAction,
    reloadAction,
  };

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [
        provideMockActions(() => actions),
        provideMockStore(),
        {
          provide: ResourceRepository,
          useValue: repository,
        },
        {
          provide: ResourceEffects,
          useFactory: (actions$: Actions, repository: ResourceRepository) =>
            new ResourceEffects<Resource>(actions$, repository, resourceActions),
          deps: [Actions, ResourceRepository],
        },
      ],
    });

    effects = TestBed.inject(ResourceEffects);
  });

  describe('loadVorgangList', () => {
    const resourceUri: ResourceUri = 'dummyResourceUri';
    const dummyResource: Resource = createDummyResource();

    beforeEach(() => {
      repository.getResource.mockReturnValue(of(dummyResource));
    });

    it('should call repository', (done) => {
      actions = of(loadAction({ resourceUri }));

      effects.loadByUri$.subscribe(() => {
        expect(repository.getResource).toHaveBeenCalledWith(resourceUri);
        done();
      });
    });

    it('should dispatch loadSuccessAction action', () => {
      actions = hot('a', { a: loadAction({ resourceUri }) });

      const expected: ObservableWithSubscriptions = hot('a', {
        a: resourceActions.loadSuccessAction({ resource: dummyResource }),
      });

      expect(effects.loadByUri$).toBeObservable(expected);
    });

    it('should dispatch loadFailureAction action', () => {
      const apiError: ApiError = createApiError();
      const errorResponse: ObservableWithSubscriptions = cold('#', {}, apiError);
      repository.getResource = jest.fn(() => errorResponse);

      actions = hot('a', { a: loadAction({ resourceUri }) });

      const expected: ObservableWithSubscriptions = cold('b', {
        b: resourceActions.loadFailureAction({ error: apiError }),
      });
      expect(effects.loadByUri$).toBeObservable(expected);
    });
  });
});
