import { faker } from '@faker-js/faker';
import { Action } from '@ngrx/store';
import { Resource, ResourceUri } from '@ngxp/rest';
import { createApiError } from 'libs/tech-shared/test/error';
import { singleResourceActions } from 'libs/tech-shared/test/ngrx';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { createEmptyStateResource, createErrorStateResource, createStateResource } from '../resource/resource.util';
import { ApiError } from '../tech.model';
import { LoadResourceFailureProps, LoadResourceSuccessProps, SingleResourceLoadActions } from './actions';
import { SingleResourceReducer } from './resource.reducer';
import { SingleResourceState, initialResourceState } from './state.model';

describe('SingleResourceReducer', () => {
  const resourceActions: SingleResourceLoadActions = singleResourceActions;

  const reducer = new SingleResourceReducer(resourceActions);

  describe('unknown action', () => {
    it('should return current state', () => {
      const action = {} as Action;

      const result = reducer.reducer(initialResourceState, action);

      expect(result).toBe(initialResourceState);
    });
  });
  describe('on load action', () => {
    const resourceUri: ResourceUri = faker.internet.url();

    it('should set loading to true', () => {
      const action = resourceActions.loadAction({ resourceUri });

      const state: SingleResourceState = reducer.reducer(initialResourceState, action);

      expect(state.resource.loading).toBeTruthy();
    });
  });

  describe('on load success action', () => {
    const resource: Resource = createDummyResource();
    const action: LoadResourceSuccessProps & Action<string> = resourceActions.loadSuccessAction({ resource });

    it('should set resource', () => {
      const state: SingleResourceState = reducer.reducer(initialResourceState, action);

      expect(state.resource).toEqual(createStateResource(resource));
    });
  });

  describe('on load failure action', () => {
    const apiError: ApiError = createApiError();
    const action: LoadResourceFailureProps & Action<string> = resourceActions.loadFailureAction({
      error: apiError,
    });

    it('should set apiError', () => {
      const state: SingleResourceState = reducer.reducer(initialResourceState, action);

      expect(state.resource).toEqual(createErrorStateResource(apiError));
    });
  });

  describe('on clear action', () => {
    const action: Action<string> = resourceActions.clearAction();

    it('should clear stateresource', () => {
      const initialState: SingleResourceState = {
        ...initialResourceState,
        resource: createStateResource(createDummyResource()),
      };

      const state: SingleResourceState = reducer.reducer(initialState, action);

      expect(state.resource).toEqual(createEmptyStateResource());
    });
  });

  describe('on reload action', () => {
    const action: Action<string> = resourceActions.reloadAction();

    it('should mark stateresource as reload', () => {
      const initialState: SingleResourceState = {
        ...initialResourceState,
        resource: createEmptyStateResource(),
      };

      const state: SingleResourceState = reducer.reducer(initialState, action);

      expect(state.resource.reload).toBeTruthy();
    });
  });
});
