import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { faker } from '@faker-js/faker';
import { MemoizedSelector, Store } from '@ngrx/store';
import { Resource, ResourceUri } from '@ngxp/rest';
import { singleCold } from 'libs/tech-shared/test/marbles';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { Observable, of } from 'rxjs';
import { StateInfoTestFactory, dummySelector } from '../../../test/ngrx';
import { StateInfo } from '../resource/resource.model';
import { StateResource, createStateResource } from '../resource/resource.util';
import { SingleResourceLoadActions, createSingleResourceActions } from './actions';
import { EffectService } from './effects.service';
import { SingleResourceState } from './state.model';
import { ResourceStateService, SingleResourceStateService } from './state.service';

import * as ResourceSelectors from './selector';

describe('StateService', () => {
  let store: Mock<Store>;
  let effectService: Mock<EffectService>;

  const stateInfo: StateInfo = StateInfoTestFactory.create();

  beforeEach(() => {
    store = mock(Store);
    effectService = mock(EffectService);
  });

  describe('Resource State Service', () => {
    let service: DummyResoureStateService<Resource>;

    beforeEach(() => {
      service = new DummyResoureStateService<Resource>(stateInfo, useFromMock(store), useFromMock(effectService));
    });

    it('should clear resource', () => {
      service.clearResource();

      expect(store.dispatch).toHaveBeenCalledWith(service.actions.clearAction());
    });

    it('should load resource', () => {
      const resourceUri: ResourceUri = faker.word.sample();

      service.loadResource(resourceUri);

      expect(store.dispatch).toHaveBeenCalledWith(service.actions.loadAction({ resourceUri }));
    });

    it('should reload resource', () => {
      service.reloadResource();

      expect(store.dispatch).toHaveBeenCalledWith(service.actions.reloadAction());
    });
  });

  describe('Single Resource State Service', () => {
    let service: SingleResourceStateService<Resource>;

    beforeEach(() => {
      service = new SingleResourceStateService<Resource>(stateInfo, useFromMock(store), useFromMock(effectService));
    });

    it('should init actions', () => {
      service.actions = undefined;

      service.init();

      expect(service.actions).not.toBeUndefined();
    });

    it('should add effects', () => {
      service.init();

      expect(effectService.addSingleResourceEffects).toHaveBeenCalledWith(service.actions);
    });

    describe('select resource', () => {
      const stateValue: MemoizedSelector<object, unknown, (s1: SingleResourceState) => unknown> = dummySelector;
      const dummyStateResource: StateResource<Resource> = createStateResource(createDummyResource());

      let selectResourceSpy: jest.SpyInstance;

      beforeEach(() => {
        store.select.mockReturnValue(of(dummyStateResource));
        selectResourceSpy = jest.spyOn(ResourceSelectors, 'selectResource').mockReturnValue(stateValue);
      });

      it('should call selector', () => {
        service.selectResource();

        expect(selectResourceSpy).toHaveBeenCalledWith(stateInfo);
      });

      it('should call store', () => {
        service.selectResource();

        expect(store.select).toHaveBeenCalledWith(stateValue);
      });

      it('should return from store selected value', () => {
        store.select.mockReturnValue(singleCold(dummyStateResource));

        const selectedResource$: Observable<StateResource<Resource>> = service.selectResource();

        expect(selectedResource$).toBeObservable(singleCold(dummyStateResource));
      });
    });

    describe('exists resource', () => {
      const stateValue: MemoizedSelector<object, unknown, (s1: SingleResourceState) => unknown> = dummySelector;
      const dummyStateResource: StateResource<Resource> = createStateResource(createDummyResource());

      let existsResourceSpy: jest.SpyInstance;

      beforeEach(() => {
        store.select.mockReturnValue(of(dummyStateResource));
        existsResourceSpy = jest.spyOn(ResourceSelectors, 'existResource').mockReturnValue(stateValue);
      });

      it('should call selector', () => {
        service.existsResource();

        expect(existsResourceSpy).toHaveBeenCalledWith(stateInfo);
      });

      it('should call store', () => {
        service.existsResource();

        expect(store.select).toHaveBeenCalledWith(stateValue);
      });

      it('should return from store selected value', () => {
        store.select.mockReturnValue(singleCold(true));

        const exists$: Observable<boolean> = service.existsResource();

        expect(exists$).toBeObservable(singleCold(true));
      });
    });
  });
});

class DummyResoureStateService<Resource> extends ResourceStateService<Resource> {
  actions: SingleResourceLoadActions;

  protected initActions(): void {
    this.actions = createSingleResourceActions(this.stateInfo);
  }
  protected initEffects(): void {
    this.effectService.addSingleResourceEffects(this.actions);
  }
  public selectResource(): Observable<StateResource<Resource>> {
    throw new Error('Method not implemented.');
  }
}
