import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { ResourceUri } from '@ngxp/rest';
import { Observable } from 'rxjs';
import { StateInfo } from '../resource/resource.model';
import { StateResource } from '../resource/resource.util';
import { ResourceActions, SingleResourceLoadActions, createSingleResourceActions } from './actions';
import { EffectService } from './effects.service';

import * as ResourceSelectors from './selector';

@Injectable({ providedIn: 'root' })
export class StateService {
  constructor(
    private store: Store,
    private effectService: EffectService,
  ) {}

  public createSingleResourceService<T>(stateInfo: StateInfo): SingleResourceStateService<T> {
    return new SingleResourceStateService(stateInfo, this.store, this.effectService);
  }
}

export abstract class ResourceStateService<T> {
  actions: ResourceActions;

  constructor(
    protected stateInfo: StateInfo,
    protected store: Store,
    protected effectService: EffectService,
  ) {
    this.init();
  }

  public init(): void {
    this.initActions();
    this.initEffects();
  }

  protected abstract initActions(): void;

  protected abstract initEffects(): void;

  public clearResource(): void {
    this.store.dispatch(this.actions.clearAction());
  }

  public loadResource(resourceUri: ResourceUri): void {
    this.store.dispatch(this.actions.loadAction({ resourceUri }));
  }

  public reloadResource(): void {
    this.store.dispatch(this.actions.reloadAction());
  }

  public abstract selectResource(): Observable<StateResource<T>>;
}

export class SingleResourceStateService<T> extends ResourceStateService<T> {
  actions: SingleResourceLoadActions;

  constructor(
    protected stateInfo: StateInfo,
    protected store: Store,
    protected effectService: EffectService,
  ) {
    super(stateInfo, store, effectService);
  }

  protected initActions(): void {
    this.actions = createSingleResourceActions(this.stateInfo);
  }

  protected initEffects(): void {
    this.effectService.addSingleResourceEffects(this.actions);
  }

  public selectResource(): Observable<StateResource<T>> {
    return this.store.select(ResourceSelectors.selectResource(this.stateInfo));
  }

  public existsResource(): Observable<boolean> {
    return this.store.select(ResourceSelectors.existResource(this.stateInfo));
  }
}
