import { ProblemDetail } from '@alfa-client/tech-shared';
import { mock, Mock, useFromMock } from '@alfa-client/test-utils';
import { Resource } from '@ngxp/rest';
import { cold } from 'jest-marbles';
import { BehaviorSubject, Observable, of, throwError } from 'rxjs';
import { DummyLinkRel, DummyListLinkRel } from '../../../test/dummy';
import { createProblemDetail } from '../../../test/error';
import { multipleCold, singleCold, singleColdCompleted } from '../../../test/marbles';
import { createDummyListResource, createDummyResource } from '../../../test/resource';
import { ApiListResourceService } from './api-list-resource.service';
import { CreateResourceData, LinkRelationName, ListItemResource, ListResourceServiceConfig } from './resource.model';
import { ResourceRepository } from './resource.repository';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createLoadingStateResource,
  createStateResource,
  ListResource,
  StateResource,
} from './resource.util';

describe('ListResourceService', () => {
  let service: ApiListResourceService<Resource, ListResource, ListItemResource>;
  let config: ListResourceServiceConfig<Resource>;
  let resourceRepository: Mock<ResourceRepository>;

  const getLinkRel: LinkRelationName = DummyListLinkRel.LIST;
  const createLinkRel: LinkRelationName = DummyLinkRel.DUMMY;
  const listResourceListLinkRel: LinkRelationName = DummyListLinkRel.LIST;
  const listResource: ListResource = createDummyListResource([getLinkRel, createLinkRel]);

  const baseResource: Resource = createDummyResource();
  const baseStateResource: StateResource<Resource> = createStateResource(baseResource);
  const baseResourceSubj: BehaviorSubject<StateResource<Resource>> = new BehaviorSubject<StateResource<Resource>>(
    baseStateResource,
  );

  beforeEach(() => {
    config = {
      baseResource: baseResourceSubj,
      getLinkRel,
      listResourceListLinkRel,
      create: { linkRel: createLinkRel },
    };
    resourceRepository = mock(ResourceRepository);

    service = new ApiListResourceService(config, useFromMock(resourceRepository));
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('create', () => {
    const toCreate: unknown = {};

    beforeEach(() => {
      service.listResource.value.resource = listResource;
    });

    it('should throw error if link is not present', () => {
      service.listResource.next(createStateResource(createDummyListResource()));

      expect(() => service.create(toCreate)).toThrowError('No creation link exists.');
    });

    it('should throw error if list resource is empty', () => {
      service.listResource.next(createEmptyStateResource());

      expect(() => service.create(toCreate)).toThrowError('No list resource available.');
    });

    it('should trigger resourceLoader', () => {
      service.create(toCreate).subscribe();

      const createResourceData: CreateResourceData<ListResource> = {
        resource: listResource,
        toCreate,
        linkRel: createLinkRel,
      };
      expect(resourceRepository.createResource).toHaveBeenCalledWith(createResourceData);
    });

    it('should return value', () => {
      const returnResource: Resource = createDummyResource();

      resourceRepository.createResource.mockReturnValue(singleCold(returnResource, '-a'));

      const createdStateResource$: Observable<StateResource<Resource>> = service.create(toCreate);

      expect(createdStateResource$).toBeObservable(
        multipleCold(createEmptyStateResource(true), createStateResource(returnResource)),
      );
    });

    it('should handle error', () => {
      const error: ProblemDetail = createProblemDetail();
      service._handleError = jest.fn();
      resourceRepository.createResource.mockReturnValue(throwError(() => error));

      service.create(toCreate).subscribe();

      expect(service._handleError).toHaveBeenCalledWith(error);
    });

    it('should emit on error', () => {
      const error: ProblemDetail = createProblemDetail();
      service._handleError = jest.fn().mockReturnValue(of(createErrorStateResource(error)));
      resourceRepository.createResource.mockReturnValue(cold('-#', null, error));

      expect(service.create(toCreate)).toBeObservable(
        cold('a(b|)', { a: createLoadingStateResource(), b: createErrorStateResource(error) }),
      );
    });
  });

  describe('handle error', () => {
    it('should return error state resource on unprocessable entity', () => {
      const error: ProblemDetail = createProblemDetail();

      expect(service._handleError(error)).toBeObservable(singleColdCompleted(createErrorStateResource(error)));
    });

    it('should throw error', () => {
      const error: ProblemDetail = { ...createProblemDetail(), status: 500 };

      expect(service._handleError(error)).toBeObservable(cold('#', null, error));
    });
  });
});
