import { Resource } from '@ngxp/rest';
import { catchError, first, map, Observable, of, startWith, switchMap, throwError } from 'rxjs';
import { isUnprocessableEntity } from '../http.util';
import { ProblemDetail } from '../tech.model';
import { ResourceListService } from './list-resource.service';
import { CreateResourceData, ListItemResource, ListResourceServiceConfig } from './resource.model';
import { ResourceRepository } from './resource.repository';
import {
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
  ListResource,
  StateResource,
} from './resource.util';

export class ApiListResourceService<
  B extends Resource,
  T extends ListResource,
  I extends ListItemResource,
> extends ResourceListService<B, T, I> {
  constructor(
    protected config: ListResourceServiceConfig<B>,
    protected repository: ResourceRepository<I>,
  ) {
    super(config, repository);
  }

  public create(toCreate: unknown): Observable<StateResource<I>> {
    this.verifyBeforeCreation();
    return this.getListResourceAsObservable().pipe(
      first(),
      switchMap((listResource: StateResource<T>) =>
        this.repository.createResource(this.buildCreateResourceData(toCreate, this.config.create.linkRel, listResource.resource)),
      ),
      map((listItemResource: I) => createStateResource(listItemResource)),
      catchError((error: ProblemDetail) => this._handleError(error)),
      startWith(createEmptyStateResource<I>(true)),
    );
  }

  _handleError(error: ProblemDetail): Observable<StateResource<I>> {
    if (isUnprocessableEntity(error.status)) {
      return of(createErrorStateResource(error));
    }
    return throwError(() => error);
  }

  private verifyBeforeCreation(): void {
    this.verifyValidListResource();
    this.throwErrorOn(!this.isCreateLinkPresent(), 'No creation link exists.');
  }

  private isCreateLinkPresent(): boolean {
    return this.hasLinkRel(this.config.create.linkRel);
  }

  private buildCreateResourceData(toCreate: any, linkRel: string, resource: T): CreateResourceData<T> {
    return {
      resource,
      linkRel,
      toCreate,
    };
  }
}
