/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { HttpErrorResponse } from '@angular/common/http';
import { fakeAsync, tick } from '@angular/core/testing';
import { Resource } from '@ngxp/rest';
import { lastValueFrom, Observable, of, throwError } from 'rxjs';
import { multipleCold, singleCold } from '../../../test//marbles';
import { createProblemDetail } from '../../../test/error';
import { createDummyResource } from '../../../test/resource';
import { SingleResourceStateService, StateService } from '../ngrx/state.service';
import { HttpError, ProblemDetail } from '../tech.model';
import { ApiResourceService } from './api-resource.service';
import { LinkRelationName, ResourceServiceConfig, SaveResourceData } from './resource.model';
import { ResourceRepository } from './resource.repository';
import { createEmptyStateResource, createErrorStateResource, createStateResource, StateResource } from './resource.util';

describe('ApiResourceService', () => {
  let service: ApiResourceService<Resource, Resource>;
  let config: ResourceServiceConfig<Resource>;
  let repository: Mock<ResourceRepository>;
  let stateService: Mock<StateService>;

  const configResource: Resource = createDummyResource();
  const configStateResource: StateResource<Resource> = createStateResource(configResource);
  const configStateResource$: Observable<StateResource<Resource>> = of(configStateResource);

  const editLinkRel: string = 'dummyEditLinkRel';
  const getLinkRel: LinkRelationName = 'dummyGetLinkRel';
  const deleteLinkRel: LinkRelationName = 'dummyDeleteLinkRel';

  beforeEach(() => {
    config = {
      stateInfo: { name: 'dummyStateName', path: 'dummyStatePath' },
      baseResource: configStateResource$,
      getLinkRel,
      edit: { linkRel: editLinkRel },
      delete: { linkRel: deleteLinkRel },
    };
    repository = mock(ResourceRepository);
    stateService = mock(StateService);
    stateService.createSingleResourceService = jest.fn().mockReturnValue(mock(SingleResourceStateService));

    service = new ApiResourceService(config, useFromMock(stateService), useFromMock(repository));
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('save and patch', () => {
    const dummyToSave: unknown = {};
    const loadedResource: Resource = createDummyResource();

    const resourceWithEditLinkRel: Resource = createDummyResource([editLinkRel]);

    beforeEach(() => {
      service.selectResource = jest.fn().mockReturnValue(of(createStateResource(resourceWithEditLinkRel)));
      service.resourceStateService.reloadResource = jest.fn();
    });

    describe('save', () => {
      it('should call repository', fakeAsync(() => {
        repository.save.mockReturnValue(of(loadedResource));

        service.save(dummyToSave).subscribe();
        tick();

        const expectedSaveResourceData: SaveResourceData<Resource> = {
          resource: resourceWithEditLinkRel,
          linkRel: editLinkRel,
          toSave: dummyToSave,
        };
        expect(repository.save).toHaveBeenCalledWith(expectedSaveResourceData);
      }));

      it('should return saved object', () => {
        repository.save.mockReturnValue(singleCold(loadedResource, '-a'));

        const saved$: Observable<StateResource<Resource | HttpError>> = service.save(dummyToSave);

        expect(saved$).toBeObservable(multipleCold(createEmptyStateResource(true), createStateResource(loadedResource)));
      });

      it('should call handleError', () => {
        const errorResponse: ProblemDetail = createProblemDetail();
        repository.save.mockReturnValue(throwError(() => errorResponse));
        service.handleError = jest.fn();

        service.save(<any>{}).subscribe();

        expect(service.handleError).toHaveBeenCalledWith(errorResponse);
      });

      it('should update state resource subject', fakeAsync(() => {
        service.resourceStateService.reloadResource = jest.fn();
        repository.save.mockReturnValue(of(loadedResource));

        service.save(dummyToSave).subscribe();
        tick();

        expect(service.resourceStateService.reloadResource).toHaveBeenCalled();
      }));
    });

    describe('patch', () => {
      it('should call repository', fakeAsync(() => {
        repository.patch.mockReturnValue(of(loadedResource));

        service.patch(dummyToSave).subscribe();
        tick();

        const expectedSaveResourceData: SaveResourceData<Resource> = {
          resource: resourceWithEditLinkRel,
          linkRel: editLinkRel,
          toSave: dummyToSave,
        };
        expect(repository.patch).toHaveBeenCalledWith(expectedSaveResourceData);
      }));

      it('should return patchd object', () => {
        repository.patch.mockReturnValue(singleCold(loadedResource, '-a'));

        const patchd$: Observable<StateResource<Resource | HttpError>> = service.patch(dummyToSave);

        expect(patchd$).toBeObservable(multipleCold(createEmptyStateResource(true), createStateResource(loadedResource)));
      });

      it('should call handleError', () => {
        const errorResponse: ProblemDetail = createProblemDetail();
        repository.patch.mockReturnValue(throwError(() => errorResponse));
        service.handleError = jest.fn();

        service.patch(<any>{}).subscribe();

        expect(service.handleError).toHaveBeenCalledWith(errorResponse);
      });

      it('should update state resource subject', fakeAsync(() => {
        service.resourceStateService.reloadResource = jest.fn();
        repository.patch.mockReturnValue(of(loadedResource));

        service.patch(dummyToSave).subscribe();
        tick();

        expect(service.resourceStateService.reloadResource).toHaveBeenCalled();
      }));
    });
  });

  describe('handleError', () => {
    it('should return error stateresource on problem unprocessable entity', (done: jest.DoneCallback) => {
      const error: ProblemDetail = createProblemDetail();

      service.handleError(<HttpErrorResponse>(<any>error)).subscribe((responseError: StateResource<unknown>) => {
        expect(responseError).toEqual(createErrorStateResource(error));
        done();
      });
    });

    it('should rethrow error', () => {
      const error: HttpErrorResponse = <HttpErrorResponse>{
        status: 500,
        statusText: 'Internal Server Error',
      };

      const thrownError$: Observable<StateResource<unknown>> = service.handleError(error);

      expect.assertions(1);
      expect(lastValueFrom(thrownError$)).rejects.toThrowError('Internal Server Error');
    });
  });
});
