/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse } from '@angular/common/http';
import { Resource } from '@ngxp/rest';
import { Observable, catchError, map, of, startWith, switchMap, tap, throwError } from 'rxjs';
import { isUnprocessableEntity } from '../http.util';
import { StateService } from '../ngrx/state.service';
import { HttpError } from '../tech.model';
import { ResourceServiceConfig } from './resource.model';
import { ResourceRepository } from './resource.repository';
import { ResourceService } from './resource.service';
import { StateResource, createEmptyStateResource, createErrorStateResource, createStateResource } from './resource.util';

export class ApiResourceService<B extends Resource, T extends Resource> extends ResourceService<B, T> {
  constructor(
    protected config: ResourceServiceConfig<B>,
    protected stateService: StateService,
    protected repository: ResourceRepository,
  ) {
    super(config, stateService);
  }

  public save(toSave: unknown): Observable<StateResource<T>> {
    return this.selectResource().pipe(
      switchMap((stateResource: StateResource<T>) =>
        this.doSave(stateResource.resource, toSave).pipe(
          tap(() => this.reloadResource()),
          map((value: T) => createStateResource<T>(value)),
          catchError((errorResponse: HttpErrorResponse) => this.handleError(errorResponse)),
        ),
      ),
      startWith(createEmptyStateResource<T>(true)),
    );
  }

  doSave(resource: T, toSave: unknown): Observable<T> {
    return this.repository.save<T>({
      resource,
      linkRel: this.config.edit.linkRel,
      toSave,
    });
  }

  public patch(toPatch: unknown): Observable<StateResource<T>> {
    return this.selectResource().pipe(
      switchMap((stateResource: StateResource<T>) =>
        this.doPatch(stateResource.resource, toPatch).pipe(
          tap(() => this.reloadResource()),
          map((value: T) => createStateResource<T>(value)),
          catchError((errorResponse: HttpErrorResponse) => this.handleError(errorResponse)),
        ),
      ),
      startWith(createEmptyStateResource<T>(true)),
    );
  }

  doPatch(resource: T, toPatch: unknown): Observable<T> {
    return this.repository.patch<T>({
      resource,
      linkRel: this.config.edit.linkRel,
      toSave: toPatch,
    });
  }

  handleError(errorResponse: HttpErrorResponse): Observable<StateResource<T>> {
    if (isUnprocessableEntity(errorResponse.status)) {
      return of(createErrorStateResource((<any>errorResponse) as HttpError));
    }
    return throwError(() => errorResponse);
  }
}
