import { mock, useFromMock } from '@alfa-client/test-utils';
import { fakeAsync, tick } from '@angular/core/testing';
import { Resource, getUrl } from '@ngxp/rest';
import { Observable, of } from 'rxjs';
import { createDummyResource } from '../../../test/resource';
import { SingleResourceStateService } from '../ngrx/state.service';
import { ResourceLoader } from './resource.loader';
import { LinkRelationName, ResourceServiceConfig } from './resource.model';
import { StateResource, createEmptyStateResource, createStateResource } from './resource.util';

import { singleColdCompleted } from 'libs/tech-shared/test/marbles';
import * as ResourceUtil from './resource.util';

describe('ResourceLoader', () => {
  let service: ResourceLoader<Resource, Resource>;
  let config: ResourceServiceConfig<Resource>;
  let resourceStateService: SingleResourceStateService<Resource>;

  const editLinkRel: string = 'dummyEditLinkRel';
  const getLinkRel: LinkRelationName = 'dummyGetLinkRel';
  const deleteLinkRel: LinkRelationName = 'dummyDeleteLinkRel';

  const configResource: Resource = createDummyResource([getLinkRel]);
  const configStateResource: StateResource<Resource> = createStateResource(configResource);
  const configStateResource$: Observable<StateResource<Resource>> = of(configStateResource);

  const dummyResource: Resource = createDummyResource();
  const dummyStateResource: StateResource<Resource> = createStateResource(dummyResource);

  beforeEach(() => {
    config = {
      baseResource: configStateResource$,
      getLinkRel,
      edit: { linkRel: editLinkRel },
      delete: { linkRel: deleteLinkRel },
    };
    resourceStateService = <any>mock(SingleResourceStateService);

    service = new ResourceLoader(config, useFromMock(<any>resourceStateService));
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('get', () => {
    let isInvalidResourceCombinationSpy: jest.SpyInstance;

    beforeEach(() => {
      resourceStateService.selectResource = jest.fn().mockReturnValue(of(dummyStateResource));

      service.handleResourceChanges = jest.fn();
      isInvalidResourceCombinationSpy = jest
        .spyOn(ResourceUtil, 'isInvalidResourceCombination')
        .mockReturnValue(true);
    });

    it('should handle config resource changed', fakeAsync(() => {
      service.get().subscribe();
      tick();

      expect(service.handleResourceChanges).toHaveBeenCalledWith(
        dummyStateResource,
        configResource,
      );
    }));

    it('should call isInvalidResourceCombinationSpy', fakeAsync(() => {
      service.get().subscribe();
      tick();

      expect(isInvalidResourceCombinationSpy).toHaveBeenCalled();
    }));

    it('should return initial value', () => {
      const apiRootStateResource$: Observable<StateResource<Resource>> = service.get();

      expect(apiRootStateResource$).toBeObservable(
        singleColdCompleted(createEmptyStateResource(true)),
      );
    });
  });

  describe('handle resource changes', () => {
    const stateResource: StateResource<Resource> = createStateResource(createDummyResource());
    const changedConfigResource: Resource = createDummyResource();

    describe('on different config resource', () => {
      beforeEach(() => {
        service.handleConfigResourceChanges = jest.fn();
      });

      it('should update state resource by config resource', () => {
        service.configResource = createDummyResource();

        service.handleResourceChanges(stateResource, changedConfigResource);

        expect(service.handleConfigResourceChanges).toHaveBeenCalled();
      });
    });

    describe('on same config resource', () => {
      const stateResource: StateResource<Resource> = createStateResource(createDummyResource());

      beforeEach(() => {
        service.configResource = configResource;
      });

      it('should call shouldLoadResource', () => {
        service.shouldLoadResource = jest.fn();

        service.handleResourceChanges(stateResource, configResource);

        expect(service.shouldLoadResource).toHaveBeenCalledWith(stateResource, configResource);
      });

      it('should load resource', () => {
        service.shouldLoadResource = jest.fn().mockReturnValue(true);
        service.loadResource = jest.fn();

        service.handleResourceChanges(stateResource, configResource);

        expect(service.loadResource).toHaveBeenCalledWith(configResource);
      });

      it('should NOT load resource', () => {
        service.loadResource = jest.fn();
        service.shouldLoadResource = jest.fn().mockReturnValue(false);

        service.handleResourceChanges(stateResource, configResource);

        expect(service.loadResource).not.toHaveBeenCalled();
      });
    });
  });

  describe('handle config resource changes', () => {
    const stateResource: StateResource<Resource> = createStateResource(createDummyResource());

    beforeEach(() => {
      service.updateStateResourceByConfigResource = jest.fn();
    });

    it('should update configresource', () => {
      service.configResource = createDummyResource();

      service.handleConfigResourceChanges(stateResource, configResource);

      expect(service.configResource).toBe(configResource);
    });

    describe('on stable stateresource', () => {
      beforeEach(() => {
        jest.spyOn(ResourceUtil, 'isStateResoureStable').mockReturnValue(true);
        service.updateStateResourceByConfigResource = jest.fn();
      });

      it('should update stateresource by configresource', () => {
        service.handleConfigResourceChanges(stateResource, configResource);

        expect(service.updateStateResourceByConfigResource).toHaveBeenCalledWith(
          stateResource,
          configResource,
        );
      });
    });

    describe('on unstable stateresource', () => {
      beforeEach(() => {
        jest.spyOn(ResourceUtil, 'isStateResoureStable').mockReturnValue(false);
      });

      it('should NOT update stateresource by configresource', () => {
        service.updateStateResourceByConfigResource = jest.fn();

        service.handleConfigResourceChanges(stateResource, configResource);

        expect(service.updateStateResourceByConfigResource).not.toHaveBeenCalled();
      });
    });
  });

  describe('update stateresource by configresource', () => {
    it('should check if should clear stateresource', () => {
      resourceStateService.clearResource = jest.fn();
      service.shouldClearStateResource = jest.fn().mockRejectedValue(true);

      service.updateStateResourceByConfigResource(dummyStateResource, configResource);

      expect(service.shouldClearStateResource).toHaveBeenCalled();
    });

    it('should clear resource if necessary', () => {
      resourceStateService.clearResource = jest.fn();
      resourceStateService.selectResource = jest
        .fn()
        .mockReturnValue(of(createStateResource(createDummyResource())));
      service.shouldClearStateResource = jest.fn().mockReturnValue(true);

      service.updateStateResourceByConfigResource(dummyStateResource, configResource);

      expect(resourceStateService.clearResource).toHaveBeenCalled();
    });

    describe('on NOT clearing stateresource', () => {
      beforeEach(() => {
        service.shouldClearStateResource = jest.fn().mockReturnValue(false);
      });

      it('should check if get link exists', () => {
        service.hasGetLink = jest.fn();

        service.updateStateResourceByConfigResource(dummyStateResource, configResource);

        expect(service.hasGetLink).toHaveBeenCalledWith(configResource);
      });

      it('should load resource on existing get link', () => {
        service.hasGetLink = jest.fn().mockReturnValue(true);
        service.loadResource = jest.fn();

        service.updateStateResourceByConfigResource(dummyStateResource, configResource);

        expect(service.loadResource).toHaveBeenCalledWith(configResource);
      });
    });
  });

  describe('should clear stateresource', () => {
    describe('on existing stateresource', () => {
      beforeEach(() => {
        resourceStateService.selectResource = jest.fn().mockReturnValue(of(dummyStateResource));
      });

      it('should return true if configresource is null', () => {
        const shouldClear: boolean = service.shouldClearStateResource(dummyStateResource, null);

        expect(shouldClear).toBeTruthy();
      });

      it('should return true if configresource has no get link', () => {
        const shouldClear: boolean = service.shouldClearStateResource(
          dummyStateResource,
          createDummyResource(),
        );

        expect(shouldClear).toBeTruthy();
      });
    });

    describe('on empty stateresource', () => {
      it('should return false', () => {
        const shouldClear: boolean = service.shouldClearStateResource(
          ResourceUtil.createEmptyStateResource(),
          null,
        );

        expect(shouldClear).toBeFalsy();
      });

      it('should return false if configresource has no get link', () => {
        const shouldClear: boolean = service.shouldClearStateResource(
          ResourceUtil.createEmptyStateResource(),
          createDummyResource(),
        );

        expect(shouldClear).toBeFalsy();
      });
    });
  });

  describe('should load resource', () => {
    const resource: Resource = createDummyResource();
    const stateResource: StateResource<Resource> = createStateResource(resource);

    let isLoadingRequiredSpy: jest.SpyInstance<boolean>;

    beforeEach(() => {
      isLoadingRequiredSpy = jest.spyOn(ResourceUtil, 'isLoadingRequired');
    });

    it('should return true on existing configresource and loading is required', () => {
      isLoadingRequiredSpy.mockReturnValue(true);

      const shouldLoad: boolean = service.shouldLoadResource(stateResource, configResource);

      expect(shouldLoad).toBeTruthy();
    });

    it('should call isLoadingRequired', () => {
      service.shouldLoadResource(stateResource, configResource);

      expect(isLoadingRequiredSpy).toBeCalledWith(stateResource);
    });

    it('should return false if configresource exists but loading is NOT required', () => {
      isLoadingRequiredSpy.mockReturnValue(false);

      const shouldLoad: boolean = service.shouldLoadResource(stateResource, configResource);

      expect(shouldLoad).toBeFalsy();
    });
  });

  describe('load resource', () => {
    it('should call resource state service load resource', () => {
      resourceStateService.loadResource = jest.fn();

      service.loadResource(configResource);

      expect(resourceStateService.loadResource).toHaveBeenCalledWith(
        getUrl(configResource, getLinkRel),
      );
    });
  });
});
