import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { faker } from '@faker-js/faker';
import { Resource } from '@ngxp/rest';
import { singleColdCompleted } from 'libs/tech-shared/test/marbles';
import { Observable, of } from 'rxjs';
import { createDummyResource } from '../../../test/resource';
import { SingleResourceStateService, StateService } from '../ngrx/state.service';
import { LinkRelationName, ResourceServiceConfig } from './resource.model';
import { ResourceService } from './resource.service';
import { StateResource, createStateResource } from './resource.util';

describe('ResourceService', () => {
  let service: DummyResourceService<Resource, Resource>;
  let config: ResourceServiceConfig<Resource>;
  let stateService: Mock<StateService>;

  const configResource: Resource = createDummyResource();
  const configStateResource: StateResource<Resource> = createStateResource(configResource);
  const configStateResource$: Observable<StateResource<Resource>> = of(configStateResource);

  const dummyResource: Resource = createDummyResource();
  const dummyStateResource: StateResource<Resource> = createStateResource(dummyResource);

  const editLinkRel: string = 'dummyEditLinkRel';
  const getLinkRel: LinkRelationName = 'dummyGetLinkRel';
  const deleteLinkRel: LinkRelationName = 'dummyDeleteLinkRel';

  beforeEach(() => {
    config = {
      stateInfo: { name: 'dummyFeatureState', path: 'dummyPath' },
      baseResource: configStateResource$,
      getLinkRel,
      edit: { linkRel: editLinkRel },
      delete: { linkRel: deleteLinkRel },
    };
    stateService = mock(StateService);
    stateService.createSingleResourceService = jest.fn().mockReturnValue(mock(SingleResourceStateService));

    service = new DummyResourceService(config, useFromMock(stateService));
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('should create state service', () => {
    expect(service.resourceStateService).toBeTruthy();
  });

  it('should create resource loader', () => {
    expect(service.resourceLoader).toBeTruthy();
  });

  describe('get', () => {
    beforeEach(() => {
      service.resourceLoader.get = jest.fn().mockReturnValue(of(dummyStateResource));
    });

    it('should call resource loader', () => {
      service.get();

      expect(service.resourceLoader.get).toHaveBeenCalled();
    });

    it('should return from loader returned value', () => {
      const stateResource$: Observable<StateResource<Resource>> = service.get();

      expect(stateResource$).toBeObservable(singleColdCompleted(dummyStateResource));
    });
  });

  describe('reload resource', () => {
    it('should call resource state service to reload resource', () => {
      service.resourceStateService.reloadResource = jest.fn();

      service.reloadResource();

      expect(service.resourceStateService.reloadResource).toHaveBeenCalled();
    });
  });

  describe('exist resource', () => {
    const EXISTS_RETURN_VALUE: boolean = true;

    beforeEach(() => {
      service.resourceStateService.existsResource = jest.fn().mockReturnValue(of(EXISTS_RETURN_VALUE));
    });

    it('should call resource state service', () => {
      service.existResource();

      expect(service.resourceStateService.existsResource).toHaveBeenCalled();
    });

    it('should return true', () => {
      const existsValueFromService$: Observable<boolean> = service.existResource();

      expect(existsValueFromService$).toBeObservable(singleColdCompleted(EXISTS_RETURN_VALUE));
    });
  });

  describe('select resource', () => {
    beforeEach(() => {
      service.resourceStateService.selectResource = jest.fn().mockReturnValue(of(dummyStateResource));
    });

    it('should call resource state service', () => {
      service.selectResource();

      expect(service.resourceStateService.selectResource).toHaveBeenCalled();
    });

    it('should return resource', () => {
      const resource$: Observable<StateResource<Resource>> = service.selectResource();

      expect(resource$).toBeObservable(singleColdCompleted(dummyStateResource));
    });
  });

  describe('load by resource uri', () => {
    const dummyUri: string = faker.word.sample();

    it('should call resource state service to clear resource', () => {
      service.resourceStateService.loadResource = jest.fn();

      service.loadByResourceUri(dummyUri);

      expect(service.resourceStateService.loadResource).toHaveBeenCalledWith(dummyUri);
    });
  });

  describe('clear resource', () => {
    it('should call resource state service to clear resource', () => {
      service.resourceStateService.clearResource = jest.fn();

      service.clearResource();

      expect(service.resourceStateService.clearResource).toHaveBeenCalled();
    });
  });

  describe('getConfig', () => {
    it('should return config', () => {
      const configFromService: ResourceServiceConfig<Resource> = service.getConfig();

      expect(configFromService).toEqual(config);
    });
  });
});

export class DummyResourceService<B extends Resource, T extends Resource> extends ResourceService<B, T> {
  constructor(
    protected config: ResourceServiceConfig<B>,
    protected stateService: StateService,
  ) {
    super(config, stateService);
  }
}
