/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Resource } from '@ngxp/rest';
import { DummyListLinkRel } from '../../../test/dummy';
import { createApiError } from '../../../test/error';
import { createDummyListResource, createDummyResource, toResource } from '../../../test/resource';
import {
  StateResource,
  containsLoading,
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
  getEmbeddedResources,
  getSuccessfullyLoaded,
  isInvalidResourceCombination,
  isLoaded,
  isLoadingRequired,
  isNotLoading,
  isValidStateResource,
} from './resource.util';

describe('resource util', () => {
  describe('isLoadingRequired', () => {
    it('should return true if not loaded', () => {
      const isRequired: boolean = isLoadingRequired(createEmptyStateResource());

      expect(isRequired).toBe(true);
    });

    it('should retrun false if already loading', () => {
      const isRequired: boolean = isLoadingRequired(createEmptyStateResource(true));

      expect(isRequired).toBe(false);
    });

    it('should return false on reload if already loading', () => {
      const isRequired: boolean = isLoadingRequired({
        ...createEmptyStateResource(true),
        reload: true,
      });

      expect(isRequired).toBe(false);
    });

    it('should return true if reload requested', () => {
      const isRequired: boolean = isLoadingRequired({
        ...createStateResource(toResource({})),
        reload: true,
      });

      expect(isRequired).toBe(true);
    });
  });

  describe('isLoaded', () => {
    it('should return false if not loaded', () => {
      const loaded: boolean = isLoaded(createEmptyStateResource());

      expect(loaded).toBe(false);
    });

    it('should retrun false if loading', () => {
      const loaded: boolean = isLoaded(createEmptyStateResource(true));

      expect(loaded).toBe(false);
    });

    it('should return false on reload if already loading', () => {
      const loaded: boolean = isLoaded({ ...createEmptyStateResource(true), reload: true });

      expect(loaded).toBe(false);
    });

    it('should return false if reload required', () => {
      const loaded: boolean = isLoaded({ ...createStateResource(toResource({})), reload: true });

      expect(loaded).toBe(false);
    });

    it('should return true if resource is present and loaded', () => {
      const loaded: boolean = isLoaded(createStateResource(toResource({})));

      expect(loaded).toBe(true);
    });
  });

  describe('isValidStateResouce', () => {
    it('should return false if not loaded', () => {
      const isValid: boolean = isValidStateResource(createEmptyStateResource());

      expect(isValid).toBe(false);
    });

    it('should retrun false if loaded but is null', () => {
      const isValid: boolean = isValidStateResource(createStateResource(null));

      expect(isValid).toBe(false);
    });

    it('should return false on loading', () => {
      const isValid: boolean = isValidStateResource(createEmptyStateResource(true));

      expect(isValid).toBe(false);
    });

    it('should retrun true if already loaded', () => {
      const isValid: boolean = isValidStateResource({
        ...createStateResource(createDummyResource()),
      });

      expect(isValid).toBe(true);
    });
  });

  describe('getEmbeddedResources', () => {
    it('should return empty array if state resource null', () => {
      const embedded = getEmbeddedResources(null, null);

      expect(embedded).toEqual([]);
    });

    it('should return empty array if resource null', () => {
      const embedded = getEmbeddedResources(createEmptyStateResource(), null);

      expect(embedded).toEqual([]);
    });

    it('should return null if embedded relation does not exist', () => {
      const stateResource = createStateResource<Resource>(createDummyResource());

      const embedded = getEmbeddedResources<string[]>(stateResource, 'users');

      expect(embedded).toBeNull();
    });

    it('should return embedded resource', () => {
      const dummyListResource = createDummyListResource();
      const stateResource = createStateResource<Resource>(dummyListResource);

      const embedded = getEmbeddedResources<Resource>(stateResource, DummyListLinkRel.LIST);

      expect(embedded).toEqual(dummyListResource._embedded[DummyListLinkRel.LIST]);
    });
  });

  describe('containsLoading', () => {
    it('should return true', () => {
      const contains = containsLoading([createEmptyStateResource(true), createEmptyStateResource(false)]);

      expect(contains).toBeTruthy();
    });

    it('should return false', () => {
      const contains = containsLoading([createEmptyStateResource(false), createEmptyStateResource(false)]);

      expect(contains).toBeFalsy();
    });
  });

  describe('getLoaded', () => {
    let loadedStateResource: StateResource<Resource>;

    beforeEach(() => {
      loadedStateResource = createStateResource(createDummyResource());
    });

    it('should return loaded', () => {
      const loaded = getSuccessfullyLoaded([createEmptyStateResource<Resource>(true), loadedStateResource]);

      expect(loaded).toEqual([loadedStateResource]);
    });
  });

  describe('getSuccessfullyLoaded', () => {
    let errorStateResource: StateResource<Resource>;
    let loadedStateResource: StateResource<Resource>;
    let loadingStateResource: StateResource<Resource>;

    beforeEach(() => {
      errorStateResource = createErrorStateResource(createApiError());
      loadedStateResource = createStateResource(createDummyResource());
      loadingStateResource = createStateResource(createDummyResource(), true);
    });

    it('should not return error state resource', () => {
      const successfullyLoaded = getSuccessfullyLoaded([errorStateResource, loadedStateResource]);

      expect(successfullyLoaded).toEqual([loadedStateResource]);
    });

    it('should not return loading state resource', () => {
      const successfullyLoaded = getSuccessfullyLoaded([loadingStateResource, loadedStateResource]);

      expect(successfullyLoaded).toEqual([loadedStateResource]);
    });
  });

  describe('is invalid resource combination', () => {
    it('should return true on loaded stateResource while configResource is null', () => {
      const stateResource: StateResource<Resource> = createStateResource(createDummyResource());

      const isInvalidCombination: boolean = isInvalidResourceCombination<Resource, Resource>(stateResource, null);

      expect(isInvalidCombination).toBeTruthy();
    });

    it('should return true on non loaded stateResource while configResource exists', () => {
      const stateResource: StateResource<Resource> = createEmptyStateResource();
      const configResource: Resource = createDummyResource();

      const isInvalidCombination: boolean = isInvalidResourceCombination<Resource, Resource>(stateResource, configResource);

      expect(isInvalidCombination).toBeTruthy();
    });

    it('should return false on loading stateResource', () => {
      const stateResource: StateResource<Resource> = {
        ...createStateResource(createDummyResource()),
        loading: true,
      };
      const configResource: Resource = createDummyResource();

      const isInvalidCombination: boolean = isInvalidResourceCombination<Resource, Resource>(stateResource, configResource);

      expect(isInvalidCombination).toBeFalsy();
    });

    it('should return true on mark as reload stateResource', () => {
      const stateResource: StateResource<Resource> = {
        ...createStateResource(createDummyResource()),
        reload: true,
      };
      const configResource: Resource = createDummyResource();

      const isInvalidCombination: boolean = isInvalidResourceCombination<Resource, Resource>(stateResource, configResource);

      expect(isInvalidCombination).toBeTruthy();
    });
  });

  describe('isNotLoading', () => {
    it('should return true for empty state resource', () => {
      const notLoading: boolean = isNotLoading(createEmptyStateResource());

      expect(notLoading).toBeTruthy();
    });

    it('should return true', () => {
      const notLoading: boolean = isNotLoading(createStateResource(createDummyResource()));

      expect(notLoading).toBeTruthy();
    });

    it('should return false', () => {
      const notLoading: boolean = isNotLoading(createEmptyStateResource(true));

      expect(notLoading).toBeFalsy();
    });
  });
});
