/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { Resource } from '@ngxp/rest';
import { cold } from 'jest-marbles';
import { createInvalidParam, createProblemDetail } from 'libs/tech-shared/test/error';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { Observable, of } from 'rxjs';
import {
  StateResource,
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
} from '../resource/resource.util';
import { InvalidParam, ProblemDetail } from '../tech.model';
import { AbstractFormService } from './formservice.abstract';

import * as ValidationUtil from '../validation/tech.validation.util';

describe('AbstractFormService', () => {
  let formService: AbstractFormService<Resource>;

  beforeEach(() => {
    formService = new TestFormService(new UntypedFormBuilder());
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  describe('submit', () => {
    describe('with ProblemDetail', () => {
      const stateResourceWithError: StateResource<Resource> = createErrorStateResource(createProblemDetail());

      beforeEach(() => {
        TestFormService.SUBMIT_OBSERVABLE = () => of(stateResourceWithError);
        formService.handleResponse = jest.fn((stateResource) => stateResource);
      });

      it('should call handle response for ProblemDetail', (done) => {
        formService.submit().subscribe(() => {
          expect(formService.handleResponse).toHaveBeenCalledWith(stateResourceWithError);
          done();
        });
      });

      it('should return state resource observable', () => {
        const submitObservable: Observable<StateResource<Resource>> = formService.submit();

        expect(submitObservable).toBeObservable(cold('(a|)', { a: stateResourceWithError }));
      });
    });

    it('should call after submit operator function', (done) => {
      const commandStateResource: StateResource<Resource> = createStateResource(createDummyResource());
      const afterSubmit = () => of(commandStateResource);
      formService.handleResponse = jest.fn();

      formService.submit(afterSubmit).subscribe(() => {
        expect(formService.handleResponse).toHaveBeenCalledWith(commandStateResource);
        done();
      });
    });

    it('should pass through as default after submit', (done) => {
      const commandStateResource: StateResource<Resource> = createStateResource(createDummyResource());
      TestFormService.SUBMIT_OBSERVABLE = () => of(commandStateResource);
      formService.handleResponse = jest.fn();

      formService.submit().subscribe(() => {
        expect(formService.handleResponse).toHaveBeenCalledWith(commandStateResource);
        done();
      });
    });
  });

  describe('handleResponse', () => {
    const problemDetail: ProblemDetail = createProblemDetail();
    const stateResource: StateResource<Resource> = createErrorStateResource(problemDetail);

    beforeEach(() => {
      formService.handleError = jest.fn();
    });

    it('should handleError on validation error', () => {
      formService.handleResponse({ ...stateResource, loading: false });

      expect(formService.handleError).toHaveBeenCalledWith(problemDetail);
    });

    it('should return stateresource while loading', () => {
      const emptyStateResource: StateResource<Resource> = createEmptyStateResource(true);

      const result: StateResource<Resource> = formService.handleResponse(emptyStateResource);

      expect(result).toBe(emptyStateResource);
    });
  });

  describe('handle error', () => {
    it('should set problem detail error', () => {
      formService.setErrorByProblemDetail = jest.fn();
      const problemDetail: ProblemDetail = createProblemDetail();

      formService.handleError(problemDetail);

      expect(formService.setErrorByProblemDetail).toHaveBeenCalledWith(problemDetail);
    });
  });

  describe('set error by problem detail', () => {
    const invalidParam: InvalidParam = createInvalidParam();
    const problemDetail: ProblemDetail = createProblemDetail([invalidParam]);

    it('should call setInvalidParamValidationError', () => {
      const setInvalidParamValidationErrorSpy: jest.SpyInstance<void> = jest
        .spyOn(ValidationUtil, 'setInvalidParamValidationError')
        .mockImplementation();

      formService.setErrorByProblemDetail(problemDetail);

      expect(setInvalidParamValidationErrorSpy).toHaveBeenCalledWith(formService.form, invalidParam, TestFormService.PATH_PREFIX);
    });
  });

  describe('patch', () => {
    it('should set form value', () => {
      const formValue: { [key: string]: string } = { [TestFormService.FIELD]: 'huhu' };

      formService.patch(formValue);

      expect(formService.form.value).toEqual(formValue);
    });
  });

  describe('reset', () => {
    it('should reset form', () => {
      formService.form.reset = jest.fn();

      formService.reset();

      expect(formService.form.reset).toHaveBeenCalled();
    });
  });

  describe('isInvalid', () => {
    it('should return false on valid state', () => {
      const isInvalid: boolean = formService.isInvalid();

      expect(isInvalid).toBeFalsy();
    });

    it('should return true on invalid state', () => {
      formService.form.controls[TestFormService.FIELD].setErrors({ required: true });

      const isInvalid: boolean = formService.isInvalid();

      expect(isInvalid).toBeTruthy();
    });
  });
});

class TestFormService extends AbstractFormService<Resource> {
  public static readonly FIELD: string = 'attribute';
  public static readonly PATH_PREFIX: string = 'path-prefix';

  public static SUBMIT_OBSERVABLE = () => of(createEmptyStateResource<Resource>());

  constructor(formBuilder: UntypedFormBuilder) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [TestFormService.FIELD]: new UntypedFormControl(null),
    });
  }

  protected doSubmit(): Observable<StateResource<Resource>> {
    return TestFormService.SUBMIT_OBSERVABLE();
  }

  protected getPathPrefix(): string {
    return TestFormService.PATH_PREFIX;
  }
}
