/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HasLinkPipe } from '@alfa-client/tech-shared';
import { getElementFromDomRoot, getElementFromFixture } from '@alfa-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { ChangeDetectorRef, SimpleChanges } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import { DropdownMenuComponent, DropdownMenuTextItemComponent, FileIconComponent, IconComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangArchiveContainerComponent } from './vorgang-archive-container/vorgang-archive-container.component';
import { VorgangDetailMoreMenuComponent } from './vorgang-detail-more-menu.component';
import { VorgangExportContainerComponent } from './vorgang-export-container/vorgang-export-container.component';
import { ProcessVorgangContainerComponent } from './vorgang-process-vorgang-container/vorgang-process-vorgang-container.component';

describe('VorgangDetailMoreMenuComponent', () => {
  let component: VorgangDetailMoreMenuComponent;
  let fixture: ComponentFixture<VorgangDetailMoreMenuComponent>;

  const moreMenuIcon: string = getDataTestIdOf('more-menu-icon');
  const archiveMenuItem: string = getDataTestIdOf('vorgang-archive-button');
  const exportMenuItem: string = getDataTestIdOf('vorgang-exportieren-button');
  const processVorgangMenuItem: string = getDataTestIdOf('vorgang-process-vorgang-button');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailMoreMenuComponent,
        MockComponent(VorgangExportContainerComponent),
        MockComponent(ProcessVorgangContainerComponent),
        MockComponent(VorgangArchiveContainerComponent),
        MockComponent(DropdownMenuComponent),
        MockComponent(DropdownMenuTextItemComponent),
        MockComponent(IconComponent),
        MockComponent(FileIconComponent),
        HasLinkPipe,
      ],
      imports: [NoopAnimationsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailMoreMenuComponent);
    component = fixture.componentInstance;
    component.vorgangWithEingang = createVorgangWithEingangResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('ngOnChanges', () => {
      it('should call vorgangWithEingangChanged if vorgangWithEingang changed', () => {
        const simpleChanges: SimpleChanges = <any>{ vorgangWithEingang: {} };
        const spy = jest.spyOn(component, 'vorgangWithEingangChanged');

        component.ngOnChanges(simpleChanges);

        expect(spy).toHaveBeenCalled();
      });

      it('should not call vorgangWithEingangChanged if vorgangWithEingang did not change', () => {
        const simpleChanges: SimpleChanges = <any>{ dummy: {} };
        const spy = jest.spyOn(component, 'vorgangWithEingangChanged');

        component.ngOnChanges(simpleChanges);

        expect(spy).not.toHaveBeenCalled();
      });
    });

    describe('vorgangWithEingangChanged', () => {
      it('should call detectChanges', () => {
        const changeDetectorRef = fixture.debugElement.injector.get(ChangeDetectorRef);
        const detectChangesSpy = jest.spyOn(changeDetectorRef.constructor.prototype, 'detectChanges');

        component.vorgangWithEingangChanged();

        expect(detectChangesSpy).toHaveBeenCalled();
      });

      it('should call isButtonDisabled', () => {
        const spy = jest.spyOn(component, 'hasMenuItems');

        component.vorgangWithEingangChanged();

        expect(spy).toHaveBeenCalled();
      });
    });

    describe('hasMenuItems', () => {
      it('should return true if at least 1 menu item exists', () => {
        component.menuItems.reset([new DropdownMenuTextItemComponent()]);

        const res: boolean = component.hasMenuItems();

        expect(res).toBeTruthy();
      });

      it('should return false if no menu item exists', () => {
        component.menuItems.reset([]);

        const res: boolean = component.hasMenuItems();

        expect(res).toBeFalsy();
      });
    });

    describe('closeMenu', () => {
      it('should call menu.closeMenu', () => {
        component.menu.closeMenu = jest.fn();

        component.closeMenu();

        expect(component.menu.closeMenu).toHaveBeenCalled();
      });
    });
  });

  describe('template', () => {
    describe('more menu icon', () => {
      it('should be visible', () => {
        component.buttonEnabled = true;

        fixture.detectChanges();

        const icon = getElementFromFixture(fixture, moreMenuIcon);
        expect(icon).toBeInTheDocument();
      });

      it('should NOT be visible', () => {
        component.buttonEnabled = false;

        fixture.detectChanges();

        const icon = getElementFromFixture(fixture, moreMenuIcon);
        expect(icon).not.toBeInTheDocument();
      });
    });

    describe('archive menu item', () => {
      const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.ARCHIVE,
      ]);

      beforeEach(() => {
        component.buttonEnabled = true;
        component.vorgangWithEingang = vorgangWithEingang;
      });

      it('should be visible', () => {
        component.vorgangWithEingang = vorgangWithEingang;
        fixture.detectChanges();

        const element = getElementFromFixture(fixture, archiveMenuItem);

        expect(element).toBeInTheDocument();
      });

      it('should NOT be visible', () => {
        component.vorgangWithEingang = createVorgangWithEingangResource();

        fixture.detectChanges();

        const element = getElementFromFixture(fixture, archiveMenuItem);
        expect(element).not.toBeInTheDocument();
      });
    });

    describe('export menu item', () => {
      const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.EXPORT]);

      beforeEach(() => {
        component.buttonEnabled = true;
      });

      it('should be visible', () => {
        component.vorgangWithEingang = vorgangWithEingang;
        fixture.detectChanges();

        const element = getElementFromDomRoot(fixture, exportMenuItem);

        expect(element).toBeInTheDocument();
      });

      it('should NOT be visible', () => {
        component.vorgangWithEingang = createVorgangWithEingangResource();

        fixture.detectChanges();

        const element = getElementFromDomRoot(fixture, exportMenuItem);
        expect(element).not.toBeInTheDocument();
      });
    });

    describe('processVorgang menu item', () => {
      const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.PROCESS_VORGANG,
      ]);

      beforeEach(() => {
        component.buttonEnabled = true;
      });

      it('should be visible if link exists', () => {
        component.vorgangWithEingang = vorgangWithEingang;

        fixture.detectChanges();

        const element = getElementFromDomRoot(fixture, processVorgangMenuItem);
        expect(element).toBeInTheDocument();
      });

      it('should NOT be visible', () => {
        component.vorgangWithEingang = createVorgangWithEingangResource();

        const element = getElementFromDomRoot(fixture, processVorgangMenuItem);
        expect(element).not.toBeInTheDocument();
      });
    });
  });
});
