/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  EnumToLabelPipe,
  formatFullDateWithTimeWithoutSeconds,
  formatToPrettyDate,
  isNotNull,
  StateResource,
} from '@alfa-client/tech-shared';
import {
  getUserName,
  UserProfileResource,
  UserProfileService,
} from '@alfa-client/user-profile-shared';
import {
  VorgangHeaderLinkRel,
  VorgangResource,
  VorgangStatusLabel,
} from '@alfa-client/vorgang-shared';
import { getAktenzeichenText } from '@alfa-client/vorgang-shared-ui';
import { Component, Input, OnInit } from '@angular/core';
import { hasLink } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { first } from 'rxjs';

@Component({
  selector: 'alfa-vorgang-list-item',
  templateUrl: './vorgang-list-item.component.html',
})
export class VorgangListItemComponent implements OnInit {
  @Input() vorgang: VorgangResource;

  readonly vorgangLinkRel = VorgangHeaderLinkRel;

  public ariaLabel: string = '';

  constructor(public userProfileService: UserProfileService) {}

  ngOnInit(): void {
    if (hasLink(this.vorgang, VorgangHeaderLinkRel.ASSIGNED_TO)) {
      this.userProfileService
        .getAssignedUserProfile(this.vorgang, VorgangHeaderLinkRel.ASSIGNED_TO)
        .pipe(
          first((userProfile: StateResource<UserProfileResource>) =>
            isNotNull(userProfile.resource),
          ),
        )
        .subscribe((userProfileStateResource: StateResource<UserProfileResource>) => {
          this.buildAriaLabel(userProfileStateResource.resource);
        });
    } else this.buildAriaLabel();
  }

  buildAriaLabel(userProfileResource: UserProfileResource = undefined) {
    const name: string = this.vorgang.name;
    const aktenzeichen: string = getAktenzeichenText(this.vorgang);
    const nummer: string = this.vorgang.nummer;
    const status: string = this.getStatus();
    const approvalStatus: string = this.getApprovalText();
    const createdAt: string = formatFullDateWithTimeWithoutSeconds(this.vorgang.createdAt);
    const userText: string = this.getUserText(userProfileResource);

    this.ariaLabel = `Vorgang: ${name}, Aktenzeichen: ${aktenzeichen}, Nummer: ${nummer} Status: ${status} ${approvalStatus}, Eingang: ${createdAt}, ${userText}`;

    if (
      hasLink(this.vorgang, VorgangHeaderLinkRel.WIEDERVORLAGEN) &&
      this.vorgang.nextFrist !== null
    ) {
      this.ariaLabel += this.getWiedervorlageText();
    }

    if (this.vorgang.hasPostfachNachricht) {
      this.ariaLabel += this.getPostfachNachricht();
    }
  }

  getWiedervorlageText(): string {
    const nextFrist: string = formatToPrettyDate(this.vorgang.nextFrist);
    return `, Nächste Wiedervorlage am ${nextFrist}`;
  }

  getPostfachNachricht(): string {
    if (this.vorgang.hasNewPostfachNachricht) {
      return ', enthält neue Postfachnachrichten';
    }

    return ', enthält Postfachnachrichten';
  }

  getStatus(): string {
    const enumToLabel = new EnumToLabelPipe();
    return enumToLabel.transform(this.vorgang.status, VorgangStatusLabel);
  }

  getApprovalText(): string {
    if (isNil(this.vorgang.antragBewilligt)) return '';

    return this.vorgang.antragBewilligt ? 'bewilligt' : 'abgelehnt';
  }

  getUserText(userProfileResource: UserProfileResource): string {
    return userProfileResource ?
        `Aktuell zugewiesener Nutzer: ${getUserName(userProfileResource)}`
      : 'Kein Bearbeiter zugewiesen';
  }
}
