/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileService } from '@alfa-client/binary-file-shared';
import { CommandOrder, CommandResource, CommandService, CreateCommand, isDone } from '@alfa-client/command-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import {
  createEmptyStateResource,
  createStateResource,
  decodeUrlFromEmbedding,
  doIfLoadingRequired,
  hasStateResourceError,
  isLoaded,
  isNotNull,
  isNotUndefined,
  replacePlaceholder,
  StateResource,
} from '@alfa-client/tech-shared';
import { SnackBarService } from '@alfa-client/ui';
import { VorgangHeaderLinkRel, VorgangResource, VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Injectable, OnDestroy } from '@angular/core';
import { Params } from '@angular/router';
import { hasLink, ResourceUri } from '@ngxp/rest';
import { isEqual, isNil, isNull, isUndefined } from 'lodash-es';
import { BehaviorSubject, combineLatest, Observable, Subscription } from 'rxjs';
import { filter, map, startWith, tap } from 'rxjs/operators';
import { WiedervorlageLinkRel, WiedervorlageListLinkRel } from './wiedervorlage.linkrel';
import { WiedervorlageMessages } from './wiedervorlage.message';
import {
  Wiedervorlage,
  WIEDERVORLAGE_UPLOADED_ATTACHMENTS,
  WiedervorlageListResource,
  WiedervorlageResource,
} from './wiedervorlage.model';
import { WiedervorlageRepository } from './wiedervorlage.repository';
import { WiedervorlageRoutes } from './wiedervorlage.route';
import { createErledigenCommand, createWiedereroeffnenCommand } from './wiedervorlage.util';

@Injectable({ providedIn: 'root' })
export class WiedervorlageService implements OnDestroy {
  static encodedWiedervorlageUriParam: string = 'wiedervorlageUrl';

  readonly wiedervorlageList$: BehaviorSubject<StateResource<WiedervorlageListResource>> = new BehaviorSubject<
    StateResource<WiedervorlageListResource>
  >(createEmptyStateResource<WiedervorlageListResource>());
  readonly wiedervorlage$: BehaviorSubject<StateResource<WiedervorlageResource>> = new BehaviorSubject<
    StateResource<WiedervorlageResource>
  >(createEmptyStateResource<WiedervorlageResource>());

  readonly submitInProgress$: BehaviorSubject<StateResource<CommandResource>> = new BehaviorSubject<
    StateResource<CommandResource>
  >(createEmptyStateResource<CommandResource>());

  readonly erledigenCommand$: BehaviorSubject<StateResource<CommandResource>> = new BehaviorSubject<
    StateResource<CommandResource>
  >(createEmptyStateResource<CommandResource>());
  readonly wiedereroeffnenCommand$: BehaviorSubject<StateResource<CommandResource>> = new BehaviorSubject<
    StateResource<CommandResource>
  >(createEmptyStateResource<CommandResource>());

  shouldReload: boolean = false;

  private subscription: Subscription;
  vorgangSubscription: Subscription;

  constructor(
    private repository: WiedervorlageRepository,
    private navigationService: NavigationService,
    private commandService: CommandService,
    private snackbarService: SnackBarService,
    private binaryFileService: BinaryFileService,
    private vorgangService: VorgangService,
  ) {
    this.listenToNavigation();
    this._listenToVorgangChange();
  }

  public getWiedervorlageList(): Observable<StateResource<WiedervorlageListResource>> {
    return combineLatest([this.wiedervorlageList$.asObservable(), this.vorgangService.getVorgangWithEingang()]).pipe(
      tap(([wiedervorlageList, vorgangResource]) => {
        if (isNotNull(vorgangResource.resource)) {
          doIfLoadingRequired(wiedervorlageList, () => this._loadWiedervorlagenByVorgang(vorgangResource.resource));
        }
      }),
      map(([wiedervorlageList]) => wiedervorlageList),
      startWith(createEmptyStateResource<WiedervorlageListResource>(true)),
    );
  }

  public getWiedervorlageListByGivenVorgang(
    vorgangResource: VorgangResource,
  ): Observable<StateResource<WiedervorlageListResource>> {
    this._refreshVorgangSubscription();
    doIfLoadingRequired(this.wiedervorlageList$.value, () => this._loadWiedervorlagenByVorgang(vorgangResource));
    return this.wiedervorlageList$.asObservable();
  }

  _refreshVorgangSubscription(): void {
    this.vorgangSubscription.unsubscribe();
    this._listenToVorgangChange();
  }

  _listenToVorgangChange(): void {
    this.vorgangSubscription = this.vorgangService
      .selectVorgangWithEingang()
      .subscribe((vorgangWithEingangStateResource: StateResource<VorgangWithEingangResource>) =>
        this._handleVorgangChange(vorgangWithEingangStateResource),
      );
  }

  _handleVorgangChange(vorgangWithEingangStateResource: StateResource<VorgangWithEingangResource>): void {
    if (this.shouldReloadList(vorgangWithEingangStateResource)) {
      this.shouldReload = true;
    }
    if (isLoaded(vorgangWithEingangStateResource) && this.shouldReload) {
      this._setWiedervorlageListReload();
      this.shouldReload = false;
    }
  }

  private shouldReloadList(vorgangWithEingangStateResource: StateResource<VorgangWithEingangResource>): boolean {
    return (
      (vorgangWithEingangStateResource.loading || vorgangWithEingangStateResource.reload) &&
      isNotNull(this.wiedervorlageList$.value.resource)
    );
  }

  _loadWiedervorlagenByVorgang(vorgangResource: VorgangResource): void {
    if (hasLink(vorgangResource, VorgangHeaderLinkRel.WIEDERVORLAGEN)) {
      const subscription: Subscription = this.repository.getWiedervorlageList(vorgangResource).subscribe((wiedervorlagenList) => {
        if (!isNull(wiedervorlagenList)) {
          this._setWiedervorlagenList(wiedervorlagenList);
          subscription.unsubscribe();
        }
      });
    } else {
      this._clearWiedervorlagenList();
    }
  }

  public loadWiedervorlagenByVorgangStrict(
    vorgangResource: VorgangResource,
  ): Observable<StateResource<WiedervorlageListResource>> {
    return this.repository.getWiedervorlageList(vorgangResource).pipe(
      map((wiedervorlagenList) => createStateResource(wiedervorlagenList)),
      startWith(createEmptyStateResource<WiedervorlageListResource>(true)),
    );
  }

  _setWiedervorlagenList(wiedervorlagenList: WiedervorlageListResource): void {
    this.wiedervorlageList$.next(createStateResource(wiedervorlagenList));
  }

  public getWiedervorlage(): Observable<StateResource<WiedervorlageResource>> {
    return this.wiedervorlage$.asObservable().pipe(
      tap((wiedervorlage) => doIfLoadingRequired(wiedervorlage, () => this.updateWiedervorlage())),
      startWith(createEmptyStateResource<WiedervorlageResource>(true)),
    );
  }

  private updateWiedervorlage(): void {
    this._setWiedervorlageLoading();

    this.loadAndSetWiedervorlageByUrl(this.getWiedervorlageUri());
  }

  private loadAndSetWiedervorlageByUrl(url: ResourceUri): void {
    this._setWiedervorlageLoading();

    const subscription: Subscription = this.repository.getWiedervorlage(url).subscribe((wiedervorlage) => {
      this.wiedervorlage$.next(createStateResource(wiedervorlage));
      subscription.unsubscribe();
    });
  }

  public isNewWiedervorlage(): boolean {
    return isEqual(this.navigationService.getParam(WiedervorlageService.encodedWiedervorlageUriParam), WiedervorlageRoutes.NEW);
  }

  private listenToNavigation(): void {
    this.unsubscribe();
    this.subscription = this.navigationService.urlChanged().subscribe((params) => this._onNavigation(params));
  }

  _onNavigation(params: Params): void {
    this.clearUploadedFiles();
    if (this.navigateToVorgangDetailPage(params)) {
      this._setWiedervorlageListReload();
      this.forceWiedervorlageReload();
      this.submitInProgress$.next(createEmptyStateResource<CommandResource>());
    }
    if (NavigationService.isVorgangListPage(params)) {
      this._clearWiedervorlagenList();
      this.submitInProgress$.next(createEmptyStateResource<CommandResource>());
    }
  }

  _clearWiedervorlagenList(): void {
    this.wiedervorlageList$.next(createEmptyStateResource<WiedervorlageListResource>());
  }

  private forceWiedervorlageReload(): void {
    this.wiedervorlage$.next(createEmptyStateResource());
  }

  private navigateToVorgangDetailPage(params: Params): boolean {
    return isUndefined(
      params[WiedervorlageService.encodedWiedervorlageUriParam] &&
        isNotUndefined(params[VorgangService.VORGANG_WITH_EINGANG_URL]),
    );
  }

  private getWiedervorlageUri(): ResourceUri {
    return this.navigationService.getDecodedParam(WiedervorlageService.encodedWiedervorlageUriParam);
  }

  _setWiedervorlageLoading(): void {
    this.wiedervorlage$.next({ ...this.wiedervorlage$.value, loading: true });
  }

  public createWiedervorlage(wiedervorlage: Wiedervorlage): Observable<StateResource<CommandResource>> {
    this.setSubmitInProgressLoading();

    return this._createCreateWiedervorlageCommand(this.wiedervorlageList$.value.resource, wiedervorlage).pipe(
      filter((commandStateResource) => !commandStateResource.loading),
      tap((commandStateResource) =>
        this._proceedAfterReceiveCommand(commandStateResource, this.buildMessage(WiedervorlageMessages.CREATED, wiedervorlage)),
      ),
      startWith(createEmptyStateResource<CommandResource>(true)),
    );
  }

  _createCreateWiedervorlageCommand(
    wiedervorlageListResource: WiedervorlageListResource,
    wiedervorlage: Wiedervorlage,
  ): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommand(
      wiedervorlageListResource,
      WiedervorlageListLinkRel.CREATE_WIEDERVORLAGE,
      this.buildCreateWiedervorlageCommand(wiedervorlage),
    );
  }

  private buildCreateWiedervorlageCommand(wiedervorlage: Wiedervorlage): CreateCommand {
    return { order: CommandOrder.CREATE_WIEDERVORLAGE, body: wiedervorlage };
  }

  public saveWiedervorlage(
    wiedervorlageResource: WiedervorlageResource,
    wiedervorlage: Wiedervorlage,
  ): Observable<StateResource<CommandResource>> {
    this.setSubmitInProgressLoading();

    return this._createSaveWiedervorlageCommand(wiedervorlageResource, wiedervorlage).pipe(
      filter((commandStateResource) => !commandStateResource.loading),
      tap((commandStateResource) =>
        this._proceedAfterReceiveCommand(commandStateResource, this.buildMessage(WiedervorlageMessages.SAVED, wiedervorlage)),
      ),
      startWith(createEmptyStateResource<CommandResource>(true)),
    );
  }

  private setSubmitInProgressLoading(): void {
    this.submitInProgress$.next({ ...this.submitInProgress$.value, loading: true });
  }

  _createSaveWiedervorlageCommand(
    wiedervorlageResource: WiedervorlageResource,
    wiedervorlage: Wiedervorlage,
  ): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommand(
      wiedervorlageResource,
      WiedervorlageLinkRel.EDIT,
      this.buildEditWiedervorlageCommand(wiedervorlage),
    );
  }

  private buildEditWiedervorlageCommand(wiedervorlage: Wiedervorlage): CreateCommand {
    return { order: CommandOrder.EDIT_WIEDERVORLAGE, body: wiedervorlage };
  }

  _proceedAfterReceiveCommand(commandStateResource: StateResource<CommandResource>, message: string): void {
    this.submitInProgress$.next({
      ...this.submitInProgress$.value,
      resource: commandStateResource.resource,
    });
    if (isDone(commandStateResource.resource)) {
      this.submitInProgress$.next(createStateResource(commandStateResource.resource));
      this.snackbarService.show(commandStateResource.resource, message);
      this._setWiedervorlageListReload();
      this.clearUploadedFiles();
    } else if (hasStateResourceError(commandStateResource)) {
      this.submitInProgress$.next(createStateResource(commandStateResource.resource));
    }
  }

  _setWiedervorlageListReload(): void {
    this.wiedervorlageList$.next({ ...this.wiedervorlageList$.value, reload: true });
  }

  public erledigen(): Observable<StateResource<CommandResource>> {
    return this.doAction(
      this.erledigenCommand$,
      WiedervorlageLinkRel.ERLEDIGEN,
      createErledigenCommand,
      WiedervorlageMessages.ERLEDIGT,
    );
  }

  public wiedereroeffnen(): Observable<StateResource<CommandResource>> {
    return this.doAction(
      this.wiedereroeffnenCommand$,
      WiedervorlageLinkRel.WIEDEREROEFFNEN,
      createWiedereroeffnenCommand,
      WiedervorlageMessages.WIEDEREROEFFNET,
    );
  }

  private doAction(
    commandStateSubj: BehaviorSubject<StateResource<CommandResource>>,
    linkRel: string,
    commandProvider: () => CreateCommand,
    snackBarMessage: string,
  ): Observable<StateResource<any>> {
    commandStateSubj.next(createEmptyStateResource(true));

    const commandSubscription: Subscription = this.commandService
      .createCommand(this.wiedervorlage$.value.resource, linkRel, commandProvider())
      .subscribe((commandStateResource) => {
        if (isDone(commandStateResource.resource)) {
          this.handleSource(commandStateResource.resource);
          this._showSnackBar(
            commandStateResource.resource,
            this.buildMessage(snackBarMessage, this.wiedervorlage$.value.resource),
          );
          commandStateSubj.next(commandStateResource);
          this.clearUploadedFiles();
          commandSubscription.unsubscribe();
        }
      });

    return commandStateSubj.asObservable();
  }

  private handleSource(commandResource: CommandResource): void {
    if (isDone(commandResource)) {
      this.loadAndSetWiedervorlageByUrl(this.getWiedervorlageUri());
    }
  }

  private buildMessage(message: string, wiedervorlage: Wiedervorlage) {
    return replacePlaceholder(message, 'betreff', wiedervorlage.betreff);
  }

  _showSnackBar(commandResource: CommandResource, message: string) {
    if (isDone(commandResource)) {
      this.snackbarService.show(commandResource, message);
    }
  }

  public getSubmitInProgress(): Observable<StateResource<CommandResource>> {
    return this.submitInProgress$.asObservable();
  }

  ngOnDestroy(): void {
    this.unsubscribe();
  }

  private unsubscribe(): void {
    if (!isNil(this.subscription)) this.subscription.unsubscribe();
  }

  public hasEditLink(wiedervorlageUrl: string): Observable<boolean> {
    const decodedUrl: string = decodeUrlFromEmbedding(wiedervorlageUrl);

    return this.repository
      .getWiedervorlage(decodedUrl)
      .pipe(map((wiedervorlage) => hasLink(wiedervorlage, WiedervorlageLinkRel.EDIT)));
  }

  public clearUploadedFiles(): void {
    this.binaryFileService.clearUploadedFiles(WIEDERVORLAGE_UPLOADED_ATTACHMENTS);
  }
}
