import { ComponentFixture, TestBed } from '@angular/core/testing';

import { EventData, getMockComponent, mock, Mock, triggerEvent } from '@alfa-client/test-utils';
import {
  OrganisationsEinheitResource,
  OrganisationsEinheitService,
  ZUSTAENDIGE_STELLE_SERVICE,
} from '@alfa-client/zustaendige-stelle-shared';
import { faker } from '@faker-js/faker';
import { Resource } from '@ngxp/rest';
import { InstantSearchResult } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createInstantSearchResult } from '../../../../../design-system/src/test/search';
import { getDataTestIdOf } from '../../../../../tech-shared/test/data-test';
import { createOrganisationsEinheitResource } from '../../../../../zustaendige-stelle-shared/test/organisations-einheit';
import { SearchZustaendigeStelleFormContainerComponent } from './search-zustaendige-stelle-form-container.component';
import { SearchZustaendigeStelleFormComponent } from './search-zustaendige-stelle-form/search-zustaendige-stelle-form.component';

describe('SearchZustaendigeStelleFormContainerComponent', () => {
  let component: SearchZustaendigeStelleFormContainerComponent<Resource>;
  let fixture: ComponentFixture<SearchZustaendigeStelleFormContainerComponent<Resource>>;

  let service: Mock<OrganisationsEinheitService>;

  const searchOrganisationsEinheitComp: string = getDataTestIdOf('search-organisations-einheit');

  const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();

  beforeEach(async () => {
    service = mock(OrganisationsEinheitService);
    await TestBed.configureTestingModule({
      imports: [SearchZustaendigeStelleFormContainerComponent],
      declarations: [MockComponent(SearchZustaendigeStelleFormComponent)],
      providers: [
        {
          provide: ZUSTAENDIGE_STELLE_SERVICE,
          useValue: service,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(SearchZustaendigeStelleFormContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service', () => {
      component.ngOnInit();

      expect(service.getSearchResultList).toHaveBeenCalled();
    });
  });

  describe('onKeyDownHandler', () => {
    it('should prevent default behavior for enter key on non buttons', () => {
      const keyboardEvent: KeyboardEvent = {
        ...new KeyboardEvent('enter'),
        key: 'Enter',
        preventDefault: jest.fn(),
        target: new EventTarget(),
      };

      component.onKeyDownHandler(keyboardEvent);

      expect(keyboardEvent.preventDefault).toHaveBeenCalled();
    });

    it('should not prevent default behavior for enter key on buttons', () => {
      const keyboardEvent: KeyboardEvent = {
        ...new KeyboardEvent('enter'),
        key: 'Enter',
        preventDefault: jest.fn(),
        target: { ...new EventTarget(), tagName: 'BUTTON' } as Element,
      };

      component.onKeyDownHandler(keyboardEvent);

      expect(keyboardEvent.preventDefault).not.toHaveBeenCalled();
    });
  });

  describe('search organisationsEinheit component', () => {
    const result: InstantSearchResult<Resource> = createInstantSearchResult();

    beforeEach(() => {
      component.searchResults$ = of([result]);
      fixture.detectChanges();
    });

    it('should be called with searchResult', () => {
      const comp: SearchZustaendigeStelleFormComponent<Resource> = getMockComponent<
        SearchZustaendigeStelleFormComponent<Resource>
      >(fixture, SearchZustaendigeStelleFormComponent);

      expect(comp.searchResults).toEqual([result]);
    });

    it('should call search on openSearchDialog output', () => {
      component.search = jest.fn();
      const searchBy: string = faker.word.sample();
      const eventData: EventData<SearchZustaendigeStelleFormContainerComponent<Resource>> = {
        fixture,
        elementSelector: searchOrganisationsEinheitComp,
        name: 'search',
        data: searchBy,
      };

      triggerEvent(eventData);

      expect(component.search).toHaveBeenCalledWith(searchBy);
    });

    it('should call selectSearchResult on selectSearchResult output', () => {
      component.selectSearchResult = jest.fn();

      const eventData: EventData<SearchZustaendigeStelleFormContainerComponent<Resource>> = {
        fixture,
        elementSelector: searchOrganisationsEinheitComp,
        name: 'selectSearchResult',
        data: organisationsEinheitResource,
      };

      triggerEvent(eventData);

      expect(component.selectSearchResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should call clearSearchResult', () => {
      component.clearSearchResult = jest.fn();

      const eventData: EventData<SearchZustaendigeStelleFormContainerComponent<Resource>> = {
        fixture,
        elementSelector: searchOrganisationsEinheitComp,
        name: 'clearSearchResult',
        data: organisationsEinheitResource,
      };

      triggerEvent(eventData);

      expect(component.clearSearchResult).toHaveBeenCalled();
    });
  });

  describe('search', () => {
    const searchBy: string = faker.word.sample();

    it('should call service', () => {
      component.search(searchBy);

      expect(service.search).toHaveBeenCalledWith(searchBy);
    });
  });

  describe('select search result', () => {
    it('should set select result', () => {
      component.selectSearchResult(organisationsEinheitResource);

      expect(service.selectSearchResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should call service', () => {
      component.selectSearchResult(organisationsEinheitResource);

      expect(service.clearSearchResult).toHaveBeenCalled();
    });

    it('should emit search result selected', () => {
      component.searchResultSelected.emit = jest.fn();

      component.selectSearchResult(organisationsEinheitResource);

      expect(component.searchResultSelected.emit).toHaveBeenCalledWith(organisationsEinheitResource);
    });
  });

  describe('clear search result', () => {
    it('should call service', () => {
      component.clearSearchResult();

      expect(service.clearSearchResult).toHaveBeenCalled();
    });
  });

  describe('isNotButtonElement', () => {
    it('should return false', () => {
      const element: Element = { tagName: 'BUTTON' } as Element;

      const result: boolean = component.isNotButtonElement(element);

      expect(result).toBe(false);
    });

    it('should return true', () => {
      const element: Element = { tagName: 'INPUT' } as Element;

      const result: boolean = component.isNotButtonElement(element);

      expect(result).toBe(true);
    });
  });
});
